<?php
	/** Datei, die die Definition der Klasse AddArticlePage enthält. */

	/** 
	* Klasse, die die Klasse Page um die Funktion zur Aufnahme eines neuen Artikels enthält 
	*/
	class AddArticlePage extends Page{
		/**
		* Gibt ein Formular für das Hinzufügen eines neuen Articles zu einer Liste zurück, wobei Artikelbezeichnung, Artikelanmerkung und Kategorie ausgewählt werden können.
		* Wenn bereits eine Kategorie angegeben ist, wird die Kategorie nicht mehr zur Auswahl gestellt. Die Angabe ist Voraussetzung für die Eingabefelder zum Intervall sowie der Anzahl der Tage, bevor ein Artikel automatisch auf die Liste gesetzt bzw. als dringend markiert wird. Im Gegensatz zu der Funktion editConvertToArticle sortiert diese Funktion die Artikel nach Artikelname und Artikelanmerkung, nicht nach Artikelnamen und verbundenen Listeneinträgen.
		* @param string $title Der Titel, der über das Formular gesetzt wird.
		* @param object $article Der Artikel, der hinzugefügt werden soll.
		* @param object $list Die Liste des Artikels.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param string $formular Die Bezeichnung des Formulars; zugleich der Name des Formulars, an das weitergeleitet wird. 
		* @param $listEntry Der Listeneintrag, der hinzugefügt werden soll.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.	
		*/
		public function getDataOfNewArticle($title,$article,$list,$prefix,$formular,$listEntry,$indent) {
			if (!empty($article) AND !empty($article->getConvertToArticleID())){
				$convertToArticle = Article::constructByID($article->getConvertToArticleID());
				$placeholder = ' value="'. $convertToArticle->getName() . '"';
			}
			else {
				if (empty($article)){
					$article = new Article(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
				}
				$placeholder = ' placeholder="Artikelbezeichnung"';
			}
			if (empty($listEntry)){
				$listEntry = new ListEntry(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			}
			if (empty($list)){
				$list = new Listing(NULL,NULL,NULL);
			}
			if (!empty($title)){
				$title = $this->linebreak($indent) . '<h2>' . $title . '</h2>';
			}
			if ($article->getCatID() == ""){ // ohne CatID Auswahlbox der 
				$selectCat = $this->selectCategory($article,$list->getCategories(),$prefix,$indent+1);
				$selectPos = "";
				$selectSpecialAttributes = "";
				$selectStandardSettings = "";
				$selectArticleToConvertInto = "";
			} else { // wenn Kategorie bereits ausgewählt, keine Auswahlbox
				$category = Category::constructByID($article->getCatID());
				$category->addSubCats();
				$selectCat = $this->selectCategory($article,array($category),$prefix,$indent+1);
				$selectPos = $this->selectSubCats($article,$category->getArticles(),$indent);
				$selectSpecialAttributes = $this->selectSpecialAttributes($article,$prefix,$indent);
				$selectStandardSettings = $this->selectStandardSettings($article,$prefix,$indent+1);
				$selectArticleToConvertInto = $this->linebreak($indent+1) . '<fieldset>' .
				$this->linebreak($indent+2) . '<legend>' . 
				$this->linebreak($indent+3) . 'Nach dem Löschen umwandeln in'. 
				$this->linebreak($indent+2) . '</legend>' . 
				$this->linebreak($indent+2) . '<input type="text" name="article_convertToArticle" class="typeaheadConvert tt-query" autocomplete="off" spellcheck="false"' . $placeholder . '>' . 
				$this->linebreak($indent+1) . '</fieldset>';
			}
			$result = $title .
			$this->startFormular("display: inline;","",$formular,"","POST","","",$indent) .
			$this->hiddenFields(array("article" => $this->objectToString($article), "listEntry" => $this->objectToString($listEntry), "listID" => $list->getID(), "formular" => $formular), $indent+1) .
			$this->formularArticleNameAndAnnotation($article,$prefix,$indent+1) .
			$selectCat .
			$selectPos .
			$selectSpecialAttributes .
			$selectStandardSettings . 
			$selectArticleToConvertInto . 
			$this->endFormular($indent) . 		
			$this->submitButtonArea("",$this->submitButton($formular,"submitButton","","","","hinzufügen"),$indent);
			return $result; 
		}
		

		
		
		/**
		* gibt ein Formular zurück, mit dem der Artikel gewählt werden kann, in den ein Listeneintrag nach seiner Löschung aus dem Papierkorb konvertiert werden soll.
		* Im Gegegensatz zu der Funktion addArticleToCat sortiert diese Funktion die Artikel nach Artikelnamen und verbundenen Listeneinträgen, nicht nach Artikelname und Artikelanmerkung.
		* @param $title Der Titel, der über das Formular gesetzt wird.
		* @param $article Das Article-Objekt.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Rückgabetext.
		*/
		public function editConvertToArticle($title,$article,$indent){
			if (!empty($article) AND !empty($article->getConvertToArticleID())){
				$convertToArticle = Article::constructByID($article->getConvertToArticleID());
				$placeholder = ' value="'. $convertToArticle->getName() . '"';
			}
			else {
				$article = new Article(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
				$placeholder = ' placeholder="Artikelbezeichnung"';
			}
			$result = $this->linebreak($indent) . '<h2>' . $title . '</h2>' .
			$this->startFormular("","","convertToArticle","","","","",$indent) . 
			$this->hiddenFields(array("formular" => "convertToArticle"),$indent + 2) .	
			$this->linebreak($indent+1) . '<fieldset>' .
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Bezeichnung'. 
			$this->linebreak($indent+2) . '</legend>' . 
			$this->linebreak($indent+2) . '<input type="text" name="convertToArticle" class="typeahead tt-query" autocomplete="off" spellcheck="false"' . $placeholder . '>' . 
			$this->linebreak($indent+1) . '</fieldset>' . 
			$this->endFormular($indent) .
			$this->submitButtonArea("",$this->submitButton("convertToArticle","submitButton","","","","einfügen"),$indent);
			return $result;
		}
		
				
		/**
		* gibt den Artikel zurück, der mit der Funktion editConvertToArticle() ausgewählt wurde.
		* @param $string Der String, den die Funktion editConvertToArticle() erzeugt hat
		* return object Der Artikel.
		*/
		public function selectArticleToConvertInto($string){
			if (!empty($string)){
				include(ABS_PATH . "dbconnect.php");			
				// zerlege den String in ein Array, dessen erster Teil den Artikelname und der zweite Teil die Liste(n) enthält, denen der Artikel zugeorndet ist
				$array = explode("[",$string); // zerlegen des Strings nach öffnender Klammer
			
				// Bearbeite das entstandene Array
				$articleName = $this->deleteBrackets($array[0]); // Lösche Leerzeichen nach Artikelnamen
				$listenarray = explode(", ", substr($array[1],0,-1)); // schneide die eckige Klammer am Ende der Liste(n) ab und zerlege den String dann an den Kommata in ein Array
			
				// Bearbeite die Listen			
				foreach($listenarray AS $listenname){
					$listenArrayWithBrackets[] = "'" . $listenname . "'"; // füge vor und nach dem Listennamen Hochkommata an
				}
				$listennamen = implode(", ", $listenArrayWithBrackets); // Wandle $listenArrayWithBrackets in einen kommaseparierten String um 
			
			
				// Suche ID des Artikels mit dem entsprechenden Namen und der entsprechenden Listenzuordnung
				$selectStr = '
					SELECT 
						DISTINCT ' . DB_TAB_PREFIX . 'articles.ID
					FROM 
		 		 		' . DB_TAB_PREFIX . 'articles 
					LEFT JOIN 
						' . DB_TAB_PREFIX . 'articlesCategories
							ON 
								' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID
					LEFT JOIN 
						' . DB_TAB_PREFIX . 'categories
							ON 
								' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . DB_TAB_PREFIX . 'categories.ID
					LEFT JOIN 
						' . DB_TAB_PREFIX . 'lists
							ON 
								' . DB_TAB_PREFIX . 'categories.listID = ' . DB_TAB_PREFIX . 'lists.ID 
							WHERE (' . DB_TAB_PREFIX . 'lists.name IN ('.  $listennamen . ') AND ' . DB_TAB_PREFIX . 'articles.name = "' . $articleName . '") ';
				$query=mysqli_query($db, $selectStr);
				$row = mysqli_fetch_object($query);
				if(mysqli_num_rows($query) > 0){ // falls Suche Ergebnis liefert (in Liste eingetragen)
					return Article::constructByID($row->ID);
				}
			}
			return NULL;
		}




		/**
		* Gibt ein Formular für das Hinzufügen eines neuen Articles zu einer Liste zurück, wobei Artikelbezeichnung, Artikelanmerkung und Kategorie ausgewählt werden können.
		* Wenn bereits eine Kategorie angegeben ist, wird die Kategorie nicht mehr zur Auswahl gestellt. Die Angabe ist Voraussetzung für die Eingabefelder zum Intervall sowie der Anzahl der Tage, bevor ein Artikel automatisch auf die Liste gesetzt bzw. als dringend markiert wird. 
		* @param string $title Der Titel, der über das Formular gesetzt wird.
		* @param object $article Der Artikel, der hinzugefügt werden soll.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param string $formular Die Bezeichnung des Formulars; zugleich der Name des Formulars, an das weitergeleitet wird. 
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.	
		*/
		public function editArticle($title,$article,$prefix,$formular,$indent) {
			if (!empty($title)){
				$title = $this->linebreak($indent) . '<h2>' . $title . '</h2>';
			}
			$result = $title .
			$this->startFormular("display: inline;","",$formular,"","POST","","",$indent) .
			$this->hiddenFields(array($this->objectToString($article), "formular" => $formular), $indent+1) .
			$this->formularArticleNameAndAnnotation($article,$prefix,$indent+1) .
			$this->selectStandardSettings($article,$prefix,$indent+1) . 
			$this->selectSpecialAttributes($article,$prefix,$indent+1) .
			$this->endFormular($indent) . 		
			$this->submitButtonArea("",$this->submitButton($formular,"submitButton","","","","ändern"),$indent);
			return $result; 
		}



		/**
		* Gibt ein Formular zurück, dass die Auswahl der Position eines Artikels in einer Kategorie erlaubt.
		* @param string $title Die Überschrift (h2), die über das Formular gesetzt wird.
		* @param object $article Der entsprechende Artikel.
		* @param object $category Die entsprechende Kategorie.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.
		*/
		public function selectPositionInCat($title,$article,$category,$indent){
			return $this->linebreak($indent) . '<h2>' . $title . '</h2>' .
			$this->startFormular("","","addSubCatPos","","","","",$indent) . 
			$this->hiddenFields(
				array(
					"formular" => "addSubCatPos",
					"newCatID" => $category->getID(),
				),
				$indent + 2) .	
			$this->selectSubCats($article,$category->getArticles(),$indent) .
			$this->endFormular($indent) .
			$this->submitButtonArea("",$this->submitButton("addSubCatPos","submitButton","","","","hinzufügen"),$indent);
		} 
		
		/**
		* gibt ein Formular zur Auswahl der Kategorie zurück, der ein neuer Article zugeordnet werden soll.
		* @param object $article Der neue Artikel.
		* @param array $categories Ein Array aus Kategorie-Objekten, der der neue Artikel zugeordnet werden soll.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.
		*/
		private function selectCategory($article,$categories,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			$catID = $article->getCatID();
			$result = $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Kategorie' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . '<select class="fullWidth" name="' . $prefix . 'catID" size="1">';
			
			foreach ($categories AS $category){
				if ($category->getID() == $catID){
					$selected = ' selected = "selected"';
				} else {
					$selected = "";
				}
				$result .= $this->linebreak($indent+2) . '<option value="' . $category->getID() . '"' . $selected . '>' . 
				$this->linebreak($indent+3) . $category->getName() .
				$this->linebreak($indent+2) . '</option>';
				
			}
			$result .= $this->linebreak($indent+1) . '</select>' . 
			$this->linebreak($indent) . '</fieldset>';
			return $result;
		}
		
		/**
		* Liefert ein Formular zur Bearbeitung der automatischen Listeneinträge zurück.
		* @param object $article Das zugehörige Article-Objekt.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/		
		private function selectSpecialAttributes($article,$prefix,$indent){
			$result = $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Automatische Listeneinträge' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . '<fieldset>' . 
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Stichtag des nächsten Kaufs' . 
			$this->linebreak($indent+2) . '</legend>' . 			
			$this->editNextBuy($article->getNextBuy(),$prefix,$indent+2) .
			$this->editInterval($article->getIntervalInDays(),$prefix,$indent+2) .
			$this->editBuyEvery($article->getBuyEvery(),$prefix,$indent+2) . 
			$this->linebreak($indent+1) . '</fieldset>' .

			$this->linebreak($indent+1) . $this->editAddToListXDaysBefore($article->getAddToListXDaysBefore(),$prefix,$indent) .
			$this->linebreak($indent+1) . $this->editSetToUrgentXDaysBefore($article->getSetToUrgentXDaysBefore(),$prefix,$indent) .
			$this->linebreak($indent) . '</fieldset>';
			return $result;
		}
		
		/**
		* Liefert ein Formular mit dem Namen interval, um einen festen Zeitabstand zwischen den Käufen in Tagen zu bearbeiten.
		* @param int $interval Der Zeitabstand in Tagen zwischen den Käufen.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function editInterval($interval,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="interval">' . 
			$this->linebreak($indent+2) . 'Setze Stichtag für den nächsten Kauf <i>x</i> Tage nach letztem Stichtag: ' . 
			$this->linebreak($indent+1) . '</label>' . 
			$this->linebreak($indent+1) . '<input name="' . $prefix . 'intervalInDays" id="interval" type="number" value="' . $interval . '" size="1">' .
			$this->linebreak($indent) . "</p>";
			return $result;
		}
		
		/**
		* Liefert ein Formular mit dem Namen addToListXDaysBefore, um die Anzahl der Tage vor dem Kaufdatum zu definieren, an denen der Artikel erneut auf die Liste gesetzt wird.
		* @param int $days Anzahl der Tage vor dem Kaufdatum, vor dem der Artikel wieder auf die Liste gesetzt wird.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function editAddToListXDaysBefore($days,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="addToListXDaysBefore">' . 
			$this->linebreak($indent+2) . 'Setze den Artikel <i>x</i> Tage vor dem Stichtag des nächsten Kaufs auf die Liste: ' . 
			$this->linebreak($indent+1) . '</label>' . 
			$this->linebreak($indent+1) . '<input name="' . $prefix . 'addToListXDaysBefore" id="addToListXDaysBefore" type="number" value="' . $days . '" size="1">' .
			$this->linebreak($indent) . "</p>";
			return $result;
		}
		
		/**
		* Liefert ein Formular mit dem Namen setToUrgentXDaysBefore, um die Anzahl der Tage vor dem Kaufdatum zu definieren, an denen der Artikel auf die Liste als dringend gesetzt wird.
		* @param int $days Anzahl der Tage vor dem Kaufdatum, vor dem der Artikel auf der Liste als dringend gesetzt wird.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function editSetToUrgentXDaysBefore($days,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="setToUrgentXDaysBefore">' . 
			$this->linebreak($indent+2) . 'Markiere den Artikel <i>x</i> Tage vor dem Stichtag des nächsten Kaufs als dringend: ' . 
			$this->linebreak($indent+1) . '</label>' . 
			$this->linebreak($indent+1) . '<input name="' . $prefix . 'setToUrgentXDaysBefore" id="setToUrgentXDaysBefore" type="number" value="' . $days . '" size="1">' .
			$this->linebreak($indent) . "</p>";
			return $result;
		}
		
		/**
		* Liefert ein Formular mit dem Namen editBuyEvery, um Wochentage als Stichtage des nächsten Kaufs zu definieren.
		* @param date $buyEvery Das Array aus Wochentagen.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function editBuyEvery($buyEvery,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			
			if(empty($buyEvery)){
				$buyEvery = array();
			}
			if (in_array("Mon",$buyEvery)){
				$mondayChecked = ' checked = "checked"';
			} else {
				$mondayChecked = "";
			}
			if (in_array("Tue",$buyEvery)){
				$tuesdayChecked = ' checked = "checked"';
			} else {
				$tuesdayChecked = "";
			}
			if (in_array("Wed",$buyEvery)){
				$wednesdayChecked = ' checked = "checked"';
			} else {
				$wednesdayChecked = "";
			}
			if (in_array("Thu",$buyEvery)){
				$thursdayChecked = ' checked = "checked"';
			} else {
				$thursdayChecked = "";
			}

			if (in_array("Fri",$buyEvery)){
				$fridayChecked = ' checked = "checked"';
			} else {
				$fridayChecked = "";
			}
			if (in_array("Sat",$buyEvery)){
				$saturdayChecked = ' checked = "checked"';
			} else {
				$saturdayChecked = "";
			}
			if (in_array("Sun",$buyEvery)){
				$sundayChecked = ' checked = "checked"';
			} else {
				$sundayChecked = "";
			}	
			
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<p>' . 
			$this->linebreak($indent+2) . 'Setze folgende Wochentage als Stichtage des nächsten Kaufs: ' . 
			$this->linebreak($indent+1) . '</p>' . 
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Mon" id="Montag"' . $mondayChecked . '>' . 
			$this->linebreak($indent+4) . 'Montag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' .
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Tue" id="Dienstag"' . $tuesdayChecked . '>' . 
			$this->linebreak($indent+4) . 'Dienstag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' . 
			
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Wed" id="Mittwoch"' . $wednesdayChecked . '>' . 
			$this->linebreak($indent+4) . 'Mittwoch' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' .
			
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Thu" id="Donnerstag"' . $thursdayChecked . '>' . 
			$this->linebreak($indent+4) . 'Donnerstag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' .
			
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Fri" id="Freitag"' . $fridayChecked . '>' . 
			$this->linebreak($indent+4) . 'Freitag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' .
			
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Sat" id="Samstag"' . $saturdayChecked . '>' . 
			$this->linebreak($indent+4) . 'Samstag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>' .
			
			
			$this->linebreak($indent+1) . '<ul>' .
			$this->linebreak($indent+2) . '<li>' .
			$this->linebreak($indent+3) . '<label>' .
			$this->linebreak($indent+4) . '<input type="checkbox" name="' . $prefix . 'buyEvery[]" value="Sun" id="Sonntag"' . $sundayChecked . '>' . 
			$this->linebreak($indent+4) . 'Sonntag' . 
			$this->linebreak($indent+3) . '</label>' .
			$this->linebreak($indent+2) . '</li>' . 
			$this->linebreak($indent+1) . '</ul>';
			
			return $result;
		}
		
		
		/**
		* Liefert ein Formular mit dem Namen editNextBuy, um den Stichtag des nächsten Kaufs zu definieren.
		* @param date $nextBuy Das Datum.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function editNextBuy($nextBuy,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="nextBuy">' . 
			$this->linebreak($indent+2) . 'Setze folgendes Datum als Stichtag des nächsten Kaufs: ' . 
			$this->linebreak($indent+1) . '</label>' . 
			$this->linebreak($indent+1) . '<input name="' . $prefix . 'nextBuy" id="nextBuy" type="date" value="' . $nextBuy . '" size="1">' .
			$this->linebreak($indent) . "</p>";
			return $result;
		}
		
		/**
		* liefert den HTML-Quelltext für die Bearbeitung des Formularfeldes Name  zurück.
		* @param object $article Der Article.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.
		*/ 
		private function formularArticleName($article,$prefix,$indent){
			if (!empty($prefix)){
				$prefix .= "_";
			}
			if (empty($article)){
				$article = new Article(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			}
			return  $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Bezeichnung' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . '<input type="text" name="' . $prefix . 'name" class ="fullWidth" value="' . $article->getName(). '">' . 
			$this->linebreak($indent) . '</fieldset>';
		}
		
		/**
		* liefert den HTML-Quelltext für die Bearbeitung der Formularfelder Name und Annotation zurück.
		* @param object $article Der Article.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.
		*/ 
		private function formularArticleNameAndAnnotation($article,$prefix,$indent){
			$articleName = $this->formularArticleName($article,$prefix,$indent);
			if (!empty($prefix)){
				$prefix .= "_";
			}
			if (empty($article)){
				$article = new Article(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			}
			return $articleName . 
			$this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Anmerkungen' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . '<input type="text" name="' . $prefix . 'annotation" class ="fullWidth" value="' .  $article->getAnnotation(). '"><p class="infotext">Anmerkungen, die Sie hier machen, sind fester Bestandteil des Artikels und werden deshalb auch bei jedem Listeneintrag ausgegeben.</p>' . 
			$this->linebreak($indent) . '</fieldset>';
		}
		
		/**
		* Liefert ein Formular zur Bearbeitung der Standardeinträge eines Artikels zurück.
		* @param object $article Der entsprechende Artikel.
		* @param string $prefix Das Präfix, das den Namen der verwendeten Formularfeldern vorangestellt wird.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/		
		private function selectStandardSettings($article,$prefix,$indent){
			$result = $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Standardeinstellungen' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . $this->printAmount($article->getStandardAmount(),$prefix,'standardAmount',$indent) .
			$this->linebreak($indent+1) . $this->printMeasure($article->getStandardMeasureID(),$prefix,'standardMeasureSerialized',$indent) .
			$this->linebreak($indent+1) . $this->editAnnotation($article->getStandardListEntryAnnotation(),NULL,"Anmerkungen, die Sie hier machen, erscheinen nur bei automatisch eingefügten Listeneinträgen.",$prefix,'standardListEntryAnnotation',$indent) .
			$this->linebreak($indent+1) . $this->selectPriority($article->getStandardPriorityID(),NULL,$prefix,'standardPrioritySerialized',$indent) .
			$this->linebreak($indent) . '</fieldset>';
			return $result;
		}
		
		/**
		* Überprüft, ob der einem ListEntry-oder Article-Objekt zugeordnete Name eines Articles auch die in Klammern gesetzte ArticleAnnotation enthält, weist diese ArticleAnnotation der Eigenschaft "ArticleAnnotation" zu und fügt dem ListEntry-Objekt die Article-ID zu.
		* @param object $obj Das zu überprüfende Article- oder ListEntry-Object.
		* @param string $typeOfReturnObject Der Objekttyp, der zurückgegeben werden soll ("ListEntry" oder "Article").
		* @return object Das überarbeitete Objekt.
		*/
		protected function reconstructArticleName($obj,$typeOfReturnObject) {
						
			// Achtung: Prüfung darf nur durchgeführt werden, wenn in Annotation (im Fall von Article) oder  ArticleAnnotation (im Fall von ListEntry) nicht bereits Inhalt hat; andernfalls scheitert Prüfung		
			if (($typeOfReturnObject == "ListEntry"  && $obj->getArticleAnnotation() != "") OR ($typeOfReturnObject == "Article"  && $obj->getAnnotation() != "")){
				return $obj;
			} else {
				include(ABS_PATH . "dbconnect.php");			
				$nameStr = "";
				$articleAnnotations = $this->extractContentFromBrackets($obj->getName()); // Inhalt des Namens wird erst einmal komplett als Annotation betrachtet und als Array gespeichert, wobei mit jeder runden Klammer ein neues Arrayelement beginnt.
				$leerzeichen = "";
				$counter = count($articleAnnotations);
				for($i = 0; $i < $counter; $i++){
					$nameStr .= $leerzeichen . $this->deleteSpaces($articleAnnotations[$i]); // Arrayelement[$i] zu nameStr hinzufügen
					unset($articleAnnotations[$i]); // Arrayelement aus $articleAnnotations löschen
					if (count($articleAnnotations) > 0){ // wenn noch Elemente in ArticleAnnotations vorhanden
						$modifiedArticleAnnotations = $articleAnnotations;
						$modifiedArticleAnnotations[$i+1] = $this->deleteBrackets($modifiedArticleAnnotations[$i+1]); // entferne die Klammern am ersten Element
						$annotationStr = " AND annotation = '" . implode(" ",$modifiedArticleAnnotations)  . "'";
					} else {
						$annotationStr = "";
					}
					$leerzeichen = " "; // nach dem ersten Durchlauf wird neuen Elementen im ArticleName ein Leerzeichen hinzugefügt 
					$selectStr = '
						SELECT
						' . DB_TAB_PREFIX . 'articles.ID,
						' . DB_TAB_PREFIX . 'articlesAnnotation.annotation
					FROM 
						' . DB_TAB_PREFIX . 'articles
					LEFT JOIN
						 ' . DB_TAB_PREFIX . 'articlesAnnotation
						 	ON
						 		' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
					WHERE 
						(
							name = "' . mysqli_real_escape_string($db, $nameStr) . '"' .
							$annotationStr .
						")";
					$query = mysqli_query($db, $selectStr); 
					$row = mysqli_fetch_object($query);
					if(mysqli_num_rows($query) > 0){ // falls Suche Ergebnis liefert (in Liste eingetragen)
						if (isset($articleAnnotations[$i+1])){
							$articleAnnotations[$i+1] = $this->deleteBrackets($articleAnnotations[$i+1]); // entferne die Klammern am ersten Element
						}
						$articleAnnotations = implode(" ",$articleAnnotations);

						// Übertragung des Ergebnisses
						if ($typeOfReturnObject == "ListEntry"){
							$obj->setArticleID($row->ID);
							$obj->setArticleAnnotation($articleAnnotations);
							$obj->setStandardsIfNULL();
							return $obj;
						} elseif($typeOfReturnObject == "Article") {
							return Article::constructByID($row->ID); // lädt die Standardeinstellungen
						}
					}
				}
				return false;			
			}
		}
	}
?>