<?php
	/** Datei, die die Definition der Klasse Article enthält. */
	
	/** 
	* Klasse zur Erstellung von Objekten des Typs Article 
	*/
	
	class Article extends SortableListElement	{ // Artikel samt Bezeichnung

		/** @var int Die ID der Kategorie, aus der heraus der Artikel aufgerufen wurde */ 
		private $catID;
		/** @var array Array sämtlicher Kategorien */  		
		private $categories;
		/** @var array Array sämtlicher Barcodes */  		
		private $barcodes;
		/** @var string Datum, an dem der Artikel erstmalig gekauft wurde */  		
		private $firstBuy;
		/** @var string Datum, an dem der Artikel letztmalig gekauft wurde */  		
		private $lastBuy;
		/** @var string Stichtag, an dem der Artikel das nächste Mal gekauft werden soll */
		private $nextBuy;
		/** @var array Array der Wochentage, an denen der Artikel regelmäßig gekauft werden soll. */
		private  $buyEvery;
		/** @var array Auflistung der bisherigen Kaufdaten als Array */  		
		private $purchases;		
		/** @var int Anzahl der bisherigen Käufe */  		
		private $numberOfPurchases;
		/** @var int Anzahl der Tage vom letzten Kauf bis zum nächsten Stichtag des Kaufes */  
		private $intervalInDays;
		/** @var int Anzahl der Tage vor dem nächsten Stichtag, bevor der Artikel automatisch wieder auf die Liste gesetzt wird */  
		private $addToListXDaysBefore;
		/** @var int Standardmenge, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte. */
		private $standardAmount;
		/** @var int Die ID der Standardmaßeinheit, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte.*/
		private $standardMeasureID;
		/** @var string Standardanmerkung für ListEntry, falls der Artikel automatisch auf die Liste gesetzt werden sollte. */
		private $standardListEntryAnnotation;
			/** @var int Die ID der Standardpriorität, falls der Artikel automatisch auf die Liste gesetzt werden sollte. */
		private $standardPriorityID;
		/** @var int Anzahl der Tage vor dem nächsten Stichtag, bevor der Artikel auf der Liste als dringend markiert wird */  
		private $setToUrgentXDaysBefore;

		/**
		* Der Konstruktor
		* @param int $ID Die ID des Objektes.
		* @param string $name Der Name des Objektes.
		* @param int $catID Die ID der zugehörigen Kategorie, aus der heraus er aufgerufen wird.
		* @param int $sort Die Position des Objektes in der Sortierreihenfolge der Kategorie, aus der heraus er aufgerufen wird.
		* @param string $annotation Die Anmerkung zum Artikel.
		* @param string $firstBuy Stichtag, an dem der Artikel das nächste Mal gekauft werden soll.
		* @param string $lastBuy Datum, an dem der Artikel letztmalig gekauft wurde.
		* @param string $nextBuy Stichtag, an dem der Artikel das nächste Mal gekauft werden soll.
		* @param array $buyEvery Array der Wochentage, an denen der Artikel regelmäßig gekauft werden soll.
		* @param array $numberOfPurchases Auflistung der bisherigen Kaufdaten als Array.
		* @param int $intervalInDays Anzahl der Tage vom letzten Kauf bis zum nächsten Stichtag des Kaufes.
		* @param int $addToListXDaysBefore Anzahl der Tage vor dem nächsten Stichtag, bevor der Artikel automatisch wieder auf die Liste gesetzt wird.
		* @param int $standardAmount Standardmenge, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte.
		* @param int $standardMeasureID Die ID Der Standardmaßeinheit, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte.
		* @param string $standardListEntryAnnotation Standardanmerkung für ListEntry, falls der Artikel automatisch auf die Liste gesetzt werden sollte.
		* @param int $standardPriorityID Die ID der Standardpriorität, falls der Artikel automatisch auf die Liste gesetzt werden sollte.		
		* @param int $setToUrgentXDaysBefore Anzahl der Tage vor dem nächsten Stichtag, bevor der Artikel auf der Liste als dringend markiert wird.
				* @param int $convertToArticleID ID des Artikels, in die ein Listeneintrag dieses Artikels nach Kauf konvertiert wird.	

		*/ 

		public function __construct($ID,$name,$catID,$sort,$annotation,$firstBuy,$lastBuy,$nextBuy,$buyEvery,$numberOfPurchases,$intervalInDays,$addToListXDaysBefore,$standardAmount,$standardMeasureID,$standardListEntryAnnotation,$standardPriorityID,$setToUrgentXDaysBefore,$convertToArticleID) {
			parent::__construct($ID,$name,$annotation,$sort);
			include(ABS_PATH . "dbconnect.php");			
			$this->catID = $catID;
			$this->categories = $this->getRelatedCategories();
			$this->barcodes = $this->getRelatedBarcodes();
			$this->prefix = "article";
			$this->table = DB_TAB_PREFIX . 'articles';
			$this->tableAnnotation = DB_TAB_PREFIX . 'articlesAnnotation';
			$this->firstBuy = $firstBuy;
			$this->lastBuy = $lastBuy;
			$this->nextBuy = $nextBuy;
			$this->buyEvery = $buyEvery;
			$this->purchases = $this->getRelatedPurchases();
			$this->numberOfPurchases = $numberOfPurchases;
			$this->intervalInDays = $intervalInDays;
			$this->standardAmount = $standardAmount;
			$this->standardMeasureID = $standardMeasureID;
			$this->standardListEntryAnnotation = $standardListEntryAnnotation;
			$this->addToListXDaysBefore = $addToListXDaysBefore;
			$this->standardPriorityID = $standardPriorityID;
			$this->setToUrgentXDaysBefore = $setToUrgentXDaysBefore;
			$this->convertToArticleID = $convertToArticleID;
		}


		/**
		* Konstruiert ein Objekt auf der Basis seiner ID
		* @param int $ID Die ID des Objektes.
		* @return object Ein Objekt vom Typ Article.
		*/ 

		public function constructByID($ID){
			include(ABS_PATH . "dbconnect.php");
			$name = "";
			$categories = array();
			$barcodes = array();
			$selectStr = '
				SELECT
					' . DB_TAB_PREFIX . 'articles.ID,
					' . DB_TAB_PREFIX . 'articles.name,
					' . DB_TAB_PREFIX . 'articles.firstBuy,
					' . DB_TAB_PREFIX . 'articles.lastBuy,
					' . DB_TAB_PREFIX . 'articles.nextBuy,
					' . DB_TAB_PREFIX . 'articlesBuyEvery.weekday,
					' . DB_TAB_PREFIX . 'articles.standardPriorityID,
					' . DB_TAB_PREFIX . 'articles.numberOfPurchases,
					' . DB_TAB_PREFIX . 'articlesIntervalInDays.intervalInDays,
					' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore.addToListXDaysBefore,
					' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore.setToUrgentXDaysBefore,
					' . DB_TAB_PREFIX . 'articlesStandards.standardAmount,
					' . DB_TAB_PREFIX . 'articlesStandards.standardMeasureID,
					' . DB_TAB_PREFIX . 'articlesStandards.standardListEntryAnnotation,
					' . DB_TAB_PREFIX . 'articlesAnnotation.annotation,
					' . DB_TAB_PREFIX . 'articlesConvertToArticle.convertToArticleID
				FROM 
					' . DB_TAB_PREFIX . 'articles
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesIntervalInDays
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesIntervalInDays.articleID					
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesStandards
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesStandards.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesBuyEvery
					ON
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesBuyEvery.articleID				
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAnnotation
					ON
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesConvertToArticle
					ON
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesConvertToArticle.articleID
				WHERE
					' . DB_TAB_PREFIX . 'articles.id = ' . $ID;
			$query = mysqli_query($db, $selectStr);
			if (mysqli_num_rows($query) > 0){
				$row = mysqli_fetch_object($query);
				if (!empty($row->weekday)){
					$weekday = explode(", ",$row->weekday);
				} else {
					$weekday = NULL;
				}
				return new Article(
					$row->ID,
					$row->name,
					NULL, //$catID
					NULL, //$sort
					$row->annotation,
					$row->firstBuy,
					$row->lastBuy,
					$row->nextBuy,
					$weekday,
					$row->numberOfPurchases,
					$row->intervalInDays,
					$row->addToListXDaysBefore,
					$row->standardAmount,
					$row->standardMeasureID,
					$row->standardListEntryAnnotation,
					$row->standardPriorityID,
					$row->setToUrgentXDaysBefore,
					$row->convertToArticleID
				);
			}
			return NULL;
		}
		
	
		
		
		/**
		* gibt die einem Artikel zugeordneten Kategorien inclusive deren Namen, Sortierreihenfolge und Listennamen als Array zurück.
		* @return array Das Array. 
		*/
		private function getRelatedCategories(){
			if (!empty($this->ID)){
				include(ABS_PATH . "dbconnect.php");
				$result = array(); 
				$selectStr = '
					SELECT
						' . DB_TAB_PREFIX . 'articlesCategories.catID,
						' . DB_TAB_PREFIX . 'categories.name AS catName,
						' . DB_TAB_PREFIX . 'articlesCategories.sort,
						' . DB_TAB_PREFIX . 'lists.name AS listName
					FROM
						' . DB_TAB_PREFIX . 'articlesCategories
					LEFT JOIN
						' . DB_TAB_PREFIX . 'categories 
							ON 
								' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . DB_TAB_PREFIX . 'categories.ID
					LEFT JOIN
						' . DB_TAB_PREFIX . 'lists
							ON
								' . DB_TAB_PREFIX . 'categories.listID = ' . DB_TAB_PREFIX . 'lists.ID
					WHERE 
						articleID = ' . $this->ID;
				$query = mysqli_query($db, $selectStr);
				while ($row = mysqli_fetch_object($query)){
					$category = array(
						"cat_ID" => $row->catID,
						"cat_name" => $row->catName,
						"sort" => $row->sort,
						"list_name" => $row->listName
					);
					array_push($result,$category);
				}
				return $result;
			}
			return NULL;
		}
			
		
		/**
		* führt eine Datenbanksuche durch und weist die dem Artikel zugeordnete ID dem Objekt zu.
		* Diese Funktion ist notwendig, um nach dem Einfügen eines neuen Artikels dem Objekt die ID zuweisen zu können.
		* @return void. 
		*/
		public function refreshID(){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT
					' . DB_TAB_PREFIX . 'articles.ID
				FROM
					' . DB_TAB_PREFIX . 'articles
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesIntervalInDays
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesIntervalInDays.articleID					
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesStandards
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesStandards.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAnnotation
					ON
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesConvertToArticle
					ON
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesConvertToArticle.articleID
				WHERE (
					name' . $this->checkIfNULL($this->name) . ' AND 
					firstBuy' .  $this->checkIfNULL($this->firstBuy) . ' AND
					lastBuy' .  $this->checkIfNULL($this->lastBuy) . ' AND
					nextBuy' .  $this->checkIfNULL($this->nextBuy) . ' AND
					numberOfPurchases = \'' . $this->numberOfPurchases . '\' AND
					intervalInDays' .  $this->checkIfNULL($this->intervalInDays) . ' AND
					addToListXDaysBefore' .  $this->checkIfNULL($this->addToListXDaysBefore) . ' AND
					setToUrgentXDaysBefore' .  $this->checkIfNULL($this->setToUrgentXDaysBefore) . ' AND 
					standardAmount' .  $this->checkIfNULL($this->standardAmount) . ' AND
					standardMeasureID' .  $this->checkIfNULL($this->standardMeasureID) . ' AND 
					standardListEntryAnnotation' .  $this->checkIfNULL($this->standardListEntryAnnotation) . ' AND
					standardPriorityID' .  $this->checkIfNULL($this->	standardPriorityID) . ' AND
					annotation' .  $this->checkIfNULL($this->annotation) . ' AND
					convertToArticleID' .  $this->checkIfNULL($this->convertToArticleID) . ')';
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			$this->ID = $row->ID;
		}
	
		/**
		* gibt die einem Artikel zugeordneten Barcodes als Array zurück.
		* @return array Das Array. 
		*/
		private function getRelatedBarcodes(){
			if (!empty($this->ID)){
				include(ABS_PATH . "dbconnect.php");
				$result = array(); 
				$selectStr = '
					SELECT
						' . DB_TAB_PREFIX . 'barcodes.name
					FROM
						' . DB_TAB_PREFIX . 'barcodes
					WHERE 
						articleID = ' . $this->ID;
				$query = mysqli_query($db, $selectStr);
				while ($row = mysqli_fetch_object($query)){
					array_push($result,$row->name);
				}
				return $result;	
			}
			return NULL;
		}

		
		/**
		* gibt die einem Artikel zugeordneten Käufe als Array zurück.
		* @return array Das Array. 
		*/
		private function getRelatedPurchases(){
			if (!empty($this->ID)){
				include(ABS_PATH . "dbconnect.php");
				$result = array(); 
				$selectStr = '
					SELECT
						' . DB_TAB_PREFIX . 'articlesBought.date
					FROM
						' . DB_TAB_PREFIX . 'articlesBought
					WHERE 
						articleID = ' . $this->ID;
				$query = mysqli_query($db, $selectStr);
				while ($row = mysqli_fetch_object($query)){
					array_push($result,$row->date);
				}
				return $result;	
			}
			return NULL;	
		}
		
		/**
		* Setter der Variable $catID
		* @param string $catID Die ID der Kategorie, aus der heraus der Artikel gerade aufgerufen wird
		* @return void
		*/ 
		public function setCatID($catID){
			$this->catID = $catID;
		}
		
		/**
		* Getter der Variable $catID
		* @return array Inhalt der Variable $catID.
		*/ 
		public function getCatID(){
			return $this->catID;
		}		

	
		/**
		* Setter der Variable $categories
		* @param string $categories Die zugeordneten Kategorien
		* @return void
		*/ 
		public function setCategories($categories){
			$this->categories = $categories;
		}
		
		/**
		* Getter der Variable $categories
		* @return array Inhalt der Variable $categories.
		*/ 
		public function getCategories(){
			return $this->categories;
		}
				
		/**
		* Setter der Variable $barcodes
		* @param string $barcodes Die zugeordneten Barcodes
		* @return void
		*/ 
		public function setBarcodes($barcodes){
			$this->barcodes = $barcodes;
		}
		
		/**
		* Getter der Variable $barcodes.
		* @return array Inhalt der Variable $barcodes.
		*/ 
		public function getBarcodes(){
			return $this->barcodes;
		}
		
		
				/**
		* Setter der Variable $barcodes.
		* @param array $array Array der Wochentage, an denen der Artikel regelmäßig gekauft wird.
		* @return void
		*/ 
		public function setBuyEvery($array){
			$this->buyEvery = $array;
		}
		
		/**
		* Getter der Variable $buyEvery.
		* @return array Inhalt der Variable $buyEvery.
		*/ 
		public function getBuyEvery(){
			return $this->buyEvery;
		}
		
		/**
		* fügt einen Barcode für den betreffenden Artikel in die MySQL-Datenbank ein.
		* @param string $barcode Der Barcode.
		* return void.  
		*/
		public function addBarcodeToDB($barcode){
			if ($barcode != ""){
				include(ABS_PATH . "dbconnect.php");
				$insertStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'barcodes SET name = "' . $barcode . '", articleID = ' . $this->ID;	
				mysqli_query($db,$insertStr);
			}
		}
				
		/**
		* Setter der Variable $standardAmount.
		* @param int $standardAmount Standardmenge, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte.
		* @return void
		*/ 
		public function setStandardAmount($standardAmount){
			$this->standardAmount = $standardAmount;
		}	
		
		/**
		* Getter der Variable $standardAmount
		* @return array Inhalt der Variable $standardAmount.
		*/ 
		public function getStandardAmount(){
			return $this->standardAmount;
		}
		
	

		/**
		* Setter der Variable $standardMeasureID.
		* @param int $standardMeasureID Die ID der Standardmaßeinheit, falls der Artikel automatisch auf die Liste gesetzt oder bei einem Listeneintrag keine entsprechenden Angaben gemacht werden sollte.
		* @return void
		*/ 
		public function setStandardMeasureID($standardMeasureID){
			$this->standardMeasureID = $standardMeasureID;
		}	
		
		/**
		* Getter der Variable $standardMeasureID
		* @return array Inhalt der Variable $standardMeasureID.
		*/ 
		public function getStandardMeasureID(){
			return $this->standardMeasureID;
		}
		
		/**
		* Setter der Variable $standardListEntryAnnotation.
		* @param string $standardListEntryAnnotation Standardanmerkung für ListEntry, falls der Artikel automatisch auf die Liste gesetzt werden sollte..
		* @return void
		*/ 
		public function setStandardListEntryAnnotation($standardListEntryAnnotation){
			$this->standardListEntryAnnotation = $standardListEntryAnnotation;
		}				
		
		/**
		* Getter der Variable standardListEntryAnnotation
		* @return array Inhalt der Variable $standardListEntryAnnotation.
		*/ 
		public function getStandardListEntryAnnotation(){
			return $this->standardListEntryAnnotation;
		}
		
		
		
		/**
		* Setter der Variable $standardPriorityID.
		* @param string $standardPriorityID Die ID der Standardpriorität, falls der Artikel automatisch auf die Liste gesetzt werden sollte.
		* @return void
		*/ 
		public function setStandardPriorityID($standardPriorityID){
			$this->standardPriorityID = $standardPriorityID;
		}			
		
		/**
		* Getter der Variable $getStandardPriorityID
		* @return array Inhalt der Variable $standardPriorityID.
		*/ 
		public function getStandardPriorityID(){
			return $this->standardPriorityID;
		}
		
		/**
		* Getter der Variable $intervalInDays
		* @return array Inhalt der Variable $intervalInDays.
		*/ 
		public function getIntervalInDays(){
			return $this->intervalInDays;
		}
		
		/**
		* Setter der Variable $intervalInDays.
		* @param int $intervalInDays Anzahl der Tage vom letzten Kauf bis zum nächsten Stichtag des Kaufes
		* @return void
		*/ 
		public function setIntervalInDays($intervalInDays){
			$this->intervalInDays = $intervalInDays;
		}	
		
		
		/**
		* Setter der Variable $addToListXDaysBefore.
		* @param int $addToListXDaysBefore Anzahl der Tage vor dem Stichtag, an dem der Artikel automatisch auf die Liste gesetzt wird.
		* @return void
		*/ 
		public function setAddToListXDaysBefore($addToListXDaysBefore){
			$this->addToListXDaysBefore = $addToListXDaysBefore;
		}			
		
		/**
		* Getter der Variable $addToListXDaysBefore
		* @return int Inhalt der Variable $addToListXDaysBefore.
		*/ 
		public function getAddToListXDaysBefore(){
			return $this->addToListXDaysBefore;
		}
		
		/**
		* Setter der Variable $setToUrgentXDaysBefore.
		* @param int $setToUrgentXDaysBefore Anzahl der Tage vor dem Stichtag, an dem der Artikel als dringend markiert wird.
		* @return void
		*/ 
		public function setSetToUrgentXDaysBefore($setToUrgentXDaysBefore){
			$this->setToUrgentXDaysBefore = $setToUrgentXDaysBefore;
		}			
		
		/**
		* Getter der Variable $setToUrgentXDaysBefore.
		* @return int Inhalt der Variable $setToUrgentXDaysBefore.
		*/ 
		public function getSetToUrgentXDaysBefore(){
			return $this->setToUrgentXDaysBefore;
		}
		
		
		/**
		* Setter der Variable $firstBuy.
		* @param string $firstBuy Das Datum des ersten Kaufs als String.
		* @return void
		*/ 
		public function setFirstBuy($firstBuy){
			$this->firstBuy = $firstBuy;
		}		
		
		/**
		* Getter der Variable $firstBuy.
		* @return string Inhalt der Variable $firstBuy.
		*/ 
		public function getFirstBuy(){
			return $this->firstBuy;
		}
		
		/**
		* Setter der Variable $lastBuy.
		* @param string $lastBuy Das Datum des letzten Kaufs als String.
		* @return void
		*/ 
		public function setLastBuy($lastBuy){
			$this->lastBuy = $lastBuy;
		}		
		
		/**
		* Getter der Variable $lastBuy.
		* @return string Inhalt der Variable $lastBuy.
		*/ 
		public function getLastBuy(){
			return $this->lastBuy;
		}
		
		/**
		* Setter der Variable $nextBuy.
		* @param string $nextBuy Das Datum des nächsten Kaufs als String.
		* @return void
		*/ 
		public function setNextBuy($nextBuy){
			$this->nextBuy = $nextBuy;
		}		
		
		/**
		* Getter der Variable $nextBuy.
		* @return string Inhalt der Variable $nextBuy.
		*/ 
		public function getNextBuy(){
			return $this->nextBuy;
		}
		
				/**
		* Setter der Variable $numberOfPurchases.
		* @param int $numberOfPurchases Die Anzahl der bisherigen Käufe.
		* @return void
		*/ 
		public function setNumberOfPurchases($numberOfPurchases){
			$this->numberOfPurchases = $numberOfPurchases;
		}		
		
		/**
		* Getter der Variable $numberOfPurchases.
		* @return int Inhalt der Variable $numberOfPurchases.
		*/ 
		public function getNumberOfPurchases(){
			return $this->numberOfPurchases;
		}
		
		
		
				/**
		* Setter der Variable $convertToArticleID.
		* @param string $convertToArticleID Das ID des Artikels, in die der Artikel nach dem Löschen aus dem Warenkorb konvertiert werden soll.
		* @return void
		*/ 
		public function setConvertToArticleID($convertToArticleID){
			$this->convertToArticleID = $convertToArticleID;
		}		
		
		/**
		* Getter der Variable $convertToArticleID.
		* @return array Inhalt der Variable $convertToArticleID.
		*/ 
		public function getConvertToArticleID(){
			return $this->convertToArticleID;
		}
		
		
		/**
		* Gibt die IDs der Kategorien zurück, denen der Artikel zugeordnet ist.
		* @return array Inhalt der Variable $nextBuy.
		*/ 
		public function getCatIDs(){
			$result = array();
			foreach ($this->categories AS $category){
				array_push($result,$category['cat_ID']);
			};
			return $result;
		}

		/**
		* prüft, ob ein Wert bereits als Name in der Datenbank existiert existiert, und gibt entsprechend true oder false zurück.
		* Für diese Funktion müssen die Eigenschaften table und name implementiert sein.
		* @param string $name Der Name, der auf Existenz geprüft werden soll.
		* @return bool Das Ergebnis, ob der Name bereits in der Datenbank existiert.
		*/
		public function nameAlreadyExists($name){
			return parent::nameAlreadyExists_wsa($name,"");
		}

		
		/**
		* fügt der in den Objekteigenschaften genannten $table einen weiteren Datenbankeintrag hinzu und passt die Sortierung entsprechend an.
		* Für die korrekte Funktion muss die Eigenschaft $table definiert sein.
		* @return void.
		*/
		public function addToDB(){
			$updateStr = 'UPDATE ' . DB_TAB_PREFIX . 'articles LEFT JOIN ' . DB_TAB_PREFIX . 'articlesCategories ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX .  'articlesCategories.articleID SET sort = sort+1 WHERE (' . DB_TAB_PREFIX . 'articlesCategories.sort >= ' . $this->sort . ' AND ' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . $this->catID . ')';
					
			$insertStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'articles SET name = ' . $this->addBracketsIfNotNull($this->name) . ', firstBuy = ' . $this->addBracketsIfNotNull($this->firstBuy) . ', lastBuy = ' . $this->addBracketsIfNotNull($this->lastBuy) . ', nextBuy = ' . $this->addBracketsIfNotNull($this->nextBuy) . ', standardPriorityID = ' . $this->addBracketsIfNotNull($this->standardPriorityID) . ', numberOfPurchases = 0';

			if ($this->annotation != ""){
				$annotationStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesAnnotation SET articleID = [[thisID]], annotation = "' . $this->annotation . '"';
			} else {
				$annotationStr = NULL;
			}

			$strArray = array();
			if ($this->addToListXDaysBefore != ""){
				$insertArticlesAddToListXDaysBefore = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore SET articleID = [[thisID]], addToListXDaysBefore = ' . $this->addBracketsIfNotNull($this->addToListXDaysBefore);
				array_push($strArray,$insertArticlesAddToListXDaysBefore);
			}

			$insertArticleCategories = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesCategories SET articleID = [[thisID]], catID = "' . $this->catID . '", sort = "' . $this->sort . '"';
			array_push($strArray,$insertArticleCategories);						
			
			if ($this->intervalInDays != "") {
				$insertArticlesIntervalInDays = 'INSERT INTO	' . DB_TAB_PREFIX . 'articlesIntervalInDays SET articleID = [[thisID]], intervalInDays = ' . $this->addBracketsIfNotNull($this->intervalInDays);
				array_push($strArray,$insertArticlesIntervalInDays);						
			} 
			
			if ($this->buyEvery != "") {
				$insertArticlesBuyEvery = 'INSERT INTO	' . DB_TAB_PREFIX . 'articlesBuyEvery SET articleID = [[thisID]], weekday = ' . $this->addBracketsIfNotNull(implode(", ", $this->buyEvery));
				array_push($strArray,$insertArticlesBuyEvery);						
			} 

			if ($this->addToListXDaysBefore != ""){
				$insertArticlesSetToUrgentXDaysBefore = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore SET articleID = [[thisID]], setToUrgentXDaysBefore = ' . $this->addBracketsIfNotNull($this->setToUrgentXDaysBefore);
				array_push($strArray,$insertArticlesSetToUrgentXDaysBefore);		
			}
			
			if ($this->standardAmount != "" OR $this->standardMeasureID != "" OR $this->standardListEntryAnnotation != ""){
				$insertArticlesStandards = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesStandards SET articleID = [[thisID]], standardAmount = ' . $this->addBracketsIfNotNull($this->standardAmount) . ',					standardMeasureID = ' . $this->addBracketsIfNotNull($this->	standardMeasureID) . ',standardListEntryAnnotation = ' . $this->addBracketsIfNotNull($this->standardListEntryAnnotation);
				array_push($strArray,$insertArticlesStandards);		
			}
			
			if ($this->convertToArticleID != "" ){
				$convertToArticle = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesConvertToArticle SET articleID = [[thisID]], convertToArticleID = ' . $this->addBracketsIfNotNull($this->convertToArticleID);
				array_push($strArray, $convertToArticle);
			}
			parent::addToDB_wp($updateStr,$insertStr,$annotationStr,$strArray);
		}

		/**
		* entfernt das Objekt aus der in den Objekteigenschaften genannten $table
		* @return void
		*/
		public function deleteFromDB(){
			include(ABS_PATH . "dbconnect.php");
			// Lösche Einträge aus articlesCategories
			$updateArray = array();
			$strArray = array();
			$selectCats = '
				SELECT 
					catID,
					sort 
				FROM 
					' . DB_TAB_PREFIX . 'articlesCategories 
				WHERE 
					articleID = ' . $this->ID . ' 
				ORDER BY 
					catID 
						DESC';
			$query = mysqli_query($db, $selectCats);
			while($row = mysqli_fetch_assoc($query)){
				$updateStr = 'UPDATE
					' . DB_TAB_PREFIX . 'articlesCategories
				SET 
					sort = sort-1 
				WHERE (
					sort >= ' . $row['sort'] . '
					AND 
					catID = ' . $row['catID'] . ')';
				array_push($updateArray,$updateStr);
			};
			

			$deleteFromCats = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesCategories WHERE articleID = "' . $this->ID . '"';
			array_push($strArray, $deleteFromCats);
			
			// Lösche Einträge aus articles
			$deleteStr = 'DELETE FROM ' . DB_TAB_PREFIX . 'articles WHERE ID = "' . $this->ID . '"';
		//	array_push($deleteArray, $deleteFromCats);
			
			if ($this->addToListXDaysBefore != ""){
				$deleteAddToListXDaysBefore = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteAddToListXDaysBefore);
			}
			
			// Lösche Einträge aus articlesAnnotation
			if ($this->annotation != ""){
				$annotationStr = 'DELETE FROM	' . DB_TAB_PREFIX . 'articlesAnnotation WHERE articleID = "' . $this->ID . '"';
			} else {
				$annotationStr = NULL;
			}
			
			// Lösche Einträge aus articlesBought
			if (count($this->purchases) > 0){
				
				$deleteBoughtArticles = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesBought WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteBoughtArticles);
			}
			
			// Lösche Einträge aus articlesConvertToArticle
			if ($this->convertToArticleID != "" ){
				$deleteConvertToArticle = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesConvertToArticle WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteConvertToArticle);
			}
			
			// Lösche Einträge aus articlesIntervalInDays
			if ($this->intervalInDays != "" ){
				$deleteIntervalInDays = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesIntervalInDays WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteIntervalInDays);
			}
			
			// Lösche Einträge aus articlesBuyEvery
			if ($this->buyEvery != "" ){
				$deleteBuyEvery = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesBuyEvery WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteBuyEvery);
			}
			
			// Lösche Einträge aus articlesIntervalInDays
			if ($this->setToUrgentXDaysBefore != "" ){
				$deleteSetToUrgentXDaysBefore = 'DELETE FROM	' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteSetToUrgentXDaysBefore);
			}
			
			// Lösche Standardeinträge
			if ($this->standardAmount != "" OR $this->standardMeasureID != "" OR $this->standardListEntryAnnotation != ""){
				$deleteArticlesStandard = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesStandards WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteArticlesStandard);
			}
			
			// lösche Barcodes
			if (count($this->getBarcodes())>0){
				$deleteBarcodes = 'DELETE FROM ' . DB_TAB_PREFIX . 'barcodes WHERE articleID = "' . $this->ID . '"';
				array_push($strArray, $deleteBarcodes);
			}

			parent::deleteFromDB_wp($updateArray,$deleteStr,$annotationStr,$strArray);
		}


		/**
		* prüft, ob für das Objekt Standardeinträge vorhanden sind, und gibt ggf. TRUE zurücl
		* @return bool.
		*/
		private function hasStandardSettings(){
			if (!empty($this->standardAmount) OR !empty($this->standardMeasureID) OR !empty($this->standardListEntryAnnotation)){
				return true;
			}
			return false;
		}

		/**
		* Gibt den Namen und ggf. die Anmerkung zum Artikel in formatierter Form zurück.
		* @return string Name und Anmerkungen als formatierter String
		*/
		public function getArticleNameAndArticleAnnotation(){
			if ($this->annotation != "") {
				$annotation = " (" . $this->annotation . ")";
			} else {
				$annotation = "";
			}
			return $this->name . $annotation;
		}

		/**
		* aktualisiert den entsprechenden Datenbankeintrag.
		* @return void
		*/
		public function updateDB(){
			include(ABS_PATH . "dbconnect.php");
			$strArray = array();

			$oldObject = $this->constructByID($this->ID);
			$rowLabel = "articleID";
			
			$updateStr = '
				UPDATE
					' . DB_TAB_PREFIX . 'articles
				SET
					name = "' . $this->name. '",
					nextBuy = ' . $this->addBracketsIfNotNULL($this->nextBuy). ',
					standardPriorityID = ' . $this->addBracketsIfNotNull($this->standardPriorityID) . '
				WHERE ID = ' . $this->ID;		

			/** Bearbeitung der automatischen Listeneinträge **/
			// intervalInDays
			if ($oldObject->intervalInDays){ // wenn Intervall eingetragen ist
				if (empty($this->intervalInDays)){ // wenn für neues Objekt kein Intervall eingetragen ist
					$deleteStr = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesIntervalInDays WHERE articleID = ' . $this->ID;
						array_push($strArray,$deleteStr);
				} else { // wenn für neues Objekt anderes Intervall eingetragen ist
					$updateStrIntervalInDays = 'UPDATE ' . DB_TAB_PREFIX . 'articlesIntervalInDays SET intervalInDays = ' . $this->addBracketsIfNotNULL($this->intervalInDays) . ' WHERE articleID = ' . $this->ID;

						array_push($strArray,$updateStrIntervalInDays);
				}
			} else { // wenn kein Intervall eingetragen ist
				if ($this->intervalInDays){ // wenn für neues Objekt Intervall eingetragen ist
					$insertStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesIntervalInDays SET articleID = ' . $this->ID . ', intervalInDays = ' . $this->addBracketsIfNotNULL($this->intervalInDays);
					array_push($strArray,$insertStr);
				}
			}

			// addToListXDaysBefore
			if ($oldObject->addToListXDaysBefore){ // wenn Zeitraum eingetragen ist
				if (empty($this->addToListXDaysBefore)){ // wenn für neues Objekt kein Zeitraum eingetragen ist
						$deleteStr = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore WHERE articleID = ' . $this->ID;
						array_push($strArray,$deleteStr);
				} else { // wenn für neues Objekt anderer Zeitraum eingetragen ist
					$updateStrAddToListXDaysBefore = 'UPDATE ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore SET addToListXDaysBefore = ' . $this->addBracketsIfNotNULL($this->addToListXDaysBefore) . ' WHERE articleID = ' . $this->ID;
						array_push($strArray,$updateStrAddToListXDaysBefore);
				}
			} else { // wenn kein Zeitraum eingetragen ist
				if ($this->addToListXDaysBefore){ // wenn für neues Objekt Zeitraum eingetragen ist
					$insertStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore SET articleID = ' . $this->ID . ', addToListXDaysBefore = ' . $this->addBracketsIfNotNULL($this->addToListXDaysBefore);
					array_push($strArray,$insertStr);
				}
			}
			
			// articlesSetToUrgentXDaysBefore
			if ($oldObject->setToUrgentXDaysBefore){ // wenn Zeitraum eingetragen ist
				if (empty($this->setToUrgentXDaysBefore)){ // wenn für neues Objekt kein Zeitraum eingetragen ist
						$deleteStr = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore WHERE articleID = ' . $this->ID;
						array_push($strArray,$deleteStr);
				} else { // wenn für neues Objekt anderer Zeitraum eingetragen ist
					$updateStrSetToUrgentXDaysBefore = 'UPDATE ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore SET setToUrgentXDaysBefore = ' . $this->addBracketsIfNotNULL($this->setToUrgentXDaysBefore) . ' WHERE articleID = ' . $this->ID;
						array_push($strArray,$updateStrSetToUrgentXDaysBefore);
				}
			} else { // wenn kein Zeitraum eingetragen ist
				if ($this->setToUrgentXDaysBefore){ // wenn für neues Objekt Zeitraum eingetragen ist
					$insertStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore SET articleID = ' . $this->ID . ', setToUrgentXDaysBefore = ' . $this->addBracketsIfNotNULL($this->setToUrgentXDaysBefore);
					array_push($strArray,$insertStr);
				}
			}
			
			// articlesConvertToArticle
			if ($oldObject->convertToArticleID){ // wenn Artikel eingetragen ist
				if (empty($this->convertToArticleID)){ // wenn für neues Objekt kein Artikel eingetragen ist
						$deleteStrArticlesConvertToArticle = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesConvertToArticle WHERE articleID = ' . $this->ID;
						array_push($strArray,$deleteStrArticlesConvertToArticle);
				} else { // wenn für neues Objekt anderer Artikel eingetragen ist
					$updateStrArticlesConvertToArticle = 'UPDATE ' . DB_TAB_PREFIX . 'articlesConvertToArticle SET convertToArticleID = ' . $this->addBracketsIfNotNULL($this->convertToArticleID) . ' WHERE articleID = ' . $this->ID;
						array_push($strArray,$updateStrArticlesConvertToArticle);
				}
			} else { // wenn kein Zeitraum eingetragen ist
				if ($this->convertToArticleID){ // wenn für neues Objekt Zeitraum eingetragen ist
					$insertStrArticlesConvertToArticle = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesConvertToArticle SET articleID = ' . $this->ID . ', convertToArticleID = ' . $this->addBracketsIfNotNULL($this->convertToArticleID);
					array_push($strArray,$insertStrArticlesConvertToArticle);
				}
			}
		
			// Stichtag des nächsten Kaufs (nextBuy)
			if ($oldObject->nextBuy){ // wenn für altes Objekt Stichtag des nächsten Kaufs eingetragen ist
				if ($this->nextBuy AND ($oldObject->nextBuy != $this->nextBuy)){ // wenn für neues Objekt Stichtag des nächsten Kaufs eingetragen ist
					$updateStrAddBracketsIfNotNULL = 'UPDATE ' . DB_TAB_PREFIX . 'articles SET nextBuy = ' . $this->addBracketsIfNotNULL($this->getNextBuy()) . ' WHERE ID = ' . $this->ID;
					array_push($strArray,$updateStrAddBracketsIfNotNULL);
				} elseif(empty($this->nextBuy)) { // wenn für neues Objekt kein Stichtag mehr eingetragen ist
					$updateStrNextBuy = 'UPDATE ' . DB_TAB_PREFIX . 'articles SET nextBuy = NULL WHERE ID = ' . $this->ID;
					array_push($strArray,$updateStrNextBuy);
				}
			} else { // wenn für altes Objekt kein Stichtag des nächsten Kaufs eingetragen ist
				if ($this->nextBuy){
					$updateStrNextBuy = 'UPDATE ' . DB_TAB_PREFIX . 'articles SET nextBuy = ' . $this->addBracketsIfNotNULL($this->getNextBuy()) . ' WHERE ID = ' . $this->ID;
					array_push($strArray,$updateStrNextBuy);
				}
			}
			if ($oldObject->buyEvery){ // wenn für altes Objekt Wochentage eingetragen sind
				if ($this->buyEvery AND ($oldObject->buyEvery != $this->buyEvery)){ // wenn für neues Objekt Wochentag eingetragen ist
					$updateStrBuyEveryIfNotNULL = 'UPDATE ' . DB_TAB_PREFIX . 'articlesBuyEvery SET weekday = ' . $this->addBracketsIfNotNULL(implode(", ", $this->getBuyEvery())) . ' WHERE articleID = ' . $this->ID;
					array_push($strArray,$updateStrBuyEveryIfNotNULL);
				} elseif(empty($this->buyEvery)) { // wenn für neues Objekt kein Wochentag mehr eingetragen ist
					$deleteStrBuyEvery = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesBuyEvery WHERE articleID = ' . $this->ID;
					array_push($strArray,$deleteStrBuyEvery);
				}
			} else {
				if ($this->buyEvery){
					$insertStrBuyEvery = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesBuyEvery SET weekday = ' . $this->addBracketsIfNotNULL(implode(", ", $this->getBuyEvery())) . ',  articleID = ' . $this->ID;
					array_push($strArray,$insertStrBuyEvery);
				}
			}

			/** Bearbeitung der Standardeinstellungen **/
			if ($oldObject->hasStandardSettings()){ // wenn altes Objekt Standardeinstellungen hat
				if ($this->hasStandardSettings()){ // wenn neues Objekt Standardeinstellungen hat, alte Standardeinstellungen updaten
					$updateStrStandards = 'UPDATE ' . DB_TAB_PREFIX . 'articlesStandards SET standardAmount = ' . $this->addBracketsIfNotNULL($this->standardAmount) . ', standardMeasureID = ' . $this->addBracketsIfNotNULL($this->standardMeasureID) . ', standardListEntryAnnotation = ' . $this->addBracketsIfNotNULL($this->standardListEntryAnnotation) . ' WHERE articleID = ' . $this->ID;
						array_push($strArray,$updateStrStandards);

				} else { // wenn neues Objekt keine Standardeinstellungen hat, alte Standardeinstellungen löschen
					$deleteStrStandards = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesStandards WHERE articleID = ' . $this->ID;
						array_push($strArray,$deleteStrStandards);					
				}
			} 
			else { // wenn altes Objekt keine Standardeinstellungen hat 
				if ($this->hasStandardSettings()){ // wenn das neue Objekt Standardeinstellungen hat
					$insertStrStandards = 'INSERT INTO ' . DB_TAB_PREFIX . 'articlesStandards SET articleID = ' . $this->ID . ', standardAmount = ' . $this->addBracketsIfNotNULL($this->standardAmount) . ', standardMeasureID = ' . $this->addBracketsIfNotNULL($this->standardMeasureID) . ', standardListEntryAnnotation = ' . $this->addBracketsIfNotNULL($this->standardListEntryAnnotation);
					array_push($strArray,$insertStrStandards);
				}
			}

			// Bearbeitung der Barcodes
			$barcodes = Main::compareArrayContent($this->barcodes,$oldObject->barcodes);
			if (count($barcodes['notIn1st']) > 0){ // wenn Barcodes im alten Objekt enthalten sind, die nicht im neuen Objekt enthalten sind
				foreach($barcodes['notIn1st'] AS $barcode){
					array_push($strArray,'DELETE FROM ' . DB_TAB_PREFIX . 'barcodes WHERE (name="' . $barcode . '" AND articleID = ' . $this->ID . ')');
				}
			}
			if (count($barcodes['notIn2nd']) > 0){ // wenn Barcodes im alten Objekt enthalten sind, die nicht im neuen Objekt enthalten sind
				foreach($barcodes['notIn2nd'] AS $barcode){
					array_push($strArray,'INSERT INTO ' . DB_TAB_PREFIX . 'barcodes SET name="' . $barcode . '", articleID = ' . $this->ID . '');
				}
			}
			
			// Bearbeitung der zugeordneten Kategorien
			$categories = Main::compareArrayContent($this->categories,$oldObject->categories);
			if (count($categories['notIn1st']) > 0){ // wenn Kategorien im alten Objekt enthalten sind, die nicht im neuen Objekt enthalten sind
				foreach($categories['notIn1st'] AS $category){
					array_push($strArray,'DELETE FROM ' . DB_TAB_PREFIX . 'articlesCategories WHERE (articleID="' . $this->ID . '" AND catID = ' . $category['cat_ID'] . ')');
					array_push($strArray,'UPDATE ' . DB_TAB_PREFIX . 'articlesCategories SET sort = sort-1 WHERE (catID = ' . $category['cat_ID'] . ' AND sort >= ' . $category['sort'] . ')');
				}
			}
			if (count($categories['notIn2nd']) > 0){ // wenn Barcodes im alten Objekt enthalten sind, die nicht im neuen Objekt enthalten sind
				foreach($categories['notIn2nd'] AS $category){
					array_push($strArray,'UPDATE ' . DB_TAB_PREFIX . 'articlesCategories SET sort = sort+1 WHERE (catID = ' . $category['cat_ID'] . ' AND sort >= ' . $category['sort'] . ')');
					array_push($strArray,'INSERT INTO ' . DB_TAB_PREFIX . 'articlesCategories SET articleID="' . $this->ID . '", catID = ' . $category['cat_ID'] . ', sort = ' .  $category['sort']);
				}
			}
			ListElement::updateDB_wp($updateStr,$rowLabel,$strArray);
		}


		/**
		* prüft, ob sich auf der Basis der in der Eigenschaft $buyEvery hinterlegten Daten ein früherer Stichtags des nächsten Kaufs ergibt, als bislang in $nextBuy hinterlegt, und passt $nextBuy gegebenenfalls an.
		* @return void
		*/
		public function reexamineNextBuyDate(){
			if (!empty($this->buyEvery)){ // wenn $buyEvery nicht leer ist
				$dateOfToday = date("Y-m-d");
				$shortestNumberOfDaysToNextBuy = Page::selectShortestNumberOfDaysToNextBuy($dateOfToday,$this->buyEvery);
				$dateNextBuyEvery = date("Y-m-d", strtotime($dateOfToday . " + " . $shortestNumberOfDaysToNextBuy . "days" ));
				if($this->nextBuy == "") { // wenn nextBuy leer ist
					$this->nextBuy = $dateNextBuyEvery;
				} else { // wenn $nextBuy nicht leer ist
					// Ermittlung der Daten
					$dateNextBuy = date("Y-m-d", strtotime($this->nextBuy)); // kann auch leer sein
					if ($dateNextBuy > $dateNextBuyEvery) {
						$this->nextBuy = $dateNextBuyEvery;
					}
				}
			}
			return;
		}

		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach unten
		* @return void
		*/
		public function sortDown(){
			$updateStrArray = array();
			$update = '
				UPDATE
					' . DB_TAB_PREFIX . 'articles
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesCategories
					ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID
				SET 
					sort = sort-1 
				WHERE (
					' . DB_TAB_PREFIX . 'articlesCategories.sort = ' . ($this->sort+1) . '
					AND 
					' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . $this->catID . " 
				)";
			array_push($updateStrArray,$update);
								
			// zu ändernden Artikel nach unten schieben
			$update = '
				UPDATE
					' . DB_TAB_PREFIX . 'articlesCategories
				SET 
					sort = sort+1 
				WHERE 
					(articleID = ' . $this->ID . '
					AND 
					' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . $this->catID . '
				)';
			array_push($updateStrArray,$update);
			parent::sortDown_wp($updateStrArray);
		}
		
		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach oben
		* @return void
		*/
		public function sortUp(){
			$updateStrArray = array();
			$update = '
				UPDATE
					' . DB_TAB_PREFIX . 'articles
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesCategories
					ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID	
				SET 
					sort = sort+1 
				WHERE (
					' . DB_TAB_PREFIX . 'articlesCategories.sort = ' . ($this->sort-1) . '
					AND 
					' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . $this->catID . '
				)';
			array_push($updateStrArray,$update);
			
			// zu ändernden Artikel nach oben schieben
			$update = '
				UPDATE
					' . DB_TAB_PREFIX . 'articlesCategories
				SET 
					sort = sort-1 
				WHERE 
					(articleID = ' . $this->ID . '
					AND 
					' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . $this->catID . '
				)';
			array_push($updateStrArray,$update);
			parent::sortUp_wp($updateStrArray);
		}

		/**
		* entfernt aus einem Array von Kategorien-IDs die Kategorien mit den angegebenen IDs aus der Menge der dem Artikel zugeordneten Kategorien.
		* @param array $catIDs Die ID der Kategorie.
		* @return void.
		*/
		public function removeCatIDs($catIDs){
			foreach($catIDs AS $catID){
				$this->removeCatID($catID);
			}
		}
		
		
		/**
		* entfernt eine Kategorie mit der angegebenen ID aus der Menge der dem Artikel zugeordneten Kategorien.
		* @param int $catID Die ID der Kategorie.
		* @return void.
		*/
		protected function removeCatID($catID){
			foreach($this->categories AS $key=>$val){
				if ($val['cat_ID'] == $catID){
					unset($this->categories[$key]);
				}
			}
			$this->categories = array_values($this->categories); // Index neu aufbauen
			return null;
		}
		
		/**
		* entfernt aus einem Array von Barcodes die Barcodes mit dem angegegeben String aus der Menge der dem Artikel zugeordneten Barcodes.
		* @param array $barcodeArray Das Array der Barcodes.
		* @return void.
		*/
		public function removeBarcodes($barcodeArray){
			foreach($barcodeArray AS $barcode){
				$this->removeBarcode($barcode);
			}	
		}	
		
		/**
		* entfernt einen Barcode mit dem angegegeben String aus der Menge der dem Artikel zugeordneten Barcodes.
		* @param string $barcode Der Barcode.
		* @return void.
		*/
		protected function removeBarcode($barcode){
			foreach($this->barcodes AS $key=>$val){
				if ($val == $barcode){
					unset($this->barcodes[$key]);
				}
			}
			$this->barcodes = array_values($this->barcodes); // Index neu aufbauen
			return null;
		}
		
		/**
		* fügt einen neuen Barcode zu der Menge der dem Artikel zugeordneten Barcodes hinzu.
		* @param string $string Der Barcode.
		* @return void.
		*/
		public function addBarcode($string){
			array_push($this->barcodes,$string);
		}
		
		/**
		* Verknüpft den Artikel mit einer neuen Kategorie.
		* @param int $catID Die ID der neuen Kategorie.
		* @param string $catName Die Bezeichnung der neuen Kategorie.
		* @param int $sort Die Sortierreihenfolge.
		* @param string $listName Die Bezeichnung der Liste.
		* @return void.
		*/
		public function addNewCat($catID,$catName,$sort,$listName){
			$category = array(
				"cat_ID" => $catID,
				"cat_name" => $catName,
				"sort" => $sort,
				"list_name" => $listName
			);
			array_push($this->categories,$category);
		}
		
		/**
		* prüft, ob für den entsprechenden Artikel noch Listeneinträge vorhanden sind, und gibt ggf. TRUE zurück.
		* @return boolean.
		*/
		public function hasCorrespondingListEntries(){
			include(ABS_PATH . "dbconnect.php");			
			$selectStr = 'SELECT ID FROM ' . DB_TAB_PREFIX . 'listEntries WHERE articleID = "' . $this->getID() . '"';
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
 			if (mysqli_num_rows($query) > 0){
 				return true;
 			}
 			return false;
		}
	}
?>