<?php
	/** Datei, die die Definition der Klasse ArticlePage enthält. */

	/** 
	* Klasse zur Erstellung der Article-Seite
	*/
	class ArticlePage extends AddArticlePage{
		/** @var int Die ID des angezeigten Artikels. */
		protected $articleID;		
		
				/**
		* Der Konstruktor
		* @param object $article Der Artikel. Falls statt einem Article-Object ein String übergeben wird, wird der String als Seitenbezeichnung verwendet und für die übrigen Werte ein NULL-Object vom Typ Article konstruiert. 
		* @param string $mode Der Seitenmodus. Mögliche Werte: "default" für eine Standardseite und "edit" für eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ ArticlePage.
		*/ 
		public function __construct($article,$mode) {
				
			if (! isset($article) OR is_string($article)){
				if (! isset($article)){
					$articleName = "Artikel <q>ohne Name</q>";
				}
				if (is_string($article)){
					$articleName = $article;
				}
				$article = new Article(NULL,$articleName,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			} else {
				if ($article->getName() == ""){
					$artice->setName("Artikel <q>ohne Name</q>");
				}
			}
			parent::__construct($article->getName(),$mode);
			$this->containsDeletableItems = true;
			$this->articleID = $article->getID();
			$this->indexPage = "article.php?ID=" . $article->getID();
			$this->editPage = "editArticle.php?ID=" . $article->getID() . "&catID=" . $article->getCatID();
			$this->catID = $article->getCatID();
			$this->editPageSubkat = false;
			$this->headerIcons = $this->getHeaderIcons();
		}
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
  		* @param array $array Ein Array optionaler zusätzlicher Angaben, die in den Head eingebunden werden.
		* @return string
 		*/	 
		public function start($array){
			return parent::start_wp($array);
		}
		
		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @return array Array mit den Icon-Angaben
		*/
		protected function getHeaderIcons(){
			if (isset($_GET['catID']) && $_GET['catID'] == 0){
				$pageUp = array(
					"targetPage" => "index.php",
					"iconTyp" => "pageUp",
				);
			} else {
				$pageUp = array(
					"targetPage" => "editCategory.php?ID=" . $this->catID,
					"iconTyp" => "pageUp",
				);
			}

			$leftIcons = array(
				"default" => NULL,
				"edit" => array($pageUp),
			);	
			return parent:: getHeaderIcons_wp($leftIcons,"");
		}
		

		/**
		* Gibt ein Formular zurück, das alle einem Artikel zugeordneten Kategorien mit einer Auswahlbox auflistet.
		* @param string $title Der Titel, der über das Formular gesetzt werden soll. 
		* @param object $article Der Artikel.
		 * @param string $formular1 Die Bezeichnung des Formulars, an die das Formular zur Löschung der Rubrikzuordnung gesendet werden soll.
		 * @param string $formular2 Die Bezeichnung des Formulars, an die das Formular zur Einfügung einer neuen Rubrikzuordnung gesendet werden soll.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.	
		*/
		public function editArticleAssignment($title,$article,$formular1,$formular2,$indent){
			include(ABS_PATH . "dbconnect.php");
			if (!empty($title)){
				$title = $this->linebreak($indent) . '<h2>' . $title . '</h2>';
			}
			$deleteCategories = "";
			$addCategory = "";
			$whereNotIn = "";
						
			if (count($article->getCategories()) > 0){ // wenn dem Artikel mindestens eine Kategorie zugeordnet ist
				$categories = "";
				foreach ($article->getCategories() AS $category){
					$categories .= $this->linebreak($indent+2) . ' <label>' .
         		$this->linebreak($indent+3) . '<input type="checkbox" name="catIDs[]" value="'. $category['cat_ID']. '">' .
         		$this->linebreak($indent+4) . $category['list_name']. ', Rubrik ' . $category['cat_name'] . 
					$this->linebreak($indent+2) . '</label><br />';
				}
				$deleteCategories = $this->startFormular("display: inline;","",$formular1,"","POST","","",$indent) .
				$this->hiddenFields(array("formular" => $formular1),$indent+1) .
				$this->linebreak($indent+1) . '<fieldset>' . 
				$this->linebreak($indent+2) . '<legend>' . 
				$this->linebreak($indent+3) . 'Zuordnung löschen' . 
				$this->linebreak($indent+2) . '</legend>' . 
				$categories .
				$this->linebreak($indent+1) . '</fieldset>' . 
				$this->endFormular($indent) . 		
				$this->submitButtonArea("",$this->submitButton($formular1,"submitButton","","","","Zuordnung löschen"),$indent);
				$whereNotIn = ' WHERE 
					' . DB_TAB_PREFIX . 'categories.ID NOT IN (' . implode(",",$article->getCatIDs()). ')';
			}
			
			
			
			
			$addCategory = $this->startFormular("display: inline;","",$formular2,"","POST","","",$indent) .
				$this->hiddenFields(array("formular" => $formular2),$indent+1) .
				$this->linebreak($indent+1) . '<fieldset>' . 
				$this->linebreak($indent+2) . '<legend>' . 
				$this->linebreak($indent+3) . 'Neue Kategorie zuordnen' . 
				$this->linebreak($indent+2) . '</legend>' . 
				$this->linebreak($indent+2) . '<select class="fullWidth" name="newCatID" size="1">' . 
				$this->linebreak($indent+3) . '<option value="">' . 
				$this->linebreak($indent+4) . '- keine -' .
				$this->linebreak($indent+3) . '</option>';
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'categories.ID,
					' . DB_TAB_PREFIX . 'lists.name AS listName,
					' . DB_TAB_PREFIX . 'categories.name AS categorieName 
				FROM 
					' . DB_TAB_PREFIX . 'categories 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'lists 
						ON 
							' . DB_TAB_PREFIX . 'categories.listID = ' . DB_TAB_PREFIX . 'lists.ID' .
				$whereNotIn . 
				' ORDER BY 
					' . DB_TAB_PREFIX . 'lists.name,' . DB_TAB_PREFIX . 'categories.name ASC';
			$query = mysqli_query($db, $selectStr);
			while($row = mysqli_fetch_object($query)){
				$addCategory .= $this->linebreak($indent+3) . '<option value="' . $row->ID . '"' . '>' . 
					$this->linebreak($indent+4) . $row->listName . ": " . $row->categorieName .
					$this->linebreak($indent+3) . '</option>';				
			}
			$addCategory .= $this->linebreak($indent+2) . '</select>' . 			
				$this->linebreak($indent+1) . '</fieldset>' . 
				$this->endFormular($indent) . 		
				$this->submitButtonArea("",$this->submitButton($formular2,"submitButton","","","","Rubrik zuordnen"),$indent);			

			$result = $title .
			$deleteCategories .			
			$addCategory;
			
			return $result; 
		}
		
		/**
		* Gibt ein Formular zurück, das alle einem Artikel zugeordneten Barcodes mit einer Auswahlbox auflistet.
		* @param string $title Der Titel, der über das Formular gesetzt werden soll. 
		* @param object $article Der Artikel.
		* @param string $formular1 Die Bezeichnung des Formulars, an die das Formular zur Bearbeitung der Barcodes gesendet werden soll.
		* @param string $formular2 Die Bezeichnung des Formulars, an die das Formular zur Löschung der Barcodes gesendet werden soll.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.	
		*/
		public function editBarcodes($title,$article,$formular1,$formular2,$indent){
			if (!empty($title)){
				$title = $this->linebreak($indent) . '<h2>' . $title . '</h2>';
			}
			$result = $title;
			$barcodes = "";
			if (!empty($article->getBarcodes())){
				foreach ($article->getBarcodes() AS $barcode){
					$barcodes .= $this->linebreak($indent+2) . ' <label>' .
         		$this->linebreak($indent+3) . '<input type="checkbox" name="barcodes[]" value="'. $barcode. '">' .
         		$this->linebreak($indent+4) . $barcode . 
					$this->linebreak($indent+2) . '</label><br />';
				}
				$result .= $this->startFormular("display: inline;","",$formular1,"","POST","","",$indent) .
				$this->hiddenFields(array("formular" => $formular1),$indent+1) .
				$this->linebreak($indent+1) . '<fieldset>' . 
				$this->linebreak($indent+2) . '<legend>' . 
				$this->linebreak($indent+3) . 'Zuordnung löschen' . 
				$this->linebreak($indent+2) . '</legend>' . 
				$barcodes .
				$this->linebreak($indent+1) . '</fieldset>' . 
				$this->endFormular($indent) . 	
				$this->submitButtonArea("",$this->submitButton($formular1,"submitButton","","","","Zuordnung löschen"),$indent);
			}
			
			$result .= $this->startFormular("display: inline;","",$formular2,"","POST","","",$indent) .
			$this->hiddenFields(array("formular" => $formular2),$indent+1) .
			$this->linebreak($indent+1) . '<fieldset>' . 
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Neuen Barcode hinzufügen' . 
			$this->linebreak($indent+2) . '</legend>' . 
			$this->linebreak($indent+2) . '	<input type="text" name="newBarcode" class ="fullWidth">' .  
			$this->linebreak($indent+1) . '</fieldset>' . 
			$this->endFormular($indent) . 	
			$this->submitButtonArea("",$this->submitButton($formular2,"submitButton","","","","hinzufügen"),$indent);
			return $result; 
		}
		
		

		
		/**
  		* Gibt eine Meldung zurück, dass der Artikel nicht gelöscht werden kann, weil sich noch Einträge des Artikels auf einer Liste befinden.
  		* @param int $indent Die Einzugtiefe.
		* @return string Der Rückgabetext.
 		*/	
		public function cannotBeDeleted($indent){
			return $this->linebreak($indent) .  '<p>Dieser Artikel kann nicht gelöscht werden, da sich noch Einträge des Artikels auf einer Liste befinden.</p>' .
			$this->linebreak($indent) .  '<p>Entfernen Sie zunächst sämtliche Listeneinträge des entsprechenden Artikels und kehren Sie dann auf diese Seite zurück.</p>' .
			$this->startFormular(NULL,"","dontDelete","","",NULL,NULL,$indent+2) .
			$this->hiddenFields(array("formular" => "dontDelete"),$indent+2) . 
			$this->endFormular($indent+2) . 
			$this->submitButtonArea(NULL,$this->submitButton("dontDelete","submitButton","","","","zurück"),$indent);
		}



		/**
		* Gibt ein Formular zurück, mit dem sich ein Barcode einem Artikel zuordnen lässt.
		* @param string $barcode Der Barcode.
		* @param int $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		public function addBarcode($barcode,$indent){
			$result = $this->startFormular("display: inline;","","addBarcode","","POST","","",$indent) . 
			$this->hiddenFields(array("formular" => "checkSelectedArticle"),$indent+1) .
			$this->linebreak($indent) . "<p>Bislang ist der Barcode <q>" . $barcode . "</q> noch keinem Artikel zugeordnet.</p>" . 
			$this->linebreak($indent) . "<p>Welchem Artikel möchten Sie den Barcode zuordnen?</p>" . 
			$this->selectArticleName(NULL,$indent) . 
			$this->endFormular($indent) . 		
			$this->submitButtonArea("",$this->submitButton("addBarcode","submitButton","","","","zuordnen"),$indent);			;
			return $result;
		}
		
		/**
		* Prüft die Eingabe eines neuen Artikels auf ihre Gültigkeit, trennt, sofern notwendig, Namen und Anmerkungen voneinander und gibt den rekonstruierten Artikel zurück.
		* Sollte kein Name eingegen worden sein oder die Rekonstruction misslingen, wird ein Formular mit einer entsprechenden Fehlermeldung zurückgegeben.
		* @param object $article Ein Objekt der Klasse Article.
		* @param int $indent Die Einzugtiefe.
		* @return object Der rekonstruierte Artikel.
		*/
		public function checkNewArticle($article,$indent){
			if ($article->getName() == ""){ // wenn kein Artikelname angegeben ist, auf vorherige Seite verweisen	
				echo $this->goBack("Bitte geben Sie eine Artikelbezeichnung ein.",NULL,"","","",$indent);
			} 
			else { // wenn Artikelname angegeben ist, weitere Überprüfung; ACHTUNG: $articleName enthält ggf. angefügte ArticleAnnotation
				$reconstructed = $this->reconstructArticleName($article,"Article"); // Trennung von articleName und articleAnnotation;
				if ($reconstructed == false){ // falls Rekonstruktion nicht gelungen ist // der Artikelname bislang nicht in der  Tabelle Articles eingetragen ist

					echo $this->goBack('In der Datenbank existiert kein Artikel mit der Bezeichnung <q>' . $article->getName() . '</q>. Bitte wählen Sie <q>zurück</q> und geben Sie einen gültigen Artikelnamen ein.',NULL,NULL,NULL,NULL,$indent);			
				} else { // wenn Rekonstruktion gelungen ist / der Artikelname der Tabelle Articles eingetragen ist
				return $reconstructed;
				}
			}
		}
		
		/**
		* behandelt die erfolgreiche Rekonstruktion eines Artikels.
		* @param $article Der rekonstruierte Artikel.
		* @param string $returnURL Seite, auf die nach einer erfolgreichen Eintragung des Artikels zurückverwiesen werden soll.
		* @param $indent Der Einzug.
		* @return string Der Quelltext zur Behandlung der erfolgreichen Rekonstruktion.
		*/
		/*
		private function reconstructionSucceeded($article,$returnURL,$indent) {
			$listEntry = new ListEntry(
				NULL, // ID,
				Main::getPrefAmountIfNULL($article->getStandardAmount()),
				Main::getPrefStandardMeasureIDIfNULL($article->getStandardMeasureID()),
				NULL, // $measureName,
				$article->getID(),
				$article->getName(),
				$article->getAnnotation(),
				$article->getStandardListEntryAnnotation(),
				NULL, //listID,
				Main::getPrefStandardPriorityIDIfNULL($article->getStandardPriorityID()),
				NULL, // $priorityName
				NULL // inBasket
				);
				echo $this->startFormular(NULL, "newListEntry.php", "autosubmit", NULL, NULL, NULL, NULL, $this->getContentIndent()) . 
				$this->hiddenFields(array("listEntry" => $this->objectToString($listEntry)) ,$this->getContentIndent()) .
				$this->endFormular($this->getContentIndent());
			//	$this->submitFormular("autosubmit");
		}
		*/
		


	}
?>