<?php
	/** Datei, die die Definition der Klasse Category enthält. */
	
	/**
	* Klasse zur Erstellung einer Kategorie 
	*/
	class Category extends SortableListElement {
		/** @var int Die ID der zugehörigen Liste. */		
		private $listID;
		/** @var array Array der zugehörigen Kategorien	. */
		private $articles;

				
		
		/**
		* Der Konstruktor
		* @param int $ID Die ID des Objektes.
		* @param string $name Der Name des Objektes.
		* @param int $listID Die ID der Liste.
		* @param string $listName Der Name der Liste.
		* @param int $sort Die Position des Objektes in der Sortierreihenfolge.
		* @return object Ein Objekt vom Typ Category.
		*/ 			
		public function __construct($ID,$name,$listID,$listName,$sort) {
			parent::__construct($ID,$name,NULL,$sort);
			include(ABS_PATH . "dbconnect.php");
			$this->listID = $listID;
			$this->listName = $listName;
			$this->articles = NULL;
			$this->prefix = "cat";
			$this->table = DB_TAB_PREFIX . 'categories';
		}
			
	
		/**
		* Konstruiert ein Objekt auf der Basis seiner ID
		* @param int $ID Die ID des Objektes.
		* @return object Ein Objekt vom Typ Category.
		*/ 
		public function constructByID($ID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = 'SELECT 
				' . DB_TAB_PREFIX . 'categories.listID,
				' . DB_TAB_PREFIX . 'lists.name AS listName,
				' . DB_TAB_PREFIX . 'categories.name,
				' . DB_TAB_PREFIX . 'categories.sort 
			FROM 
				' . DB_TAB_PREFIX . 'categories 
			LEFT JOIN
				' . DB_TAB_PREFIX . 'lists
				ON
					' . DB_TAB_PREFIX . 'categories.listID = ' . DB_TAB_PREFIX . 'lists.ID
			WHERE 
				' . DB_TAB_PREFIX . 'categories.ID = ' . intval($ID);
			$query = mysqli_query($db, $selectStr);
			if (mysqli_num_rows($query) > 0){
				$row = mysqli_fetch_object($query);
				return new Category($ID,$row->name,$row->listID,$row->listName,$row->sort);
			}
			return NULL;
		}
		
		
		/**
		* Setter der Variable $listID
		* @param int $listID Die ID der Liste, der die Kategorie zugeordnet ist
		* @return void
		*/ 
		public function setMeasureID($listID){
			$this->listID = $listID;
		}
		
		/**
		* Gibt die ID der Liste zurück, der die Kategorie zugeordnet ist.
		* @return int Die ID der Liste.
		*/ 
		public function getListID(){
			return $this->listID;
		}
		
		/**
		* Gibt den Namen der Liste zurück, der die Kategorie zugeordnet ist.
		* @return string Der Name der Liste.
		*/ 
		public function getListName(){
			return $this->listName;
		}
		
		
		/**
		* Getter der Variable $articles
		* @return array Inhalt der Variable $articles.
		*/ 
		public function getArticles(){
			return $this->articles;
		}
		
		/**
		* prüft, ob ein Wert bereits als Name in der Datenbank existiert existiert, und gibt entsprechend true oder false zurück.
		* Für diese Funktion müssen die Eigenschaften table und name implementiert sein.
		* @param string $name Der Name, der auf Existenz geprüft werden soll.
		* @return bool Das Ergebnis, ob der Name bereits in der Datenbank existiert.
		*/
		public function nameAlreadyExists($name){
			return parent::nameAlreadyExists_wsa($name," AND `listID` = '" . $this->listID . "'");
		}
		
		/**
		* fügt der in den Objekteigenschaften genannten $table einen weiteren Datenbankeintrag hinzu und passt die Sortierung entsprechend an.
		* Für die korrekte Funktion muss die Eigenschaft $table definiert sein.
		* @return void
		*/
		public function addToDB(){
			$updateStr = "
				UPDATE " .
					$this->table . "
				SET 
					sort = sort+1 
				WHERE 
					listID = '" . $this->listID . "' AND
					sort >= " . $this->sort; // Platz für die Liste schaffen			
			
			$insertStr = "
				INSERT INTO " .
					$this->table . "
				SET
					listID = '" . $this->listID . "', 
					name = '" . $this->name . "', 
					sort = " . $this->sort;	
			return parent::addToDB_wp($updateStr,$insertStr,"","");
		}
		
		/**
		* aktualisiert den entsprechenden Datenbankeintrag.
		* @return void
		*/
		public function updateDB(){
			include(ABS_PATH . "dbconnect.php");
			$strArray = array();
			$oldObject = $this->constructByID($this->ID);
			$oldObject->addSubCats();
			$rowLabel = "catID";
			
			$updateStr = '
				UPDATE
					' . DB_TAB_PREFIX . 'categories
				SET
					name = ' . $this->addBracketsIfNotNULL($this->name) . '
				WHERE ID = ' . $this->ID;		
			
			
			// Zuordnung der Artikel zu den Rubriken bearbeiten
			$articles = Main::compareArrayContent($this->articles,$oldObject->articles);
			foreach($articles['notIn1st'] AS $article){
				$deleteStr = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesCategories WHERE (articleID = ' . $article->ID . ' AND catID = ' . $article->catID . ')';
				array_push($strArray,$deleteStr);
				$updateStr1 = 'UPDATE
						' . DB_TAB_PREFIX . 'articlesCategories
						SET 
						sort = sort-1 
					WHERE (
						sort >= ' . $article->sort . '
						AND 
						catID = ' . $article->catID . ')';
				array_push($strArray,$updateStr1);
			}
			
			foreach($articles['notIn2nd'] AS $article){
				$updateStr2 = 'UPDATE
						' . DB_TAB_PREFIX . 'articlesCategories
					SET 
						sort = sort+1 
					WHERE (
						sort >= ' . $article->getSort() . '
						AND 
						catID = ' . $article->getCatID() . ')';
					array_push($strArray,$updateStr2);
				$insertStr = '
					INSERT INTO
						' . DB_TAB_PREFIX . 'articlesCategories
					SET
						articleID = ' . Main::addBracketsIfNotNULL($article->getID()) . ',
						catID = ' . Main::addBracketsIfNotNULL($article->getCatID()) . ',
						sort = ' . Main::addBracketsIfNotNULL($article->getSort());
					array_push($strArray,$insertStr);
			}
			return parent::updateDB_wp($updateStr,$rowLabel,$strArray);
		}
		
		/**
		* Sucht alle Artikel einer Kategorie und gibt sie als Array zurück.
		* @return array Array der zugehörigen Kategorien
		*/
		public function addSubCats(){
			if (!empty($this->ID)){
				/* Sucht alle Warenkategorien einer Liste und gibt sie in einem Array zurück */
				include(ABS_PATH . "dbconnect.php");					
				$selectStr = '
					SELECT 
						' . DB_TAB_PREFIX . 'articles.ID,
						' . DB_TAB_PREFIX . 'articlesCategories.sort
					FROM 
						' . DB_TAB_PREFIX . 'articlesCategories 
					LEFT JOIN 
						' . DB_TAB_PREFIX . 'articles 
						ON 
							' . DB_TAB_PREFIX . 'articlesCategories.articleID = ' . DB_TAB_PREFIX . 'articles.ID 
					WHERE catID  = "' . intval($this->ID) . '" 
					ORDER BY 
						sort ASC';
				$query = mysqli_query($db, $selectStr);
				$result = array();

				while($row = mysqli_fetch_object($query)) {
					$article = Article::constructByID($row->ID);
					$article->setSort($row->sort);
					$article->setCatID($this->ID);
					array_push($result,$article);
				}
				$this->articles = $result; 

			}
		}
		
		/**
		* fügt einen neuen Artikel zu der Menge der dem Kathegorie zugeordneten Artikel hinzu.
		* @param object $article Der Artikel.
		* @return void.
		*/
		public function addArticle($article){
			array_push($this->articles,$article);
		}
		
		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach unten
		* @return void
		*/
		public function sortDown(){
			$updateStrArray = array();
			$updateStr = 'UPDATE ' . $this->table . ' SET sort = sort-1 WHERE (sort = ' . ($this->sort+1) . ' AND listID = ' . $this->getListID() . ')';
			array_push($updateStrArray,$updateStr);

				// zu ändernde Subkategorie nach unten schieben
			$updateStr = "UPDATE " . $this->table . "	SET sort = sort+1 WHERE ID = " . $this->ID; // Sortierung aufrücken
			array_push($updateStrArray,$updateStr);
			parent::sortDown_wp($updateStrArray);
		}
		
		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach oben
		* @return void
		*/
		public function sortUp(){
			$updateStrArray = array();
			$updateStr = 'UPDATE ' . $this->table . ' SET sort = sort+1 WHERE (sort = ' . ($this->sort-1) . ' AND listID = ' . $this->getListID() . ')';
			array_push($updateStrArray,$updateStr);

			// zu ändernde Subkategorie nach oben schieben
			$updateStr = 'UPDATE ' . $this->table . ' SET sort = sort-1 WHERE ID = ' . $this->ID; // Sortierung aufrücken
			array_push($updateStrArray,$updateStr);
			parent::sortUp_wp($updateStrArray);
		}
		
		
		/**
		* entfernt das Objekt aus der in den Objekteigenschaften genannten $table
		* @return void
		*/
		public function deleteFromDB(){
			// sorgt dafür, dass zusätzlich die Kategorien der Liste gelöscht werden.
			$updateStr = 'UPDATE ' . DB_TAB_PREFIX . 'categories SET sort = sort-1 WHERE ( sort >= ' . $this->sort . ' AND listID = ' . $this->listID . ')'; 
			
			$strArray = array();
			// Lösche die Zuordnung der Artikel zu der entsprechenden Kategorie (nicht die Artikel selbst!) 
			$deleteArticlesCategories = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesCategories WHERE catID = ' . $this->ID; 
			array_push($strArray,$deleteArticlesCategories);
			parent::deleteFromDB_wp($updateStr,NULL,NULL,$strArray);
		}
	}
?>