<?php
	/** Datei, die die Definition der Klasse CategoryPage enthält. */
	/** 
	* Klasse zur Erstellung der Category-Seite
	*/
	class CategoryPage extends AddArticlePage{
		/** @var int Die ID der angezeigten Kategorie. */
		protected $catID;		
		/** @var int Die ID der Liste, der die angezeigte Kategorie zugeordnet ist. */
		protected $listID;		
		/**
		* Der Konstruktor.
		* @param object $category Die Kategorie als Objekt.
		* @param string $mode Der Modus. Mögliche Werte sind "default" und "edit". "Default" gibt eine Standardseite aus, "edit" eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ CategoryPage.
		*/ 
		public function __construct($category,$mode) {
			parent::__construct($category->getName(),$mode);
			$this->containsDeletableItems = true;
			$this->catID = $category->getID();
			$this->indexPage = "category.php?ID=" . $category->getID();
			$this->editPage = "editCategory.php?ID=" . $category->getID();
			$this->editPageSubkat = "editArticle.php";
			$this->listID = $category->getListID();
			$this->headerIcons = $this->getHeaderIcons();
		}
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
		* @return string
 		*/	 
		public function start(){
			$searchData = '<script src="jquery-3.3.1.min"></script>
		<script src="bootstrap.min.js"></script>
		<script src="typeahead.min.js"></script>
		<script>
			$(document).ready(function(){
				$(\'input.typeahead\').typeahead({
					name: \'typeahead\',
					remote:\'search.php?key=%QUERY&withoutCatID=' . $this->catID. ' \',
					limit : 10
				});
			});
		</script>
		<script>
			$(document).ready(function(){
				$(\'input.typeaheadConvert\').typeahead({
					name: \'typeaheadConvert\',
					remote:\'search.php?key=%QUERY&mode=convertArticle\',
					limit : 10
				});
			});
		</script>';
			$headData = array($searchData);
			return parent::start_wp($headData);
		}

		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @return array Array mit den Icon-Angaben
		*/

		protected function getHeaderIcons(){
			$pageUp = array(
				"targetPage" => "editList.php?ID=" . $this->listID,
				"iconTyp" => "pageUp",
			);
			$leftIcons = array(
				"default" => NULL,
				"edit" => array($pageUp),
			);	
			
			return parent::getHeaderIcons_wp($leftIcons,"");
		}
		
		/**
		* Gibt ein Formular zurück, dass das Hinzufügen eines Articles zu einer Kategorie erlaubt.
		* @param $title Der Titel, der über das Formular gesetzt wird.
		* @param string $anchor Die Sprungmarke zur Überschrift. Ist sie angegeben, kann mit einem Link direkt auf die entsprechende Überschrift gesprungen werden.
		* @param $article Ein Artikel-Objekt.
		* @param $articles Ein Array der Article.
		* @param $indent Der Einzug.
		* @return string Der HTMl-Quelltext zur Ausgabe des Formulars.
		*/
		public function addArticleToCat($title,$anchor,$article,$articles,$indent){

			if (!empty($article) AND !empty($article->getName())){
				$placeholder = ' value="'. $article->getName() . '"';
			}
			else {
				$article = new Article(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
				$placeholder = ' placeholder="Artikelbezeichnung"';
			}
			if (empty($anchor)){
				$h2ID = "";
			} else {
				$h2ID = ' id="' . $anchor . '"';
			}
			$result = $this->linebreak($indent) . '<h2' . $h2ID . '>' . $title . '</h2>' .
			$this->startFormular("","","addArticleToCat","","","","",$indent) . 
			$this->hiddenFields(array("formular" => "addArticleToCat"),$indent + 2) .	
			$this->linebreak($indent+1) . '<fieldset>' .
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Bezeichnung'. 
			$this->linebreak($indent+2) . '</legend>' . 
			$this->linebreak($indent+2) . '<input type="text" name="article_name" class="typeahead tt-query" autocomplete="off" spellcheck="false"' . $placeholder . '>' . 
			$this->linebreak($indent+1) . '</fieldset>' . 
			$this->selectSubCats($article,$articles,$indent) .
			$this->endFormular($indent) .
			$this->submitButtonArea("",$this->submitButton("addArticleToCat","submitButton","","","","hinzufügen"),$indent);
			return $result;
		}
		
		/**
		* Prüft die Eingabe eines neuen Artikels auf ihre Gültigkeit und trennt, sofern notwendig, Namen und Anmerkungen voneinander.
		* @param object $category Die Kategorie, der der Artikel hinzugefügt werden soll.
		* @param object $article Ein Objekt der Klasse Article.
		* @param int $indent Die Einzugtiefe.
		* @return void.
		*/
		public function checkNewArticleForCat($category,$article,$indent){
			if (empty($article->getName())){ // wenn kein Artikelname angegeben ist, auf vorherige Seite verweisen	
				echo $this->goBack("Bitte geben Sie eine Artikelbezeichnung ein.",array("article" => Main::objectToString($article), "formular" => "addedArticleWithEmptyNameToCat"),"","","",$indent);
			} 
			else { // wenn Artikelname angegeben ist, weitere Überprüfung; ACHTUNG: $articleName enthält ggf. angefügte ArticleAnnotation
				$reconstructed = $this->reconstructArticleName($article,"Article"); // Trennung von articleName und articleAnnotation;
				if ($reconstructed == false){ // falls Rekonstruktion nicht gelungen ist // der Artikelname bislang nicht in der  Tabelle Articles eingetragen ist
					echo $this->goBack('Bislang existiert kein Artikel mit der Bezeichnung <q>' . $article->getName() . '</q>. Bitte wählen Sie <q>zurück</q> und geben Sie einen gültigen Artikelnamen ein.',array("article" => Main::objectToString($article), "formular" => "articleCouldNotBeConstructed"),"","","",$indent);
				} else { // wenn Rekonstruktion gelungen ist / der Artikelname der Tabelle Articles eingetragen ist
				$category->addArticle($reconstructed);
				$category->updateDB();
				}
			}
		}	
	}
?>