<?php

	/** Datei, die die Definition der Klasse IndexPage enthält. */
	
	/** 
	* Index-Seite
	*/
	class IndexPage extends Page{
		
		/**
		* Der Konstruktor
		* @param string $mode Der Modus. Mögliche Werte sind "default" und "edit". "Default" gibt eine Standardseite aus, "edit" eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ IndexPage.
		*/ 
		public function __construct($mode) {
			include(ABS_PATH . "dbconnect.php");
			$selectTitle = '
				SELECT 
					' . DB_TAB_PREFIX . 'prefs.name
				FROM 
					' . DB_TAB_PREFIX . 'prefs 
				WHERE
					' . DB_TAB_PREFIX . 'prefs.ID = 1';
			$query = mysqli_query($db, $selectTitle);
			$row = mysqli_fetch_object($query);
			parent::__construct($row->name,$mode);
			$this->containsDeletableItems = false;
			$this->indexPage = "index.php";
			$this->editPage = "editIndex.php";
			$this->editPageSubkat = "editList.php";
			$this->headerIcons = $this->getHeaderIcons();
		}
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
		* @return string
 		*/	 
		public function start(){
			return parent::start_wp("");
		}
	
		/**
  		* gibt den übergebenen Index aus, wobei nur die Listen ausgegeben werden, die ungekaufte Artikel oder noch im Warenkorb liegende Artikel verzeichnen.
  		* @param object $index Ein Objekt der Klasse Index.
  		* @param int $indent Die Tiefe des Einzugs nach dem Zeilenumbruch.
		* @return string Der Quellcode.
 		*/
		public function showLists($index,$indent){
			$result = $this->linebreak($indent) . '<ul>';
			$numberOfLists = 0;
			foreach($index->getLists() as $list){
				if ($list->getStatusOfListEntries()['notInBasket'] > 0 OR $list->getStatusOfListEntries()['inBasket'] > 0){ // Ausgabe erfolgt nur bei mindestens einem zu kaufenden Artikel
				$numberOfLists++;
				$result .= $this->linebreak($indent+1) . '<li class="' . $this->formatListEntry($list) . '">' . 
				$this->linebreak($indent+2) . '<a href="list.php?ID=' . $list->getID() . '">' . 
				$this->linebreak($indent+3) . $list->getName() . ' (' . $list->getStatusOfListEntries()['notInBasket']. ')' .
				$this->linebreak($indent+2) . '</a>' .
			 	$this->linebreak($indent+1) . '</li>';
				}
			} 
			$result .= $this->linebreak($indent) . '</ul>';
			if ($numberOfLists > 0){
				return $result;
			} else {
				return $this->linebreak($indent) . "Die Einkaufsliste ist leer.";
			}
			
		}
		
		
		
		/**
  		* gibt einen String aus, der der Formatierung eines Listeneintrages dient.
  		* Sollte die Liste Artikel enthalten, die noch nicht im Warenkorb liegen, wird dem String "articlesNotInBasket" hinzugefügt. Im Falle dringender Artikel wird "urgent" hinzugefügt 
  		* @param object $list Die entsprechende Liste als Object
		* @return string
 		*/
		protected function formatListEntry($list){
			if (!empty($list)){
				$listStatus = $list->getStatusOfListEntries();
				$priority = "";
				$result = array();
				if ($listStatus['notInBasket'] > 0){
					array_push($result,"notInBasket");
				} 
				if ($listStatus['lowPriority'] > 0){
					$priority = "low";
				}
				if ($listStatus['standardPriority'] > 0){
					$priority = "standard";
				}
				if ($listStatus['highPriority'] > 0){
					$priority = "urgent";
				}
				if ($priority != ""){
					array_push($result,$priority);
				}
				$result = implode(" ",$result);
				return $result;
			} else {
				return NULL;
			}		
		}
		
		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @return array Array mit den Icon-Angaben
		*/
		protected function getHeaderIcons(){
			$addToTrolley = array(
				"targetPage" => "newListEntry.php",
				"arrayHiddenFields" => array(
					"returnURL" => "index.php",
				),
				"iconTyp" => "addToTrolley",
			);	
			$leftIcons = array(
				"default" => array($addToTrolley),
				"edit" => NULL,
			);	
			return parent:: getHeaderIcons_wp($leftIcons,"");
		}
		
		/**
		* gibt alle Maßangaben in einem Array aus Measure-Objekten zurück.
		* @return array Das Array mit den Maßangaben 
		*/
		public function getMeasures(){
			include(ABS_PATH . "dbconnect.php");
			$result = array();
			$selectStr = '
				SELECT 
					ID,
					name,
					sort
				FROM
					' . DB_TAB_PREFIX . 'measures
				ORDER BY
					sort';
			$query = mysqli_query($db, $selectStr);
			while($row = mysqli_fetch_object($query)){
				$measure = new Measure($row->ID,$row->name,$row->sort);
				array_push($result,$measure);
			}
			return $result;
		}
		
		/**
		* liefert ein Formular mit dem Namen updateArticleStandards, um die Standardeinstellungen für neue Artikel zu bearbeiten.
		* @param string $title Die Überschrift (h2), die dem Formular vorangestellt wird.
		* @param obj Index Der Index.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		public function editStandardsForArticles($title,$index,$indent){
			$result = 
				$this->linebreak($indent) . '<h2>' .
				$this->linebreak($indent+1) . $title . 
				$this->linebreak($indent) . '</h2>' . 
				$this->startFormular("display: inline;","","updateArticleStandards","","POST","","",$indent) . 
				$this->hiddenFields(array("formular" => "updateArticleStandards"),$indent+1) .			
				$this->printStandardAmount($index->getStandardAmount(),$indent) .
				$this->printStandardMeasureID($index->getStandardMeasureID(),$indent) .
				$this->printPriorityID($index->getStandardPriorityID(),$indent) .
				$this->endFormular($indent) .
				$this->submitButtonArea(NULL,$this->submitButton("updateArticleStandards","submitButton","Submit","submit","","ändern"),$indent);
			return $result;
		}
		
		/**
		* liefert ein Input-Feld, um die Standardmenge für neue Artikel zu bearbeiten.
		* @param int $standardAmount Die Standardmenge.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function printStandardAmount($standardAmount,$indent){
			include(ABS_PATH . "dbconnect.php");
			$result = 
			$this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="standardAmount">' . 
			$this->linebreak($indent+2) . 'Standardmenge für neue Artikel: ' . 
			$this->linebreak($indent+1). '</label>' . 
			$this->linebreak($indent+1). '<input type="text" id="standardAmount" name="standardAmount" placeholder="' . $standardAmount. '">' .
			$this->linebreak($indent) . "</p>";

			return $result;
		}
		
		/**
		* liefert ein SELECT-Feld, um die Standardpriorität für neue Artikel zu bearbeiten.
		* @param int $standardMeasureID Die ID der bisherigen Standardmaßangabe.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function printStandardMeasureID($standardMeasureID,$indent){
			include(ABS_PATH . "dbconnect.php");
			$result = 
			$this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="standardMeasureID">' . 
			$this->linebreak($indent+2) . 'Standardmaß für neue Artikel: ' . 
			$this->linebreak($indent+1). '</label>' . 
			$this->linebreak($indent+1). '<select name="standardMeasureID" id="standardMeasureID" size="1">';
			
			//	sämtliche StandardPriorityIDs ermitteln		
			$selectStr = '
				SELECT 
					ID,
					name
				FROM 
					' . DB_TAB_PREFIX . 'measures';
			$query = mysqli_query($db, $selectStr);
			while($measure = mysqli_fetch_object($query)){
				$result .= $this->linebreak($indent+2) . "<option value=\"" . $measure->ID . "\"";
				if ($measure->ID == $standardMeasureID) {
					$result .= " selected = \"selected\"";
				}
				$result .= ">" . $measure->name . "</option>";
			}
			$result .= $this->linebreak($indent+1) . "</select>" .
			$this->linebreak($indent) . "</p>";

			return $result;
		}
		
		/**
		* liefert ein SELECT-Feld, um die Standardpriorität für neue Artikel zu bearbeiten.
		* @param int $standardPriorityID Die ID der bisherigen Standardpriorität.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		private function printPriorityID($standardPriorityID,$indent){
			include(ABS_PATH . "dbconnect.php");
			$result = 
			$this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="standardPriorityID">' . 
			$this->linebreak($indent+2) . 'Standardpriorität für neue Artikel: ' . 
			$this->linebreak($indent+1). '</label>' . 
			$this->linebreak($indent+1). '<select name="standardPriorityID" id="standardPriorityID" size="1">';
			
			//	sämtliche StandardPriorityIDs ermitteln		
			$selectStr = '
				SELECT 
					ID,
					name
				FROM 
					' . DB_TAB_PREFIX . 'priority';
			$query = mysqli_query($db, $selectStr);
			while($priority = mysqli_fetch_object($query)){
				$result .= $this->linebreak($indent+2) . "<option value=\"" . $priority->ID . "\"";
				if ($priority->ID == $standardPriorityID) {
					$result .= " selected = \"selected\"";
				}
				$result .= ">" . $priority->name . "</option>";
			}
			$result .= $this->linebreak($indent+1) . "</select>" .
			$this->linebreak($indent) . "</p>";

			return $result;
		}
	}
?>
