<?php
	/** Datei, die die Definition der Klasse ListElement enthält. */
	
	/**
	zentrale Funktionen, die zur Objekterstellung notwendig sind
	*/
	class ListElement extends Main{
		/** @var string Die Anmerkungen zum Objekt. */		
		protected $annotation;
		/** @var string Das Präfix des Objektes wird verwendet, um die Attribute des Objektes eindeutig einem Objekttyp zuordnen zu können und wird zum Beispiel im Zusammenspiel mit der Funktion hiddenFields verwendet */
		protected $prefix;
		/** @var bool Angabe, ob sich das Objekt sortieren lässt */
		protected $sortable;
		/** @var string Bezeichnung der MySQL-Tabelle, in der die Informationen zum Objekt gespeichert sind */
		protected $table;
		/** @var string Bezeichnung der MySQL-Tabelle, in der die Anmerkungen zum Objekt gespeichert sind */
		protected $tableAnnotation;
		
		/**
		* Der Konstruktor
		* @param string $name Der Name des Objektes.
		* @param string $annotation Die Anmerkungen zum Objekt.
		* @return object Ein Objekt vom Typ ListElement.
		*/ 
		public function __construct($name,$annotation) {
			parent::__construct($name);
			$this->annotation = $annotation;
			$this->prefix = "obj";
			$this->sortable = false;
			$this->table = NULL;
			$this->tableAnnotation = NULL;
		}
		
		/**
		* Konstruiert ein Objekt auf der Basis seiner ID.
		* *ACHTUNG*: Das Attribut $listID muss, sofern benötigt, in diesem Fall nachträglich mit setListID() gesetzt werden!
		* @param int $ID Die ID des Objektes.
		* @return object Ein Objekt vom Typ listElement.
		*/ 	
		public function constructByID($ID){
			return new ListElement(NULL,NULL);
		}
		
				
		/**
		* Setter der Variable $annotation.
		* @param string $annotation Die Anmerkungen zum konkreten Objekt
		* @return void
		*/ 
		public function setAnnotation($annotation){
			$this->annotation = $annotation;
		}	
		
		/**
		* Gibt die Anmerkungen zum konkreten Objekt zurück.
		* @return string Die Anmerkungen zum Objekt.
		*/ 
		public function getAnnotation(){
			return $this->annotation;
		}
		
		/**
		* Setter der Variable $prefix
		* @param string $prefix Das Präfix
		* @return void
		*/ 
		public function setPrefix($prefix){
			$this->prefix = $prefix;
		}

		/**
		* Getter der Variable $prefix
		* @return string Inhalt der Variable $prefix.
		*/ 
		public function getPrefix(){
			return $this->prefix;
		}
		
		/**
		* Getter der Variable $tableAnnotation
		* @return string Inhalt der Variable $tableAnnotation.
		*/ 
		public function getTableAnnotation(){
			return $this->tableAnnotation;
		}

		/**
		* Ersetzt den Teilstring "[[thisID]]" durch die ID des Objektes.
		* Bei der Übergabe von MYSQL-Strings an übergeordnete Klassen kann es passieren, dass die ID des Objektes noch nicht feststeht bzw. noch 0 ist. Mit dieser Funktion kann die neue ID auch in Strings eingefügt werden, die aus Kindfunktionen an Vaterfunktionen übergeben werden.
		* @param string $string Der String mit dem Teilstring "[[thisID]]".
		* @return string Ein String, bei dem der Teilstring "[[thisID]]" durch die ID des Objektes ersetzt ist.
		*/
		protected function insertNewID($string){
			$result = str_replace("[[thisID]]", $this->ID, $string);
			return $result;
		}

		/**
		* fügt der in den Objekteigenschaften genannten $table einen weiteren Datenbankeintrag hinzu.
		* @param string $updateStr Der String zur Neusortierung, die durch das folgende Einfügen eines neuen Objektes notwendig ist. Achtung: Die Neusortierung erfolgt nur, falls das Objekt die Eigenschaft sortable hat.
		* @param string $insertStr Der String zur Einfügung des Objektes in die als "table" definierte MySQL-Tabelle. Falls leer, wird nur der Name in die entsprechende Tabelle eingetragen. 
		* @param string $annotationStr Der String zur Einfügung entsprechender Anmerkungen in die Tabelle, die als Objekteigenschaft "tableAnnotation" eingetragen ist. Wird nur ausgeführt, wenn die Eigenschaft "tableAnnotation" definiert ist und für das Objekt auch tatsächlich eine "annotation" als Eigenschaft eingetragen ist.
		* @param array $strArray Ein Array aus Strings mit optionalen, zusätzlichen MySQL-Befehlen, die abschließend ausgeführt werden. Falls nur ein Befehl übermittelt werden soll, kann der Parameter auch vom Typ String sein.
		* @return void
		*/
		public function addToDB_wp($updateStr,$insertStr,$annotationStr,$strArray){
			include(ABS_PATH . "dbconnect.php");			
			// Ausführung $updateStr und $insertStr
			if ($this->table != ""){ // Wenn Tabelle definiert ist
				if (!is_null($updateStr)){
					mysqli_query($db, $updateStr);
				}
				if (is_null($insertStr)){ // wenn $insertStr nicht als Parameter übergeben
					$insertStr = 'INSERT INTO ' . $this->table . ' SET name = "' . $this->name . '"';
				}
				mysqli_query($db,$insertStr);
				$this->setID($db->insert_id);
			}
			
			// Ausführen des $annotationStr, sofern $tableAnnotation definiert, Annotation nicht NULL und $annotationStr vorhanden ist
			if ($this->tableAnnotation != "" AND $this->getAnnotation() != "" AND !is_null($annotationStr)){
				mysqli_query($db, $this->insertNewID($annotationStr));
			} 
			
			if (!empty($strArray)){
				if (is_array($strArray)){ // wenn es wirklich ein Array ist
					foreach($strArray AS $str){
						mysqli_query($db, $this->insertNewID($str));
					}
				} elseif (is_string($strArray)) { // wenn es ein String ist
					mysqli_query($db, $this->insertNewID($strArray));
				}
			}
		}
		
		
		
		
		/**
		* aktualisiert den entsprechenden Datenbankeintrag.
		* @param string $updateStr Der String zur Aktualisierung des Objektes in die als "table" definierte MySQL-Tabelle. Falls leer, wird nur der Name in die entsprechende Tabelle aktualisiert. 
		* @param string $rowLabel Die Bezeichnung der Tabellenspalte, die die Objekt-ID in der Tabelle "tableAnnotation" trägt.
		* @param array $strArray Ein Array aus Strings mit optionalen, zusätzlichen MySQL-Befehlen, die abschließend ausgeführt werden. Falls nur ein Befehl übermittelt werden soll, kann der Parameter auch vom Typ String sein.
		* @return void
		*/
		public function updateDB_wp($updateStr,$rowLabel,$strArray){
			include(ABS_PATH . "dbconnect.php");
			$oldObject = $this->constructByID($this->ID);
			if (!empty($this->table)){ // Wenn Tabelle definiert ist
				// Ausführen des $updateStr
				if (is_null($updateStr)){
					$updateStr = '
					UPDATE ' .
						$this->table . '
					SET
						name = \'' . $this->name . '\'
					WHERE 
						ID = \'' . $this->ID . '\'';
				}
				mysqli_query($db,$updateStr);
			}
			
			// Ausführen des $annotationStr, sofern $tableAnnotation und $rowLabel definiert sind

			if (!is_null($this->tableAnnotation) AND !is_null($rowLabel)){
				if ($oldObject->annotation == ""){ // wenn bislang keine Annotation eingetragen ist
					if ($this->annotation != ""){ // wenn Annotation des Update-Objektes nicht leer ist
					$insertStr = '
						INSERT INTO ' .
							$this->tableAnnotation . '
						SET 
							annotation="' . $this->annotation . '", ' . 
							$rowLabel . '=' . $this->ID;
					mysqli_query($db, $insertStr);
					}
				} else { // wenn bereits eine Annotation eingetragen ist
					if ($this->annotation == ""){ // wenn neue Annotation NULL ist
						$deleteStr = '
							DELETE 
								FROM ' . 
								$this->tableAnnotation . '
								WHERE ' .
								$this->tableAnnotation . '.' . $rowLabel . '=' . $this->ID;
						mysqli_query($db, $deleteStr);
					} else { // wenn neue Annotation nicht NULL ist
						if ($this->annotation != $oldObject->annotation){ // wenn Differenz zwischen neuer Anmerkung und alter Anmerkung
						$updateStr = '
							UPDATE ' . 
								$this->tableAnnotation . '
							SET 
								annotation = "' . $this->annotation . '"
								WHERE ' .
							$this->tableAnnotation . '.' . $rowLabel . '=' . $this->ID;
						mysqli_query($db, $updateStr);
						} 
					}
				};
			}
			
			if ($strArray != ""){
				if (is_array($strArray)){ // wenn es wirklich ein Array ist
					foreach($strArray AS $str){
						mysqli_query($db, $str);
					}
				} elseif (is_string($strArray)) { // wenn es ein String ist
					mysqli_query($db, $strArray);
				}
			}
		}
		
		/**
		* entfernt das Objekt aus der in den Objekteigenschaften genannten $table
		* @param string $updateStr Der String bzw. das Array mit Befehlen zur Neusortierung, die durch die Löschung des Objektes notwendig ist. Achtung: Die Neusortierung erfolgt nur, falls das Objekt die Eigenschaft sortable hat.
		* @param string $deleteStr Der String zur Löschung des Objektes aus der als "table" definierten MySQL-Tabelle. Falls leer, ist die ID das einzige Kriterien zur Löschung aus der $table.
		* @param string $annotationStr Der String zur Löschung entsprechender Anmerkungen aus der Tabelle, die als Objekteigenschaft "tableAnnotation" eingetragen ist. Wird nur ausgeführt, wenn die Eigenschaft "tableAnnotation" definiert ist und für das Objekt auch tatsächlich eine "annotation" als Eigenschaft eingetragen ist.
		* @param array $strArray Ein Array aus Strings mit optionalen, zusätzlichen MySQL-Befehlen, die abschließend ausgeführt werden.
		* @return void
		*/
		public function deleteFromDB_wp($updateStr,$deleteStr,$annotationStr,$strArray){
			if ($this->table != ""){ // Wenn Tabelle definiert ist
				// Ausführen des $deleteStr
				if (!is_null($deleteStr)){
					$this->dbquery($deleteStr);
				} else {
					$deleteStr = '
						DELETE 
							FROM ' .
						$this->table . '
							WHERE 
								ID = ' . $this->ID;
					$this->dbquery($deleteStr);
				}
						
				// Ausführung des $updateStr
				if (!is_null($updateStr)){
					// $this->dbquery($updateStr);
					if (is_array($updateStr)){
						foreach ($updateStr AS $updateString){
							$this->dbquery($updateString);
						}
					} else {
						$this->dbquery($updateStr);
					}
				}
			}
			
			// Ausführen des $annotationStr, sofern $tableAnnotation definiert und $annotationStr vorhanden ist
			if (!is_null($this->tableAnnotation) AND !is_null($annotationStr)){
				$this->dbquery($annotationStr);
			}
			
			if (!is_null($strArray)){
				if (is_array($strArray)){
					foreach ($strArray AS $stringArray){
						$this->dbquery($stringArray);
					}
				} else {
					$this->dbquery($strArray);
				}
			}
		}
		
		/**
		* Ändert den Namen des Objektes in der in den Objekteigenschaften genannten $table
		* @param string $newName Der neue Name.
		* @return void.
		*/
		public function rename($newName){
			if ($this->table != ""){
				include(ABS_PATH . "dbconnect.php");
				$updateStr = '
					UPDATE ' .
						$this->table . '		
					SET 
						name = "' . $newName . '"  
					WHERE 
						ID = ' . $this->ID;
				mysqli_query($db, $updateStr);
			}
		}


		/**
		* Gibt den Namen und die Anmerkung in formatierter Form zurück
		* @return string Name und Anmerkungen zum ListEntry als formatierter String
		*/
		public function getNameAndAnnotation(){
			if (! is_null($this->annotation)) {
				$annotation = " (" . $this->annotation . ")";
			}	
			return $this->name . $annotation;
		}

	

	}
?>