<?php

	/** Datei, die die Definition der Klasse ListEntry enthält. */
	/** 
	* Klasse zur Erstellung eines ListEntries
	*/
	class ListEntry extends ListElement { // Artikel, der in eine Liste aufgenommen wurde
		/** @var int Die ListEntry-ID. */		
		protected $ID;
		/** @var int Die Menge des ListEntries. */		
		protected $amount;
		/** @var int Die ID für die Maßangabe. */		
		protected $measureID;
		/** @var string Der Name für die Maßangabe. */		
		protected $measureName;
		/** @var int Die ID für den Artikel. */		
		protected $articleID;
		/** @var string Die Artikelanmerkungen. */		
		protected $articleAnnotation;
		/** @var int Die ID der Liste, der der ListEntry unter anderem zugeordnet ist. */		
		protected $listID;
		/** @var int Die ID der Priorität des ListEntries. */		
		protected $priorityID;
		/** @var string Die Bezeichnung der Priorität des ListEntries. */		
		protected $priorityName;
		/** @var string Das Kaufdatum des ListEntries. */		
		protected $inBasket;
		/** Der Status des ListEntries (inBasket/notInBasket, urgent) */
		protected $status;
		
		/**
		* Der Konstruktor.
		* @param int $ID Die ID des Objektes.
		* @param int $amount Die Menge.
		* @param int $measureID Die ID der Maßeinheit.
		* @param string $measureName Die Bezeichnung der Maßeinheit.
		* @param int $articleID Die ID des Artikels.
		* @param string $name Die Bezeichnung des Artikels. auf dem der ListEntry basiert.
		* @param string $articleAnnotation Die Anmerkungen zum Artikel.
		* @param string $annotation Die Anmerkungen zum ListEntry.
		* @param int $listID Die ID der Liste, der der ListEntry (u.a.) zugeordnet ist.
		* @param int $priorityID Die ID der Priorität des ListEntries.
		* @param string $priorityName Die Bezeichnung der Priorität des ListEntries.
		* @param bool $inBasket Gibt an, ob sich der ListEntry im Einkaufskorb befindet.
		* @return object Ein Objekt vom Typ listEntry.
		*/ 
		public function __construct($ID,$amount,$measureID,$measureName,$articleID,$name,$articleAnnotation,$annotation,$listID,$priorityID,$priorityName,$inBasket) {
			include(ABS_PATH . "dbconnect.php");			
			parent::__construct($name,$annotation);
			$this->prefix = "listEntry";
			$this->table = DB_TAB_PREFIX . 'listEntries';
			$this->tableAnnotation = DB_TAB_PREFIX . 'listEntriesAnnotation';			
			$this->ID = $ID;
			$this->amount = $amount;
			$this->measureID = $measureID;
			if (empty($measureName)){
				$measureName = $this->selectMeasureName($measureID);
			} 
			$this->measureName = $measureName;
			$this->articleID = $articleID;
			$this->articleAnnotation = $articleAnnotation;
			$this->listID = $listID;
			$this->priorityID = $priorityID;
			if ($priorityName == ""){
				$this->priorityName = $this->selectPriorityName($priorityID);
			} else {
				$this->priorityName = $priorityName;
			}
			$this->inBasket = $inBasket;
			$this->status = $this->checkIfInBasket() . " " . $this->checkPriority();
		}
		
		/**
		* Konstruiert ein Objekt auf der Basis seiner ID.
		* *ACHTUNG*: Das Attribut $listID muss, sofern benötigt, in diesem Fall nachträglich mit setListID() gesetzt werden!
		* @param int $ID Die ID des Objektes.
		* @param int $ID Die ID Liste, der der ListEntry (u.a.) zugeordnet ist.
		* @return object Ein Objekt vom Typ ListEntry.
		*/ 	
		public function constructByID($ID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = 'SELECT 
				' . DB_TAB_PREFIX . 'listEntries.amount,
				' . DB_TAB_PREFIX . 'listEntries.measureID,
				' . DB_TAB_PREFIX . 'measures.name AS measureName,
				' . DB_TAB_PREFIX . 'listEntries.articleID,
				' . DB_TAB_PREFIX . 'articles.name,
				' . DB_TAB_PREFIX . 'articlesAnnotation.annotation AS articleAnnotation,
				' . DB_TAB_PREFIX . 'listEntriesAnnotation.annotation,
				' . DB_TAB_PREFIX . 'listEntries.priorityID,
				' . DB_TAB_PREFIX . 'priority.name AS priorityName,
				' . DB_TAB_PREFIX . 'listEntries.inBasket
			FROM 
				' . DB_TAB_PREFIX . 'listEntries 
			LEFT JOIN
				' . DB_TAB_PREFIX . 'articles 
				ON 
					' . DB_TAB_PREFIX . 'listEntries.articleID = ' . DB_TAB_PREFIX . 'articles.ID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAnnotation
						ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
			LEFT JOIN
				' . DB_TAB_PREFIX . 'listEntriesAnnotation
					ON ' . DB_TAB_PREFIX . 'listEntries.ID = ' . DB_TAB_PREFIX . 'listEntriesAnnotation.listEntryID			
			LEFT JOIN
				' . DB_TAB_PREFIX . 'priority
					ON ' . DB_TAB_PREFIX . 'listEntries.priorityID = ' . DB_TAB_PREFIX . 'priority.ID
			LEFT JOIN
				' . DB_TAB_PREFIX . 'measures
					ON ' . DB_TAB_PREFIX . 'listEntries.measureID = ' . DB_TAB_PREFIX . 'measures.ID
			WHERE 
				' . DB_TAB_PREFIX . 'listEntries.ID =' . $ID;
//			echo "<p>" . $selectStr. "</p>";
			$query = mysqli_query($db, $selectStr);
			if (mysqli_num_rows($query) > 0){
				$row = mysqli_fetch_object($query);
				return new ListEntry($ID,$row->amount,$row->measureID,$row->measureName,$row->articleID,$row->name,$row->articleAnnotation,$row->annotation,NULL,$row->priorityID,$row->priorityName,$row->inBasket);
			}
			return NULL;
		}
		
				/**
		* Konstruiert einen neuen, noch nicht in der Datenbank enthaltenen Listeneintrag auf der Basis eines Barcodes.
		* *ACHTUNG*: Das Attribut $listID muss, sofern benötigt, in diesem Fall nachträglich mit setListID() gesetzt werden! Mengen- und Maßangaben entsprechen bei diesem Objekt den Standardvorgaben.
		* @param string $barcode Der Barcode.
		* @return object Ein Objekt vom Typ ListEntry.
		*/ 	
		public function constructByBarcode($barcode){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = 'SELECT 
				' . DB_TAB_PREFIX . 'articlesStandards.standardAmount,
				' . DB_TAB_PREFIX . 'articlesStandards.standardMeasureID,
				' . DB_TAB_PREFIX . 'measures.name AS standardMeasureName,
				' . DB_TAB_PREFIX . 'barcodes.articleID,
				' . DB_TAB_PREFIX . 'articles.name,
				' . DB_TAB_PREFIX . 'articlesAnnotation.annotation AS articleAnnotation,
				' . DB_TAB_PREFIX . 'articlesStandards.standardListEntryAnnotation,
				' . DB_TAB_PREFIX . 'articles.standardPriorityID,
				' . DB_TAB_PREFIX . 'priority.name AS priorityName
			FROM 
				' . DB_TAB_PREFIX . 'barcodes 
			LEFT JOIN
				' . DB_TAB_PREFIX . 'articles 
				ON 
					' . DB_TAB_PREFIX . 'barcodes.articleID = ' . DB_TAB_PREFIX . 'articles.ID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAnnotation
						ON ' . DB_TAB_PREFIX . 'barcodes.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesStandards
						ON ' . DB_TAB_PREFIX . 'barcodes.articleID = ' . DB_TAB_PREFIX . 'articlesStandards.articleID
						LEFT JOIN
					' . DB_TAB_PREFIX . 'measures
						ON ' . DB_TAB_PREFIX . 'articlesStandards.standardMeasureID = ' . DB_TAB_PREFIX . 'measures.ID				
				
				LEFT JOIN
				' . DB_TAB_PREFIX . 'priority
					ON ' . DB_TAB_PREFIX . 'articles.standardPriorityID = ' . DB_TAB_PREFIX . 'priority.ID
				WHERE 
					' . DB_TAB_PREFIX . 'barcodes.name = "' . $barcode . '"';
			$query = mysqli_query($db, $selectStr);
			if (mysqli_num_rows($query) > 0){ // falls die Abfrage erfolgreich war
				$row = mysqli_fetch_object($query);
				$listEntry = new ListEntry(NULL,$row->standardAmount,$row->standardMeasureID,$row->standardMeasureName,$row->articleID,$row->name,$row->articleAnnotation,$row->standardListEntryAnnotation,NULL,$row->standardPriorityID,$row->priorityName,NULL);
				$listEntry->setStandardsIfNULL();
				return $listEntry;
			}
			return NULL;
		}
		
		/**
		* Setter der Variable $ID
		* @param int $ID Die ID des Objektes
		* @return void
		*/ 
		public function setID($ID){
			$this->ID = $ID;
		}
		
		/**
		* Gibt die ID zurück
		* @return int Die ID
		*/ 
		public function getID(){
			return $this->ID;
		}
		
		/**
		* Setter der Variable $amount
		* @param int $amount Die Menge
		* @return void
		*/ 
		public function setAmount($amount){
			if ($amount > 0){
				$this->amount = $amount;
			}			
		}
		
				
		/**
		* Gibt die Menge $amount zurück
		* @return int Die Menge
		*/ 
		public function getAmount(){
			return $this->amount;
		}
		
		/**
		* Setter der Variable $measureID; die Variable $measureName wird automatisch neu gesetzt.
		* @param int $measureID Die ID für die Maßangabe.
		* @return void
		*/ 
		public function setMeasureID($measureID){
			$this->measureID = $measureID;
			$this->measureName = $this->selectMeasureName($measureID);
		}
		
		/**
		* Gibt die ID für die Maßangabe zurück
		* @return int Die ID der Maßangabe
		*/ 
		public function getMeasureID(){
			return $this->measureID;
		}
		
		
		/**
		* Gibt die Bezeichnung für die Maßangabe zurück.
		* @return string Die Bezeichnung der Maßangabe.
		*/ 
		public function getMeasureName(){
			return $this->measureName;
		}

		/**
		* Setter der Variable $articleID
		* @param int $articleID Die ID des Artikels
		* @return void
		*/ 
		public function setArticleID($articleID){
			$this->articleID = $articleID;
		}
		
		/**
		* Gibt die ID des Artikels zurück
		* @return int Die ID des Artikels
		*/ 
		public function getArticleID(){
			return $this->articleID;
		}

		/**
		* Setter der Variable $articleAnnotation
		* @param string $articleAnnotation Die Anmerkungen zum Artikel
		* @return void
		*/ 
		public function setArticleAnnotation($articleAnnotation){
			$this->articleAnnotation = $articleAnnotation;
		}
		
		/**
		* Gibt die Anmerkungen zum Artikel zurück
		* @return string Die Anmerkungen zum Artikel
		*/ 
		public function getArticleAnnotation(){
			return $this->articleAnnotation;
		}
			
		
		/**
		* Setter der Variable $listID
		* @param int $listID Die ID der Liste
		* @return void
		*/ 
		public function setListID($listID){
			$this->listID = $listID;
		}
		
		/**
		* Gibt die ID der Liste zurück
		* @return int Die ID der Liste
		*/ 
		public function getListID(){
			return $this->listID;
		}		
		
		/**
		* Setter der Variablen $priorityID und $priorityName.
		* @param int $priorityID Die ID der Priorität.
		* @return void
		*/ 
		public function setPriorityID($priorityID){
			$this->priorityID = $priorityID;
			$this->priorityName = $this->selectPriorityName($priorityID);
		}
		
		/**
		* Gibt die ID der Priorität zurück
		* @return int Die ID der Priorität
		*/ 
		public function getPriorityID(){
			return $this->priorityID;
		}



		/**
		* Ermittelt die Bezeichnung der Maßangabe auf der Basis ihrer ID und gibt sie zurück.
		* @param int $measureID Die ID der Maßangabe.
		* @return string Die Bezeichnung der Maßangabe.
		*/
		protected function selectMeasureName($measureID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					name 
				FROM 
					' . DB_TAB_PREFIX . 'measures 
				WHERE 
					ID = ' . intval($measureID);
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if (!empty($row->name)){
				return $row->name;
			}
		}
		
		/**
		* Ermittelt die Bezeichnung der Priorität auf der Basis ihrer ID und gibt sie zurück.
		* @param int $priorityID Die ID der Priorität.
		* @return string Die Bezeichnung der Priorität.
		*/
		protected function selectPriorityName($priorityID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					name 
				FROM 
					' . DB_TAB_PREFIX . 'priority 
				WHERE 
					ID = ' . intval($priorityID);
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if (!empty($row->name)){
				return $row->name;
			}
		}
		
		/**
		* Gibt die Bezeichnung der Priorität zurück.
		* @return string Die Bezeichnung der Priorität.
		*/ 
		public function getPriorityName(){
			return $this->priorityName;
		}
		
		/**
		* Gibt den Status des ListEntries zurück
		* @return string Der Status des ListEntries
		*/ 
		public function getStatus(){
			return $this->status;
		}
		
		/**
		* Gibt den Namen und ggf. die Anmerkungen zum Artikel und dem ListEntry in formatierter Form zurück.
		* @return string Name und Anmerkungen als formatierter String.
		*/
		public function getNameAndAnnotation(){
			if ($this->articleAnnotation != "") {
				$articleAnnotation = " (" . $this->articleAnnotation . ")";
			} else {
				$articleAnnotation = NULL;
			}
			
			if ($this->annotation != "") {
				$annotation = " (" . $this->annotation . ")";
			} else {
				$annotation = NULL;
			}
			
			return $this->name . $articleAnnotation . $annotation;
		}
		
		/**
		* Gibt den Namen und ggf. die Anmerkung zum Artikel in formatierter Form zurück.
		* Um auch die Anmmerkungen zum ListEntry auszugeben, ist die Funktion getNameAndAnnotation() zu verwenden.
		* @return string Name und Anmerkungen als formatierter String
		*/
		public function getArticleNameAndArticleAnnotation(){
			if ($this->articleAnnotation != "") {
				$articleAnnotation = " (" . $this->articleAnnotation . ")";
			} else {
				$articleAnnotation = "";
			}
			return $this->name . $articleAnnotation;
		}
		
	
		/**
		* löscht sämtliche ListEntries mit den im Array $array übergebenen IDs.
		* @param $array Das Arrays mit den IDs der zu löschenden ListEntries.
		* @return void
		*/
		
		public function deleteByIDs($array){
			include(ABS_PATH . "dbconnect.php");
			$ids = implode(' OR ' . DB_TAB_PREFIX . 'listEntries.ID = ',$array);
			$deleteStr = '
				DELETE FROM 
					' . DB_TAB_PREFIX . 'listEntries 
				WHERE ' . DB_TAB_PREFIX . 'listEntries.ID =' . $ids;
			$ids = implode(' OR ' . DB_TAB_PREFIX . 'listEntriesAnnotation.listEntryID = ',$array);			
			$deleteStrAnnotation = '
				DELETE FROM 
					' . DB_TAB_PREFIX . 'listEntriesAnnotation 
				WHERE ' . DB_TAB_PREFIX . 'listEntriesAnnotation.listEntryID = ' . $ids;
			mysqli_query($db, $deleteStr);
			mysqli_query($db, $deleteStrAnnotation);
		}


		/**
		* Prüft den Kaufzustand eines ListEntries und gibt je nach Kaufzustand den String "inBasket" bzw. "NotInBasket" zurück.
		* @return string Der Kaufzustand.
		*/
		private function checkIfInBasket(){
			if ($this->inBasket != "") {
				return "inBasket"; }
			else {
				return "notInBasket";
			}
		}
		
		/**
		* Prüft, ob der ListEntry die Prioritäts-ID 3 ("urgent") besitzt, und gibt gegebenenfalls den String "urgent" zurück.
		* @return string "urgent", falls Prioritäts-ID 3.
		*/
		private function checkPriority(){
			if ($this->priorityID == 1){
				return "low";
			} elseif ($this->priorityID == 2){
				return "standard";
			} else if ($this->priorityID == 3){
				return "urgent";
			}
		}

		/**
		*  konvertiert den Warenkorbzustand eines ListEntries von 0 zu 1 oder von 1 zu 0 und führt die entsprechenden Änderungen in der MySQL-Tabelle aus
		* @return void
		*/ 
		public function convertBasketStatus(){
			include(ABS_PATH . "dbconnect.php");
			if ($this->inBasket == ""){	
				// return "'" . date("Y-m-d H:i:s") . "'";
				// $inBasket = "'" . date("Y-m-d H:i:s") . "'";
				$inBasket = "1";
			} else {
				// return "NULL";
				$inBasket = "NULL";
			}
			$update = '
				UPDATE 
					' . DB_TAB_PREFIX . 'listEntries 
				SET 
					' . DB_TAB_PREFIX . 'listEntries.inBasket = ' . $inBasket . ' 
				WHERE 
					' . DB_TAB_PREFIX . 'listEntries.ID= ' . $this->ID;
			mysqli_query($db, $update);
		}

		
		/**
		* fügt der in den Objekteigenschaften genannten $table einen weiteren Datenbankeintrag hinzu.
		* @return void
		*/
		public function addToDB(){
			require_once("./config.php");
			$insertStr = "
				INSERT INTO " .
						$this->table . "
				SET 
					amount=" . intval($this->amount) . ", 
					measureID =" . intval($this->measureID) . ", 
					articleID=" . intval($this->articleID) . ",
					priorityID=" . intval($this->priorityID);
			$annotationStr = '
				INSERT INTO 
					' . DB_TAB_PREFIX . 'listEntriesAnnotation
				SET 
					annotation = "' . $this->annotation . '",
					listEntryID=[[thisID]]';
			parent::addToDB_wp(NULL,$insertStr,$annotationStr,NULL);	
		}
		
		/**
		* setzt die für den Artikel gesetzten Standardeinträge, wenn die entsprechende Eigenschaft nicht gesetzt ist
		* @return object Das Objekt mit gegebenenfalls gesetzten Standardeinträgen 
		*/
		public function setStandardsIfNULL(){
			$article = Article::constructByID($this->articleID);
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'prefs.standardAmount,
					' . DB_TAB_PREFIX . 'prefs.standardMeasureID,
					' . DB_TAB_PREFIX . 'prefs.standardPriorityID 
				FROM ' . DB_TAB_PREFIX . 'prefs WHERE ID=1';
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if (empty($this->amount)){
				if (!empty($article->getStandardAmount())){
					$this->amount = $article->getStandardAmount();
				} else {
					$this->amount = $row->standardAmount;
				}
			}
			
			if (empty($this->measureID)){
				if (!empty($article->getStandardMeasureID())){
					$this->setMeasureID($article->getStandardMeasureID());
				} else {
					$this->setMeasureID($row->standardMeasureID);
				}
			}
			
			if (empty($this->annotation) && !empty($article->getStandardListEntryAnnotation())){
				$this->annotation = $article->getStandardListEntryAnnotation();
			}
			
			if (empty($this->priorityID)){
				if (!empty($article->getStandardPriorityID())){
					$this->setPriorityID($article->getStandardPriorityID());
				} else {
					$this->setPriorityID($row->standardPriorityID);
				}
			}
		}
	
		/**
		* aktualisiert den entsprechenden Datenbankeintrag.
		* @param string $updateStr Der String zur Aktualisierung des Objektes in die als "table" definierte MySQL-Tabelle. Falls leer, wird nur der Name in die entsprechende Tabelle aktualisiert. 
		* @param string $rowLabel Die Bezeichnung der Tabellenspalte, die die Objekt-ID in der Tabelle "tableAnnotation" trägt.
		* @param array $strArray Ein Array aus Strings mit optionalen, zusätzlichen MySQL-Befehlen, die abschließend ausgeführt werden. Falls nur ein Befehl übermittelt werden soll, kann der Parameter auch vom Typ String sein.
		* @return void
		*/
		public function updateDB(){
			$updateStr = '
				UPDATE 
					' . DB_TAB_PREFIX . 'listEntries	
				SET 
					' . DB_TAB_PREFIX . 'listEntries.amount =' . $this->addBracketsIfNotNull($this->amount) . ',
					' . DB_TAB_PREFIX . 'listEntries.measureID =' . $this->addBracketsIfNotNull($this->measureID) . ',
					' . DB_TAB_PREFIX . 'listEntries.priorityID =' . $this->addBracketsIfNotNull($this->priorityID) . ',
					' . DB_TAB_PREFIX . 'listEntries.inBasket =' . $this->addBracketsIfNotNull($this->inBasket) . '
				WHERE 
					ID=' . $this->ID;
			$rowLabel = "listEntryID";


			parent::updateDB_wp($updateStr,$rowLabel,NULL);	
		}
	}
?>