<?php
	/** Datei, die die Definition der Klasse ListEntryPage enthält. */
	/** 
	* Klasse zur Erstellung der ListEntry-Seite
	*/
	class ListEntryPage extends AddArticlePage{
		/** @var obj Der Listeneintrag. */
		protected $listEntry;
		
		/**
		* Der Konstruktor
		* @param string $title Der Seitentitel.
		* @param obj $listEntry Der Listeneintrag.
		* @param string $mode Der Modus. Mögliche Werte sind "default" und "edit". "Default" gibt eine Standardseite aus, "edit" eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ ListEntryPage.
		*/ 
		public function __construct($title,$listEntry,$mode) {
			parent::__construct($title,$mode);
			$this->listEntry = $listEntry;
	//		$this->containsDeletableItems = true;
			$this->headerIcons = $this->getHeaderIcons();
		}
		
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
  		* @param array $array Ein Array optionaler zusätzlicher Angaben, die in den Head eingebunden werden.
		* @return string
 		*/	 
		public function start($array){
			return parent::start_wp($array);
		}
		
		/**
		* gibt ein Formular zur Bearbeitung der Eigenschaften eines ListEntries zurück.
		* @param object $listEntry Der ListEntry.
		* @param int $listID Die ID der Liste, der der ListEntry (u.a.) zugeordnet ist.
		* @param int $indent Die Einzugtiefe.
		* @return string Der HTML-Quelltext für das Formular
		*/
		public function editListEntry($listEntry,$listID,$indent){
			$hiddenFields = array(
						"ID" => $listEntry->getID(),
						"listID" => $listID,
						"formular" => "update",
					);
			return $this->startFormular("display: inline;","","update","","POST","","",$indent) .
			$this->formularListEntry($listEntry,NULL,"Anmerkungen, die Sie hier machen, werden <emph>nur diesem einen Listeneintrag</emph> angefügt.","",$indent+1) .
			$this->hiddenFields($hiddenFields,$indent+1) .
			$this->endFormular($indent) . 
			$this->startFormular("display: inline;","list.php?ID=" . $listID,"back","","POST","","",$indent) . 		
			$this->endFormular($indent) .
			$this->submitButtonArea($this->submitButton("back","submitButton","Submit","submit","","zurück"),$this->submitButton("update","submitButton","Submit","submit","","Senden"),$indent);
		}		
		
		/**
		* liefert den HTML-Quelltext für die Bearbeitung der Formularfelder Menge, Maß, Artikelbezeichnung, Anmerkungen und Priorität zurück.
		* Wird im Rahmen der Funktion editListEntry() verwendet.
		* @param object $listEntry Der ListEntry.
		* @param string $legend Die Legende unter dem Formularfeld "Anmerkungen"; falls leer, wird "Anmerkungen" gesetzt.
		* @param string $info Informationstext, der unter dem Formularfeld "Anmerkungen" ausgegeben werden kann.
		* @param string $prefix Das Präfix, das an die Funktionen printAmount, printMeasure und editAnnotation übergeben wird.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.
		*/ 
		private function formularListEntry($listEntry,$legend,$info,$prefix,$indent){
			if ($listEntry == ""){
				$listEntry = new ListEntry("","","","","","","","","","","","");
			}
			
			if($listEntry->getAnnotation() != ""){
				$annotation = $listEntry->getAnnotation();
			} else {
				$annotation = NULL;
			}
			if($listEntry->getPriorityID() != ""){
				$priorityID = $listEntry->getPriorityID();
			} else {
				$priorityID = NULL;
			}
			return $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Menge und Maß' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->printAmount($listEntry->getAmount(),$prefix,NULL,$indent+1) .
			$this->printMeasure($listEntry->getMeasureID(),$prefix,NULL,$indent+1) .
			$this->linebreak($indent) . '</fieldset>' . 
			$this->selectArticleName($listEntry,$indent) .
			$this->editAnnotation($annotation,$legend,$info,$prefix,NULL,$indent) . 
			$this->selectPriority($priorityID,NULL,NULL,NULL,$indent);
		}
		
		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @return array Array mit den Icon-Angaben
		*/
		protected function getHeaderIcons(){
			$deleteIcon = array(
					"targetPage" => $this->editPage,
					"arrayHiddenFields" => array(
						"formular" => "delete",
						"listEntry" => Main::objectToString($this->listEntry),
					),
					"iconTyp" => "delete",
				);
			$rightIcons = array();
			$rightIcons['edit'] = array();
			array_push($rightIcons['edit'],$deleteIcon);
			return parent:: getHeaderIcons_wp("",$rightIcons);
		}
		
		/**
		* Gibt ein Formular für das Hinzufügen eines neuen ListEntries in eine Liste zurück.
		* @param $listEntry Der ListEntry, der hinzugefügt werden soll (enthält nur dann Daten, wenn es zuvor zu einer Fehleingabe gekommen ist).
		* @param string $returnURL Seite, auf die nach einer erfolgreichen Eintragung des Artikels zurückverwiesen werden soll.
		* @param int $indent Die Einzugtiefe.
		* @return string Der Quelltext.	
		*/
		public function addListEntry($listEntry,$returnURL,$indent) {
			if ($listEntry->getName()!= ""){ // wenn Name vorhanden ist, wird er in Funktion selectArticleName nicht mehr als $_POST['name'] übergeben und muss deshalb vorab übergeben werden
				$nameAsHiddenField = $this->hiddenFields(array("name" => $listEntry->getName()),$indent+1);
			} else {
				$nameAsHiddenField = "";
			}
			$result = $this->startFormular("display: inline;","","update","","POST","","",$indent) .
	//		$this->hiddenFields(array("listEntry" => $this->objectToString($listEntry), "formular" => "checkNewListEntry","returnURL" => $returnURL),$indent+1) . // ListEntry darf nicht als hiddenField übergeben werden, da es sonst bei checkNewListEntry in newListEntry.php zu einem Fehler kommt. Falls anderswo notwendig, muss  newListEntry.php so umprogrammiert werden, dass der ListEntry auf der Basis der Formulare per Switch ermittelt wird.
			$this->hiddenFields(array("formular" => "checkNewListEntry","returnURL" => $returnURL),$indent+1) .
			$nameAsHiddenField .
			$this->formularListEntry($listEntry,NULL,"Anmerkungen, die Sie hier machen, werden <emph>nur diesem einen Listeneintrag</emph> angefügt.","",$indent+1) .
			$this->endFormular($indent) . 		
			$this->submitButtonArea("",$this->submitButton("update","submitButton","","","","Senden"),$indent);
			return $result; 
		}
		
		/**
		* Prüft die Eingabe eines neuen ListEntries auf ihre Gültigkeit, trennt, sofern notwendig, Namen und Anmerkungen voneinander und behandelt die Rekonstruktion des Listeneintrages.
		* @param object $listEntry Ein Objekt der Klasse ListEntry.
		* @param string $returnURL Seite, auf die nach einer erfolgreichen Eintragung des Artikels zurückverwiesen werden soll.
		* @param int $indent Die Einzugtiefe.
		* @return void.
		*/
		public function checkNewListEntry($listEntry,$returnURL,$indent){
			if ($listEntry->getName() == ""){ // wenn kein Artikelname angegeben ist, auf vorherige Seite verweisen	
				echo $this->goBack("Bitte geben Sie eine Artikelbezeichnung ein.",array("listEntry" => $this->objectToString($listEntry), "formular" => "addListEntry"),"","","",$indent);
			} 
			else { // wenn Artikelname angegeben ist, weitere Überprüfung; ACHTUNG: $articleName enthält ggf. angefügte ArticleAnnotation
				$reconstructed = $this->reconstructArticleName($listEntry,"ListEntry"); // Trennung von articleName und articleAnnotation;
				if ($reconstructed == false){ // falls Rekonstruktion nicht gelungen ist // der Artikelname bislang nicht in der  Tabelle Articles eingetragen ist
					echo $this->reconstructionFailed($listEntry,$indent);					
				} else { // wenn Rekonstruktion gelungen ist / der Artikelname der Tabelle Articles eingetragen ist
					$this->reconstructionSucceeded($reconstructed,$returnURL,$indent);
				}
			}
		}

		
		/**
		* gibt eine Meldung zurück, wenn die Rekonstruktion eines ListEntryNames gescheitert ist.
		/* Die Meldung bietet die Möglichkeit, zur Dateneingabe zurückzukehren oder einen entsprechenden neuen Artikel anzulegen. 
		* @param array $listEntry Ein ListEntry.
		* @param $indent Der Einzug.
		* @return String Die Meldung über die gescheiterte Rekonstruktion.
		*/
		private function reconstructionFailed($listEntry,$indent){
			if (empty($_POST['listID'])){ // wenn noch keine Liste ausgewählt wurde
				$index = new Index();
				$index->addSubCats();
				$lists = "";
				foreach($index->getLists() AS $list){
					$lists .= $this->linebreak($indent+3) . '<option value="' . $list->getID(). '">' . 
					$this->linebreak($indent+4) . $list->getName() .
					$this->linebreak($indent+3) . '</option>';
				}
				$selectList =	
					$this->linebreak($indent+1) . '<fieldset>' .
					$this->linebreak($indent+2) . '<legend>' . 
					$this->linebreak($indent+3) . 'Zuordnung zu einer Liste'. 
					$this->linebreak($indent+2) . '</legend>' .
					$this->linebreak($indent+2) . '<select class="fullWidth" name="listIDAdded" size="1">' . 
					$lists .
					$this->linebreak($indent+2) . '</select>' . 
					$this->linebreak($indent+1) . '</fieldset>';
				$infotext = " Hierzu müssen sie zunächst eine Liste wählen, der der Artikel zugeordnet werden soll. ";
			} else {
				$selectList = "";
				$infotext = "";
			}	
			$startForm2 = $this->startFormular("display: inline;","newArticle.php","newArticleName","","POST","","",$indent);
			$submitButtonForm2 = $this->submitButton("newArticleName","submitButton","Submit","submit","","hinzufügen");
			$listEntryWithoutName = new ListEntry($listEntry->getID(),$listEntry->getAmount(),$listEntry->getMeasureID(),$listEntry->getMeasureName(),$listEntry->getArticleID(),NULL,$listEntry->getArticleAnnotation(),$listEntry->getAnnotation(),$listEntry->getListID(),$listEntry->getPriorityID(),$listEntry->getPriorityName(),NULL);
		//	$listEntryWithoutName = $listEntry;
		//	$listEntryWithoutName->setName(NULL);
			$info = 'Bislang existiert kein Artikel mit der Bezeichnung <q>' .  $listEntry->getName() . '</q>. Bitte wählen Sie <q>zurück</q> und geben Sie einen gültigen Artikelnamen ein.' . 
				$this->linebreak($indent) . '</p>' . 
				$this->linebreak($indent) . '<p>' . 
				$this->linebreak($indent+1) . 'Alternativ können Sie <q>' . $listEntry->getName() . '</q> auch als neuen Artikel zur Datenbank hinzufügen. ' . $infotext . 'Anschließend wird er der Einkaufsliste in der gewünschten Menge hinzugefügt.';
			return $this->goBack(
				$info,
				array("listEntry" => $this->objectToString($listEntryWithoutName)/*, "formular" => $insertFormular*/),
				$startForm2 . $selectList,
				array("listEntry" => $this->objectToString($listEntry), "formular" => "newArticleNameForSpecialList"),
				$submitButtonForm2,
				$indent
			);
		}
		
		/**
		* behandelt die erfolgreiche Rekonstruktion eines ListEntries.
		* @param $reconstructed Der rekonstruierte ListEntry.
		* @param string $returnURL Seite, auf die nach einer erfolgreichen Eintragung des Artikels zurückverwiesen werden soll.
		* @param $indent Der Einzug.
		* @return string Der Quelltext zur Behandlung der erfolgreichen Rekonstruktion.
		*/
		private function reconstructionSucceeded($reconstructed,$returnURL,$indent) {
			$reconstructed->setStandardsIfNULL();
			$arrayOfListEntries = $this->selectListEntriesOfSameName($reconstructed->getName()); // speichere alle ListEntry mit demselben ArticleName in einem Array
			if(count($arrayOfListEntries) > 0){ // wenn bereits ein anderer Artikel mit demselben Artikelnamen in der Liste ListEntries vorhanden ist
				echo $this->handleInsertionWithSameListEntry($reconstructed,$arrayOfListEntries,2);
			} else { // wenn gültiger Artikelname, aber kein anderer Artikel mit dem Artikelnamen in Liste ListEntries ist
				$reconstructed->addToDB();
				$this->goToPage($returnURL);
			}
		}
		
		
		/**
		* behandelt die Einfügung eines neuen ListEntries für den Fall, dass es bereits einen anderen ListEntry mit demselben ArticleName gibt.
		* @param $listEntry Der neue ListEntry.
		* @param $arrayOfListEntries Ein Array aus ListEntries mit demselben Namen, erstellt mit der Funktion selectListEntriesOfSameName().
		* @param $indent Der Einzug.
		*/
		private function handleInsertionWithSameListEntry($listEntry,$arrayOfListEntries,$indent){
			$numberOfListEntries = count($arrayOfListEntries);
			$listEntryAnnotation = $listEntry->getAnnotation();
			$arrayOfListEntryWithSameMeasureID = $this->searchIdenticalMeasureID($listEntry->getMeasureID(),$arrayOfListEntries); // sucht in der Liste mit identischen ArticleName nach einem Eintrag mit identischer MeasureID
			$arrayHiddenFields = array(
					"formular" => "insertNewListEntry",
					"ListEntryIDsOfSameName" => $this->ArrayElementIDs($arrayOfListEntries),	
			);
			if ($listEntryAnnotation != ""){
				$annotation = ' der Bemerkung <q>' . $listEntryAnnotation . '</q> und';
			} else {
				$annotation = "";
			}
			
			$result = $this->linebreak($indent) . '<p>Sie möchten der Liste <q>' . $listEntry->getAmount() . ' ' . $listEntry->getMeasureName() . ' ' . $listEntry->getName() . '</q> mit' . $annotation . ' der Dringlichkeit <q>' . $listEntry->getPriorityName() . '</q> hinzufügen. ' . 
			$this->showExistingListEntries($arrayOfListEntries,$indent) .
			$this->linebreak($indent) . '<p>Wie möchten Sie verfahren?</p>' .
			$this->startFormular("display: inline;","newListEntry.php","goBack","","POST","","",$indent) . 
			$this->hiddenFields(array("formular" => "addListEntry","listID" => $listEntry->getListID()),$indent + 1) .
			$this->endFormular($indent) . 		
			$this->startFormular("display: inline;","","insertListEntry","","POST","","",$indent) . // $style,$action,$id,$enctype,$method,$name,$target,$indent
			$this->hiddenFields($arrayHiddenFields + array("listEntry" => $this->objectToString($listEntry)),$indent + 1) .
			$this->hiddenFields(
				array("iM" => Main::objectToString($arrayOfListEntryWithSameMeasureID)),
				$indent + 1
			) . // Informationen der $arrayOfListEntryWithSameMeasureID als HiddenFields hinzufügen
			$this->linebreak($indent+1) . '<fieldset>';
			
			
			if ($arrayOfListEntryWithSameMeasureID != ""){ // wenn bereits ein Eintrag mit derselben MeasureID existiert
			$relationOfPriority = $this->comparePriorities($listEntry,array($arrayOfListEntryWithSameMeasureID));
				$result .= $this->reactionLeaveUnchanged("r1","checked",$indent+2); // den aktuellen Eintrag verwerfen
				if ($relationOfPriority['relation']=="higher" && $this->sameAnnotations($listEntryAnnotation,array($arrayOfListEntryWithSameMeasureID))){ // wenn der neue Eintrag die gleiche Anmerkung, aber eine höhere Priorität hat 
					$result .=  $this->reactionUpdatePriority($listEntry->getPriorityName(),"r2","",$indent+2); // die Priorität updaten;
				}
				if (($relationOfPriority['relation']=="lower" OR $relationOfPriority['relation']=="same") && !$this->sameAnnotations($listEntryAnnotation,array($arrayOfListEntryWithSameMeasureID))){ // wenn der neue Eintrag nicht die gleiche Anmerkung, aber eine geringere oder die gleiche Priorität hat
					$result .=  $this->reactionUpdateAnnotation($listEntryAnnotation,$arrayOfListEntryWithSameMeasureID->getAnnotation(),"r3",$indent+2); // Anmerkung updaten
				}				
				// Möglichkeit, Artikel mit derselben Maßeinheit zu einem neuen Eintrag zusammenzufassen, wobei gilt: 
					// $amount wird standardmäßig addiert, kann aber im Formular geändert werden
					// $annotation wird standardmäßig addiert, kann aber im Formular geändert werden
					// $priorityID richtet sich nach der höchsten Priorität
				$annotation = array();
				if ($listEntryAnnotation != ""){
					array_push($annotation,$listEntryAnnotation);
				}
				if ($arrayOfListEntryWithSameMeasureID->getAnnotation() != ""){
					array_push($annotation,$arrayOfListEntryWithSameMeasureID->getAnnotation());
				}

				$newListEntry = new ListEntry(
					"", // ID		
					$listEntry->getAmount() + $arrayOfListEntryWithSameMeasureID->getAmount(),
					$listEntry->getMeasureID(),
					$listEntry->getMeasureName(),
					$listEntry->getArticleID(),
					$listEntry->getName(),
					$listEntry->getArticleAnnotation(),
					implode(", ",$annotation),
					$listEntry->getListID(),
					$relationOfPriority['highest'],
					"", //priorityName
					"" //inBasket
				);
				$result .=  $this->reactionMergeToNewEntry(
					$newListEntry,
					"r4",
					$indent+2
				);
				if ($numberOfListEntries >= 2){ // wenn zwei ListEntry gleichen Typs, aber mit unterschiedlicher MeasureID vorhanden sind
				echo "wenn zwei ListEntry gleichen Typs, aber mit unterschiedlicher MeasureID vorhanden sind";
					$annotation = array();
					if ($listEntryAnnotation != ""){
						array_push($annotation,$listEntryAnnotation);
					}
					foreach ($arrayOfListEntries AS $element){
						if ($element->getAnnotation() != ""){
							array_push($annotation,$element->getAnnotation());
						}
					}
					$newListEntry = new ListEntry(
						"", // ID	
						"", // Amount
						"", // MeasureID
						"", // MeasureName
						$listEntry->getArticleID(),
						$listEntry->getName(),
						$listEntry->getArticleAnnotation(),
						implode(", ",$annotation),
						$listEntry->getListID(),
						$this->comparePriorities($listEntry,$arrayOfListEntries)['highest'],
						"", //priorityName
						"" //inBasket
					);
					$result .=  $this->reactionMergeAllToNewEntry(	
						$newListEntry,
						"r5",
						$indent+2
					);
				}
			} else { // wenn gleiche ListEntry andere MeasureIDs haben
				echo "wenn gleiche ListEntry andere MeasureIDs haben";
				$annotation = array();
				if ($listEntryAnnotation != ""){
					array_push($annotation,$listEntryAnnotation);
				} 
				foreach ($arrayOfListEntries AS $element){
					if ($element->getAnnotation() != ""){
						array_push($annotation,$element->getAnnotation());
					}
				}
				$annotation = implode(", ",$annotation);
				$newListEntry = new ListEntry(
					"", // ID	
					"", // Amount
					"", // MeasureID
					"", // MeasureName
					$listEntry->getArticleID(),
					$listEntry->getName(),
					$listEntry->getArticleAnnotation(),
					$annotation,
					$listEntry->getListID(),
					$this->comparePriorities($listEntry,$arrayOfListEntries)['highest'],
					"", //priorityName
					"" //inBasket
				);
				$result .=  $this->reactionAddAsNewMeasure($listEntry,$arrayOfListEntries,"r1","checked",$indent) .
				$this->reactionMergeAllToNewEntry(
					$newListEntry,
					"r2",
					$indent
				);
			}
			// Ergebnisausgabe
			$result .=  $this->linebreak($indent+1) . '</fieldset>' .
			$this->endFormular($indent) .
			$this->submitButtonArea(
				$this->submitButton("goBack","submitButton","","","","zurück"),
				// $form,$class,$name,$type,$value,$text				
				
				$this->submitButton("insertListEntry","submitButton","","","","Senden"),
				$indent);
			return $result;
		}
		

		
		/**
		* prüft, ob sich in dem übergebenen Array ein ListEntry mit derselben wie der übergebenen MeasureID befindet, und gibt ggf. den ListEntry mit derselben MeasureID als Array zurück.
		* Falls kein ListEntry mit identischer MeasureID gefunden wird, wird null zurückgegeben.
		* @param $measureID Die im Array zu suchende MeasureID.
		* @param $listEntries Das Array aus zu überprüfenden ListEntries.
		* @return array Der ListEntries mit derselben MeasureID als Array.
		*/
		private function searchIdenticalMeasureID($measureID,$listEntries){
			foreach($listEntries AS $listEntry){
				if ($listEntry->getMeasureID() == $measureID){
					return $listEntry;
				}
			}
			return null;
		}
		
		/**
		* speichert alle ListEntries mit demselben Namen in einem Array.
		* @param string $name Der Name.
		* @return array Das Array mit den ListEntry-Objekten.
		*/
		private function selectListEntriesOfSameName($name){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT
					' . DB_TAB_PREFIX . 'listEntries.ID
				FROM 
					' . DB_TAB_PREFIX . 'listEntries
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articles
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID=' . DB_TAB_PREFIX . 'articles.ID	
				WHERE
					' . DB_TAB_PREFIX . 'articles.name = "' . $name . '"'; 
			$query = mysqli_query($db, $selectStr);
			$result = array(); 
			while($row = $query->fetch_object()){
				array_push($result,ListEntry::constructByID($row->ID));
			}
			return $result;					
		}
		
		/**
		* prüft die Priorität eines ListEntries im Vergleich zu einem Array aus ListEntries, und gibt sie als assoziatives Array zurück.
		* Das zurückgegebene Array verfügt über die Schlüssel "relation", "highest" und "lowest". Falls die Priorität des übergebenen ListEntries geringer ist als die niedrigste Priorität des übermittelten Arrays, trägt "relation" den Wert "lower", andernfalls "same" bei Gleichheit oder "higher", falls sie höher ist. Die Schlüssel "highest" und "lowest" übermittelt den höchsten bzw. niedrigen Prioritätswert des Arrays und des übermittelten ListEntry-Objektes. 
		* @param $listEntry Der ListEntry.
		* @param $arrayOfListEntries Ein Array aus ListEntries, die hinsichtlich ihrer Priorität verglichen werden sollen.
		* @return array Ein assoziatives Array mit den Schlüsseln "relation", "highest" und "lowest". 
		
		*/
		private function comparePriorities($listEntry,$arrayOfListEntries){
			$relation = "";			
			$priorityID = $listEntry->getPriorityID();
			$highest = $arrayOfListEntries[0]->getPriorityID(); // Wert des ersten Array-Elementes 			
			$lowest = $arrayOfListEntries[0]->getPriorityID(); //Wert des ersten Array-Elementes 			
			foreach($arrayOfListEntries AS $listEntry){
				if ($listEntry->getPriorityID() > $highest) $highest = $listEntry->getPriorityID();
				if ($listEntry->getPriorityID() < $lowest) $lowest = $listEntry->getPriorityID();
			}
			if (($priorityID == $highest) AND ($priorityID == $lowest)){ // wenn im Array keine Daten enthalten waren, die kleiner oder größer als $this->priority sind
				$relation = "same";
			} elseif ($priorityID < $lowest){ // falls neue Priorität geringer
				$relation = "lower";
				$lowest = $priorityID;
			} elseif($priorityID > $highest) {
				$relation = "higher";
				$highest = $priorityID;
			} 
			$result = array(
				"relation" => $relation,
				"highest" => $highest,
				"lowest" => $lowest,
			);
			return $result;
		}
		
		/**
		* Zeigt bereits existierende ListEntry mit einer Einleitung versehen in Kurzform dar.
		* Die Funktion ruft im Wesentlichen die Funktion listOfListEntries() auf.
		* @param $arrayOfListEntries Ein Array aus ListEntries, wobei jeder ListEntry aus einem assoziativen Array mit den Schlüsseln "amount", "measureName" und "name" bestehen muss.
		* @param $indent Der Einzug.
		* @return string Der Quelltext für die Liste.
		*/
		private function showExistingListEntries($arrayOfListEntries,$indent){
				if (count($arrayOfListEntries) == 1){
					$str = "existiert bereits folgender Eintrag";
				} else {
					$str = "existieren bereits folgende Einträge";
				}
				return 'Es ' . $str . ':</p>' .
				$this->listOfListEntries($arrayOfListEntries,$indent+1);
		}
		
		/**
		* Gibt den Quelltext für eine ungeordnete Liste von ListEntries aus.
		* @param $arrayOfListEntries Ein Array aus ListEntries.
		* @param $indent Der Einzug.
		* @return string Der Quelltext für die Liste.
		*/
		private function listOfListEntries($arrayOfListEntries,$indent){
			// gibt eine Liste bestehender Listartikel aus
			$result = $this->linebreak($indent) . "<ul>";
			foreach($arrayOfListEntries AS $listEntry){
				$result .= $this->linebreak($indent+1) . "<li>" . $listEntry->getAmount() . " " . $listEntry->getMeasureName() . " " . $listEntry->getName(). "</li>";
			}
			$result .= $this->linebreak($indent) . "</ul>";
			return $result;
		}
		
		/**
		* liefert einen Radio-Button mit der Option, den neuen Eintrag zu verwerfen und zur Liste zurückzukehren, zurück.
		* @param $id Die ID des Radio-Buttons.
		* @param $checked Falls nicht leer, wird der Radio-Button als ausgewählt angezeigt.
		* @param $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionLeaveUnchanged($id,$checked,$indent){
			if ($checked != ""){
				$checked = ' checked = "checked"';
			}
			return $this->linebreak($indent) . '<p>' .
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '"'. $checked . ' name="reaction" value="leaveUnchanged">' .
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' .
      		$this->linebreak($indent+2) . 'Den neuen Eintrag verwerfen und zur Liste zurückkehren.' .
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>';
		}
		
		/**
		* liefert einen Radio-Button mit der Option, den bestehenden ListEntry-Eintrag auf die neue Priorität zu setzen, zurück.
		* @param string $priorityName Die bezeichnung der neuen Priorität.
		* @param int $id Die ID des Radio-Buttons.
		* @param string $checked Falls nicht leer, wird der Radio-Button als ausgewählt angezeigt.
		* @param int $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionUpdatePriority($priorityName,$id,$checked,$indent){
			if ($checked != ""){
				$checked = ' checked = "checked"';
			}
			return $this->linebreak($indent) . '<p>' . 
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '"'. $checked . ' name="reaction" value="updatePriority">' . 
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' . 
      		$this->linebreak($indent+2) . ' Den bestehenden Eintrag auf die Dringlichkeitsstufe <q>' . $priorityName . '</q> setzen.' . 
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>';
		}
		
		/**
		* Prüft, ob die Anmerkung eines ListEntries sowie Anmerkungen anderer, in einem Array übergebenen ListEntries identisch ist.
		* @param string $listEntryAnnotation Die Anmerkung eines ListEntries.
		* @param array $listEntryArray Ein Array aus den zu vergleichenden ListEntries inklusive ihrer Anmerkungen.
		* @return bool True, falls die Anmerkungen identisch sind, und false, falls mindestens eine Anmerkung nicht identisch ist.
		*/
		private function sameAnnotations($listEntryAnnotation,$listEntryArray){
			$result = true;
			foreach($listEntryArray AS $element){
				if ($element->getAnnotation() != $listEntryAnnotation){
				$result = false;
				}
				return $result;
			}
		}
		
		/**
		* liefert einen Radio-Button mit der Option, den bestehenden Eintrag mit der aktuellen Anmerkung zu überschreiben, zurück.
		* @param $newAnnotation Die Anmerkung, mit der die bisherige Anmerkung gegebenenfalls überschrieben werden soll.
		* @param $oldAnnotation Die Anmerkung, die gegebenenfalls überschrieben werden soll.
		* @param $id Die ID des Radio-Buttons.
		* @param $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionUpdateAnnotation($newAnnotation,$oldAnnotation,$id,$indent){
			if ($newAnnotation == ""){
				$annotationStr = ' Die bisherige Anmerkung <q>' . $oldAnnotation . '</q> löschen.';
			} else {
				$annotationStr = 'Dem bestehenden Eintrag die aktuelle Anmerkung <q>' . $newAnnotation . '</q> geben.';
			}
			return $this->linebreak($indent) . '<p>' .
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '" name="reaction" value="updateAnnotation">' .
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' .
      		$this->linebreak($indent+2) . $annotationStr .
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>';
		}
		
		/**
		* liefert einen Radio-Button mit der Option, die beiden Einträge mit derselben Maßangabe zu einem neuen Eintrag zusammenfassen, zurück.
		* @param $listEntry Der ListEntry.
		* @param $id Die ID des Radio-Buttons.
		* @param $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionMergeToNewEntry($listEntry,$id,$indent){
			return $this->linebreak($indent) . '<p>' . 
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '" name="reaction" value="mergeToNewEntry">' . 
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' . 
      		$this->linebreak($indent+2) . ' die beiden Einträge mit derselben Maßangabe zu einem neuen Eintrag zusammenfassen:' . 
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>' .
				$this->formularListEntry($listEntry,NULL,"Anmerkungen, die Sie hier machen, werden <emph>nur diesem einen Listeneintrag</emph> angefügt.","merge",$indent);
		}
		
		/**
		* liefert einen Radio-Button mit der Option, alle Einträge (auch die mit anderen Maßangaben!) zu einem neuen Eintrag zusammenfassen, zurück.
		* @param $listEntry Der ListEntry.
		* @param $id Die ID des Radio-Buttons.
		* @param $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionMergeAllToNewEntry($listEntry,$id,$indent){
			return $this->linebreak($indent) . '<p>' . 
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '" name="reaction" value="MergeAllToNewEntry">' . 
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' . 
      		$this->linebreak($indent+2) . ' Alle Einträge (auch die mit anderen Maßangaben!) zu einem neuen Eintrag zusammenfassen:' . 
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>' .
				$this->formularListEntry($listEntry,NULL,"Anmerkungen, die Sie hier machen, werden <emph>nur diesem einen Listeneintrag</emph> angefügt.","mergeAll",$indent);
		}
		
		/**
		* liefert einen Radio-Button mit der Option, den ListEntry-Eintrag als weiteren Eintrag mit anderer Maßeinheit hinzufügen, zurück.
		* @param $listEntry Der ListEntry.
		* @param $newList Ein Array der bisherigen ListEntry-Einträge desselben Namens.
		* @param $id Die ID des Radio-Buttons.
		* @param $checked Falls nicht leer, wird der Radio-Button als ausgewählt angezeigt.
		* @param $indent Der Einzug.
		* @return string Der Quelltext. 
		*/
		private function reactionAddAsNewMeasure($listEntry,$newList,$id,$checked,$indent){
			if ($checked != ""){
				$checked = ' checked = "checked"';
			}
			array_push($newList,$listEntry); // füge das aktuelle Objekt der $newList hinzu
			return $this->linebreak($indent) . '<p>' .
      		$this->linebreak($indent+1) . '<input type="radio" id="' . $id . '"'. $checked . ' name="reaction" value="addAsNewMeasure">' .
      		$this->linebreak($indent+1) . '<label for="' . $id . '" class="highlighted">' .
      		$this->linebreak($indent+2) . ' Als weiteren Eintrag mit anderer Maßeinheit hinzufügen.' .
      		$this->linebreak($indent+1) . '</label>' . 
      		$this->linebreak($indent) . '</p>' .
      		$this->linebreak($indent) . '<p>Das Ergebnis lautet dann:</p>'.
      		$this->listOfListEntries($newList,$indent);
		}		
	}
?>