<?php
	/** Datei, die die Definition der Klasse ListPage enthält. */
	/** 
	* Klasse zur Erstellung der Listen-Seite
	*/
	class ListPage extends Page{
 		/** @var int Die ID der angezeigten Liste. */
		protected $listID;		
		
		/**
		* Der Konstruktor
		* @param string $name Der Name des ListPage-Objektes.
		* @param int $listID Die ID der Liste, der die ListPage zugeordnet ist.
		* @param string $mode Der Seitenmodus. Mögliche Werte: "default" für eine Standardseite und "edit" für eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ ListPage.
		*/ 
		public function __construct($name,$listID,$mode) {
			parent::__construct($name,$mode);
			$this->containsDeletableItems = true;
			$this->listID = $listID;
			$this->indexPage = "list.php?ID=" . $listID;
			$this->editPage = "editList.php?ID=" . $listID;
			$this->editPageSubkat = "editCategory.php";
			$this->headerIcons = $this->getHeaderIcons();
		}
		
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
		* @return string
 		*/	 
		public function start(){
			return parent::start_wp("");
		}
		
		/**
		* Gibt eine Auflistung der ListEntry einer Liste zurück.
		* @param int $listID Die ID der Liste, deren ListEntry ausgegeben werden soll.
		* @param int $indent Der Einzug.
		* @return string Der HTML-Quelltext.
		*/
		public function printListEntries($listID,$indent){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT	
               ' . DB_TAB_PREFIX . 'listEntries.ID, 
					' . DB_TAB_PREFIX . 'listEntries.amount,
					' . DB_TAB_PREFIX . 'listEntries.measureID,
					' . DB_TAB_PREFIX . 'measures.name AS measureName,
					' . DB_TAB_PREFIX . 'listEntries.articleID,
					' . DB_TAB_PREFIX . 'articles.name,
					' . DB_TAB_PREFIX . 'articlesAnnotation.annotation AS articleAnnotation,
					' . DB_TAB_PREFIX . 'listEntries.priorityID,
					' . DB_TAB_PREFIX . 'priority.name AS priorityName,
					' . DB_TAB_PREFIX . 'listEntries.inBasket,
					' . DB_TAB_PREFIX . 'listEntriesAnnotation.annotation
				FROM 
					' . DB_TAB_PREFIX . 'listEntries
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'measures 
						ON 
							' . DB_TAB_PREFIX . 'listEntries.measureID = ' . DB_TAB_PREFIX . 'measures.ID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articles 
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID=' . DB_TAB_PREFIX . 'articles.ID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articlesAnnotation 
						ON 
							' . DB_TAB_PREFIX . 'articlesAnnotation.articleID=' . DB_TAB_PREFIX . 'articles.ID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articlesCategories
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'priority
						ON 
							' . DB_TAB_PREFIX . 'listEntries.priorityID = ' . DB_TAB_PREFIX . 'priority.ID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'categories
						ON 
							' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . DB_TAB_PREFIX . 'categories.ID
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'listEntriesAnnotation 
						ON 
							' . DB_TAB_PREFIX . 'listEntries.ID = ' . DB_TAB_PREFIX . 'listEntriesAnnotation.listEntryID 
				WHERE 
					listID = ' . $listID . ' 
				ORDER BY 
					' . DB_TAB_PREFIX . 'categories.sort ASC, ' . DB_TAB_PREFIX . 'articles.name ASC';
			$query = mysqli_query($db, $selectStr);
			$result = $this->linebreak($indent) . '<div class="list">';
			while($row = mysqli_fetch_object($query)) {
				$listEntry = new ListEntry($row->ID,$row->amount,$row->measureID,$row->measureName,$row->articleID,$row->name,$row->articleAnnotation,$row->annotation,$listID,$row->priorityID,$row->priorityName,$row->inBasket);
				$result .= $this->printDetails("amount",$listEntry->getAmount() . " " . $listEntry->getMeasureName(),$listEntry,$indent+1) .
				$this->printDetails("article",$listEntry->getNameAndAnnotation(),$listEntry,$indent+1) .
				$this->linebreak($indent+1) . "<div class=\"edit\">" .
				$this->printFormIcon(
					"editListEntry.php",
					array("listEntry" => $this->objectToString($listEntry), "formular" => "editListEntry","listID" => $listID),
					"edit",
					"icon listicon",
					$indent+2) .				
				$this->linebreak($indent+1) . "</div>";
			}
			$result .= $this->linebreak($indent) . "</div>";
			return $result;
		}

		
		/**
		* Gibt Details zum ListEntry zurück.
		* Diese Funktion wird zur Ausgabe der Listen auf der Seite list.php verwendet. Sie dient zum einen der Ausgabe von Menge und Maß des ListEntries, zum anderen der Ausgabe des Namens sowie der Anmerkungen zu Artikel und ListEntry.
		* @param string $className Die Bezeichnung der Klasse für das umschließende div.
		* @param string $articleDetails Die Details, die ausgegeben werden sollen.
		* @param array $listEntry Der Listeneintrag.
		* @param int $indent Der Einzug.
		* @return String Details.
		*/
		private function printDetails($className,$articleDetails,$listEntry,$indent){
			$result = $this->linebreak($indent) . "<div class=\"". $className. "\">" .
			$this->startFormular("display: inline;","editListEntry.php","","","POST","","",$indent+1) .
			$this->hiddenFields(array("listEntry" => $this->objectToString($listEntry), "formular" => "convertBasketStatus"),$indent + 2) .
			$this->linebreak($indent + 2) . $this->submitButton("","listEntry " . $listEntry->getStatus(),"Submit","submit","",$articleDetails) .			
			$this->endFormular($indent + 1) . 
			$this->linebreak($indent) . "</div>";
			return $result;
		}
		
		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @return array Array mit den Icon-Angaben
		*/
		protected function getHeaderIcons(){
			$pageUp = array(
				"targetPage" => "editIndex.php",
				"iconTyp" => "pageUp",
			);
			$addToTrolley = array(
				"targetPage" => "newListEntry.php",
				"arrayHiddenFields" => array(
					"listID" => $this->listID,
					"returnURL" => "list.php?ID=" . $this->listID,
				),
				"iconTyp" => "addToTrolley",
			);	
			$deleteTrolley = array(
				"targetPage" => "",
				"arrayHiddenFields" => array(
					"formular" => "deleteListEntriesInBasket",
				),
				"iconTyp" => "deleteTrolley",
			);	

			$leftIcons = array(
				"default" => array($addToTrolley,$deleteTrolley),
				"edit" => array($pageUp),
			);		
			return parent::getHeaderIcons_wp($leftIcons,"");
		}
	}
?>