<?php

	/** Datei, die die Definition der Klasse Listing enthält. */
	/** 
	* Klasse zur Erstellung der Liste
	*/
	class Listing extends SortableListElement { // eine einzelne Liste inklusive ihrer Einträge 
		/** @var array Die Informationen über die Kategorien, die die Funktion addSubCats() ermittelt hat, standardmäßig NULL */  		
		private $categories;
		/** @var array Die Informationen über den Status der zugeordneten ListEntries, die die Funktion statusOfListEntries() ermittelt hat */
		private $statusOfListEntries;

			
		/**
		* Der Konstruktor
		* @param int $ID Die ID des Objektes.
		* @param string $name Der Name des Objektes.
		* @param int $sort Die Position des Objektes in der Sortierreihenfolge.
		* @return object Ein Objekt vom Typ Listing.
		*/ 	
		public function __construct($ID,$name,$sort) {
			include(ABS_PATH . "dbconnect.php");
			parent::__construct($ID,$name,NULL,$sort);
			$this->categories = NULL;
			$this->statusOfListEntries = $this->statusOfListEntries();
			$this->prefix = "list";
			$this->table = DB_TAB_PREFIX . 'lists';
		}

		/**
		* Konstruiert ein Objekt auf der Basis seiner ID
		* @param int $ID Die ID des Objektes.
		* @return object Ein Objekt vom Typ Listing.
		*/ 	
		public function constructByID($ID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = 'SELECT 
				' . DB_TAB_PREFIX . 'lists.name,
				' . DB_TAB_PREFIX . 'lists.sort 
			FROM 
				' . DB_TAB_PREFIX . 'lists 
			WHERE 
				ID LIKE ("' . intval($ID) . '")';
			$query = mysqli_query($db, $selectStr);
			if (mysqli_num_rows($query) > 0){
				$row = mysqli_fetch_object($query);
				return new Listing($ID,$row->name,$row->sort);
			}
			return NULL;
		}
		
				
		/**
		* prüft, ob ein Wert bereits als Name in der Datenbank existiert existiert, und gibt entsprechend true oder false zurück.
		* Für diese Funktion müssen die Eigenschaften table und name implementiert sein.
		* @param string $name Der Name, der auf Existenz geprüft werden soll.
		* @return bool Das Ergebnis, ob der Name bereits in der Datenbank existiert.
		*/
		public function nameAlreadyExists($name){
			return parent::nameAlreadyExists_wsa($name,"");
		}
		
		/**
		* Getter der Variable $statusOfListEntries
		* @return array Inhalt der Variable $statusOfListEntries.
		*/ 
		public function getStatusOfListEntries(){
			return $this->statusOfListEntries;
		}
		
		/**
		* Getter der Variable $categories
		* @return array Inhalt der Variable $categories.
		*/ 
		public function getCategories(){
			return $this->categories;
		}
		
		/**
		* fügt eine weitere Liste in die Datenbank ein, passt die Sortierung der Listen entsprechend an und legt für die neue Liste eine Rubrik "Sonstiges" an.
		* Für die korrekte Funktion muss die Eigenschaft $table definiert sein.
		* @return void
		*/
		public function addToDB(){
			$updateStr = "
				UPDATE " .
					$this->table . "
				SET 
					sort = sort+1 
				WHERE 
					sort >= " . $this->sort; // Platz für die Liste schaffen			
			
			$insertStr = "
				INSERT INTO " .
					$this->table . "
				SET
					name = '" . $this->name . "', 
					sort = " . $this->sort;	
			$arrayStr = 'INSERT INTO ' . DB_TAB_PREFIX . 'categories SET listID = [[thisID]], name = "Sonstiges", sort = 1';
			return parent::addToDB_wp($updateStr,$insertStr,"",$arrayStr);
		}


		/**
		* aktualisiert den entsprechenden Datenbankeintrag.
		* @return void
		*/
		public function updateDB(){
			$updateStr = '
				UPDATE 
					' . DB_TAB_PREFIX . 'lists 
				SET 
					name =' . $this->addBracketsIfNotNULL($this->name) . ',
					sort = ' . $this->addBracketsIfNotNULL($this->sort) . ' 
				WHERE 
					' . DB_TAB_PREFIX . 'lists.ID = ' . $this->ID;
			parent::updateDB_wp($updateStr,NULL,NULL);
		}
		

		
		/**
		* löscht ein Objekt aus der in den Objekteigenschaften genannten $table und passt die Sortierung entsprechend an.
		* Für die korrekte Funktion muss die Eigenschaft $table definiert sein.
		* @return void
		*/
		public function deleteFromDB(){
			$strArray = array();
			
			// Lösche die Kategorien, die der Liste zugeordnet sind
			$deleteCategories = 'DELETE FROM ' . DB_TAB_PREFIX . 'categories WHERE listID = ' . $this->ID;
			array_push($strArray,$deleteCategories);
			
			// Lösche die Zuordnung der Artikel zu den Kategorien, die der Liste zugeordnet sind
			$categoryArray = array();
			foreach($this->getCategories() AS $category){
				array_push($categoryArray,$category->getID());
			}
			if (count($categoryArray) > 0){
				$deleteArticlesCategories = 'DELETE FROM ' . DB_TAB_PREFIX . 'articlesCategories WHERE catID IN (' . implode(",",$categoryArray) . ')'; 
				array_push($strArray,$deleteArticlesCategories);
			}
		 	parent::deleteFromDB_wp(NULL,NULL,NULL,$strArray);
		}
		
		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach unten
		* @return void
		*/
		public function sortDown(){
			parent::sortDown_wp(NULL);
		}
		
		/**
		* sortiert eine Subkategorie in der Reihenfolge um eins nach oben
		* @return void
		*/
		public function sortUp(){
			parent::sortUp_wp(NULL);
		}

		
		/**
		* Sucht alle Warenkategorien einer Liste und gibt sie in einem Array zurück.
		* @return array Die Warenkategorien einer Liste als Array.
		*/
		public function addSubCats(){
			/* Sucht alle Warenkategorien einer Liste und gibt sie in einem Array zurück */
			include(ABS_PATH . "dbconnect.php");
			$selectStr = 'SELECT 
				' . DB_TAB_PREFIX . 'categories.ID
				FROM 
					' . DB_TAB_PREFIX . 'categories 
				WHERE 
					listID LIKE ("' . intval($this->ID) . '") 
				ORDER BY 
					sort ASC';
			$query = mysqli_query($db, $selectStr);
			$result = array();
			while($row = mysqli_fetch_object($query)){
				$category = Category::constructByID($row->ID);
				array_push($result,$category);
			}
				$this->categories = $result; 
		}
		
		/**
		* gibt ein Array zurück, das je in einem eigenen Schlüssel (1) die IDs der in den Warenkorb gelegten und (u.a.) der entsprechenden Liste zugeordneten Listeneinträge, (2) die ihnen zugehörigen ArticleIDs, (3) die IDs der erstmalig gekauften Artikel sowie, sofern für den entsprechenden Artikel ein Intervall oder ein fester Kauftag definiert sind, (4) ein Array mit der ArticleID und dem nächsten Kaufdatum enthält .
		* @return array Ein Array mit den Schlüsseln "IDs", "articleIDs", "firstBuy" und "nextBuy".
		*/
		private function findListEntriesInBasket(){
			include(ABS_PATH . "dbconnect.php");
			$result = array(
				"delete_IDs" => array(),
				"convert_IDs" => array(),
				"articleIDs" => array(),
				"firstBuy" => array(),
				"nextBuy" => array(),
			);
			$selectStr = '
				SELECT
					' . DB_TAB_PREFIX . 'listEntries.ID,
					' . DB_TAB_PREFIX . 'listEntries.articleID,
					' . DB_TAB_PREFIX . 'articles.firstBuy,
					' . DB_TAB_PREFIX . 'articles.nextBuy,
					' . DB_TAB_PREFIX . 'articlesConvertToArticle.convertToArticleID,
					' . DB_TAB_PREFIX . 'articlesIntervalInDays.intervalInDays,
					' . DB_TAB_PREFIX . 'articlesBuyEvery.weekday
				FROM 	
					' . DB_TAB_PREFIX . 'listEntries 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articles 
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID=' . DB_TAB_PREFIX . 'articles.ID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articlesConvertToArticle 
						ON 
							' . DB_TAB_PREFIX . 'articles.ID=' . DB_TAB_PREFIX . 'articlesConvertToArticle.articleID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articlesIntervalInDays 
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID=' . DB_TAB_PREFIX . 'articlesIntervalInDays.articleID 
				LEFT JOIN 
					' . DB_TAB_PREFIX . 'articlesCategories
						ON 
							' . DB_TAB_PREFIX . 'listEntries.articleID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID		
				LEFT JOIN
					' . DB_TAB_PREFIX . 'categories
						ON 
							' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . DB_TAB_PREFIX . 'categories.ID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesBuyEvery
						ON 
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesBuyEvery.articleID
				WHERE 
					listID = ' . $this->ID . ' AND inBasket IS NOT NULL';
			$query = mysqli_query($db, $selectStr);
			while($row = $query->fetch_array(MYSQLI_ASSOC)){
				if (empty($row['convertToArticleID'])){ // wenn Artikel nicht konvertiert werden soll
					if (!in_array($row['ID'],$result['delete_IDs'])){
						array_push($result['delete_IDs'],$row['ID']);
					}
				} else { // wenn Artikel konvertiert werden soll
					if (!in_array($row['ID'],$result['convert_IDs'])){
						array_push($result['convert_IDs'],array("ID" => $row['ID'], "to_articleID" => $row['convertToArticleID']));
					}
				}

				if (! in_array($row['articleID'],$result['articleIDs'])){
					array_push($result['articleIDs'],$row['articleID']);
				}
				if ($row['firstBuy'] == NULL){
					array_push($result['firstBuy'],$row['articleID']);
				}
				
				// Definition von NextBuy
				if ($row['intervalInDays'] != NULL OR $row['weekday'] != NULL){ // wenn nextBuy ermittelt werden muss, weil intervalInDays oder articlesBuyEvery.weekday definiert sind
					
					// Definition von $nextBuyIntervalInDays					
					if ($row['intervalInDays'] != NULL){
						if (is_null($row['nextBuy'])){ // wenn Datum für NextBuy nicht definiert ist
							$nextBuyIntervalInDays = date("Y-m-d", strtotime("+" . $row['intervalInDays'] . " days")); // Datum von heute plus intervalInDays
						} else {
							$nextBuyIntervalInDays = date("Y-m-d", strtotime($row['nextBuy'] . "+" . $row['intervalInDays'] . " days")); // Datum von heute plus intervalInDays
						}
					} else {
						$nextBuyIntervalInDays = NULL;
					}
					
					// Definition von $nextBuyWeekday
					if ($row['weekday'] != NULL){
						$weekdays = explode(", ", $row['weekday']);
						if (is_null($row['nextBuy'])){ // wenn Datum für NextBuy nicht definiert ist
							$nextBuyWeekday = date("Y-m-d", strtotime("+" . Page::selectShortestNumberOfDaysToNextBuy("now",$weekdays) . " days")); // Datum von heute plus Zeit bis zum nächsten Wochentag 
						} else {
							$nextBuyWeekday = date("Y-m-d", strtotime($row['nextBuy'] . "+" . Page::selectShortestNumberOfDaysToNextBuy($row['nextBuy'],$weekdays) . " days")); // Datum von nextBuy plus Zeit bis zum nächsten Wochentag 
						}
					} else {
						$nextBuyWeekday = NULL;
					}
					
					// Definition von $dateOfNextBuy
					if (! is_null($nextBuyIntervalInDays) AND ! is_null($nextBuyWeekday)){ // wenn beide definiert sind
						if ($nextBuyIntervalInDays < $nextBuyWeekday){
							$dateOfNextBuy = $nextBuyIntervalInDays;
						} else {
							$dateOfNextBuy = $nextBuyWeekday;
						}
					} elseif(! is_null($nextBuyIntervalInDays)) {
							$dateOfNextBuy = $nextBuyIntervalInDays;
					} elseif(! is_null($nextBuyWeekday)) {
							$dateOfNextBuy = $nextBuyWeekday;
					}

					array_push($result['nextBuy'],array("ID" => $row['articleID'],"date" => $dateOfNextBuy));
				}
				
			}
			return $result;				
		}

		

		/**
		* Löscht die in den Warenkorb gelegten und (u.a.) der entsprechenden Liste zugeordneten ListArtikels und setzt die Eigenschaft lastBought in der Tabelle articles für die entsprechenden Artikel auf das aktuelle Datum.
		* @return void
		*/
		public function deleteListEntriesInBasket(){
			include(ABS_PATH . "dbconnect.php");
			$articlesInBasket = $this->findListEntriesInBasket();			
			// Lösche den Listeneintrag, sofern Artikel nicht konvertiert werden soll
			if (!empty($articlesInBasket['delete_IDs'])){
				$deleteStr1 = '
					DELETE 
						' . DB_TAB_PREFIX . 'listEntries 
					FROM 
						' . DB_TAB_PREFIX . 'listEntries 
					WHERE 
						ID 
							IN ('. implode(",", $articlesInBasket['delete_IDs']) . ")";
				// Lösche ggf. vorhandene Anmerkungen zum Listeneintrag, sofern Artikel nicht konvertiert werden soll
				$deleteStr2 = '
					DELETE 
						' . DB_TAB_PREFIX . 'listEntriesAnnotation 
					FROM 
						' . DB_TAB_PREFIX . 'listEntriesAnnotation 
					WHERE 
						listEntryID 
							IN (' . implode(",", $articlesInBasket['delete_IDs']) . ')';
				mysqli_query($db, $deleteStr1);
				mysqli_query($db, $deleteStr2);
			} 

			// Aktualisiere für jeden gekauften Artikel das Datum des letzten Kaufes
			if (!empty($articlesInBasket['articleIDs'])){
				$updateArticle = '
					UPDATE
						' . DB_TAB_PREFIX . 'articles
					SET
						lastBuy = "' . date("Y-m-d") . '",
						numberOfPurchases = numberOfPurchases+1 
					WHERE 
						ID 
							IN ('. implode(",", $articlesInBasket['articleIDs']) . ')';
				mysqli_query($db, $updateArticle);
			}

						
			// Setze das Datum für den ersten Kauf, sofern noch nicht gesetzt
			if (!empty($articlesInBasket['firstBuy'])){
				$updateFirstBuy = '
					UPDATE
						' . DB_TAB_PREFIX . 'articles
					SET
						firstBuy = "' . date("Y-m-d") . '"
					WHERE
						ID 
							IN ('. implode(",", $articlesInBasket['firstBuy']) . ')';
				mysqli_query($db, $updateFirstBuy);
			}	

			// Konvertiere Artikel
			foreach($articlesInBasket['convert_IDs'] AS $article){ // Artikel in Tabelle articlesBought eintragen
				$convertStr = '
					UPDATE 
						' . DB_TAB_PREFIX . 'listEntries
					SET
						articleID = "' . $article['to_articleID'] . '",
						inBasket = NULL 
					WHERE
						ID = "' . $article['ID'] . '"';
			 	mysqli_query($db, $convertStr);
			}
			

			foreach($articlesInBasket['articleIDs'] AS $articleID){ // Artikel in Tabelle articlesBought eintragen
				$insertStr = '
					INSERT INTO 
						' . DB_TAB_PREFIX . 'articlesBought
					SET
						articleID = "' . $articleID . '",
						date = "' . date("Y-m-d") . '"';
			 	mysqli_query($db, $insertStr);
			}
			
			foreach($articlesInBasket['nextBuy'] AS $article){ //nextBuy aktualisieren
				$updateNextBuy = '
					UPDATE
						' . DB_TAB_PREFIX . 'articles
					SET
						nextBuy = "' . $article['date'] . '"
					WHERE 
						ID = "' . $article['ID'] . '"';
			 	mysqli_query($db, $updateNextBuy);
			}
		}

		/**
		* Überprüft den Status der einer Liste zugeordneten ListEntry.
		* Die Funktion gibt die Zahl der gekauften und der ungekauften ListEntries sowie die Anzahl der ungekauften ListEntries mit geringer, normaler und hoher Priorität in einem Array zurück. Die Anzahl  der gekauften ListEntries kann über $status['inBasket'] abgerufen werden, die der ungekauften ListEntries über $status['notInBasket']. Die Anzahl der Listeneinträge mit geringer Priorität wird in $status ['lowPriority'] zurückgegeben, die mit Standardpriorität in $status ['standardPriority'], die mit hoher Priorität in $status['highPriority'] .
		* @return array Status der ListEntries.
		*/
		private function statusOfListEntries(){
			if ($this->ID != ""){
				include(ABS_PATH . "dbconnect.php");
				$lowPriority = 0;
				$standardPriority = 0;
				$highPriority = 0;
				$inBasket = 0;
				$notInBasket = 0;
				$selectStr = '
					SELECT
						' . DB_TAB_PREFIX . 'listEntries.priorityID, 
						' . DB_TAB_PREFIX . 'listEntries.inBasket 
					FROM 
						' . DB_TAB_PREFIX . 'listEntries
					LEFT JOIN 
						' . DB_TAB_PREFIX . 'articlesCategories
							ON 
								' . DB_TAB_PREFIX . 'listEntries.articleID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID
					LEFT JOIN
						' . DB_TAB_PREFIX . 'categories
							ON 
								' . DB_TAB_PREFIX . 'articlesCategories.catID = ' . DB_TAB_PREFIX . 'categories.ID
					WHERE 
						listID = ' . $this->ID;
				$query = mysqli_query($db, $selectStr);
				while($row = mysqli_fetch_object($query)){
					if ($row->inBasket == "1"){
						$inBasket++;
					} else {
						$notInBasket++;
						if($row->priorityID == 1){
							$lowPriority++;
						} elseif($row->priorityID == 2){
							$standardPriority++;
						} elseif($row->priorityID == 3){
							$highPriority++;
						}	
					}		
				}
				$result = array(
					"inBasket" => $inBasket,
					"notInBasket" => $notInBasket,
					"lowPriority" => $lowPriority,
					"standardPriority" => $standardPriority,
					"highPriority" => $highPriority,
					); 
				return $result;
			}
		}
	}
	
?>