<?php 
	/** Datei, die die Definition der Klasse Main enthält. */
	/**
 	* definiert zentrale Elemente
 	*/
	class Main {
	 /** @var string Der Name des Elementes. */
		protected $name;
		
		/**
		* Der Konstruktor
		* @param string $name Der Name
		* @return object Ein Objekt vom Typ Main
		*/ 
		public function __construct($name){
			$this->name = $name;
		}
		
		/**
		* Setter der Variable $name
		* @param string $name Der Name
		* @return void
		*/ 
		public function setName($name){
			$this->name = $name;
		}
			
		/**
		* Getter der Variable $name
		* @return string Inhalt der Variable $name
		*/ 
		public function getName(){
			return $this->name;
		}
	
		/**
  		* prüft, ob ein Wert NULL ist, und gibt entweder " IS NULL" oder " = $string" zurück.
  		* @param string Der übergebene String .
  		* @return string Der überarbeitete String.
  		*/
		protected function checkIfNULL($string){
			if (is_null($string) OR empty($string)){
				return " IS NULL";
			} else {
				return ' = \'' . $string . '\'';
			}
		}
		
				
		/**
  		* setzt einen String in einfache Hochkommata, sofern er nicht den Wert "NULL" bzw. leer hat; anderfalls wird NULL zurückgegeben.
  		* @param string Der übergebene String .
  		* @return string Der überarbeitete String.
  		*/
		protected function addBracketsIfNotNULL($string){
			if (empty($string) OR $string == "null"){
				return "NULL";
			} else {
				return '\'' . $string . '\'';
			}
		}
		
		/**
		* Gibt NULL zurück, falls das übergebene Array leer ist, andernfalls $array
		* @param array $array Das übergebene Array.
		* @return null
		*/		
		protected function setArrayToNullIfEmpty($array){
			$array = array_filter($array);
			if (!empty($array)) {
				return $array;
			} else {
				return NULL;
			}
		}
		
				
		/**
  		* gibt NULL zurück, wenn der übergebene $string "0" ist, andernfalls den übergebenen String.
  		* @param string Der übergebene String .
  		* @return string Der überarbeitete String.
  		*/

		public function isNullIfZero($string){
			if ($string == "="){
				return NULL;
			} 
			else {
				return $string;
			}
		}
		
		/**
		* Escaped jeden Schlüssel und jeden Wert eines zweidimensionalen Arrays.
		* @param array $array Das zu escapende Array.
		* 
		*/
		public function escapeArray($array){
			include(ABS_PATH . "dbconnect.php");
			$result = array();
 			foreach ($array AS $key=>$value ) {
    			$result[mysqli_real_escape_string($db,$key)] = mysqli_real_escape_string($db,$value);
			}
			return $result;
		}
		
		/**
		* konvertiert ein Array in einen String.
		* @param array $array Das zu konvertierende Array.
		* @return string Der konvertierte String.
		*/
		public function arrayToString($array){
			return base64_encode(serialize($array));
		}
		
		/**
		* konvertiert ein Objekt in einen String.
		* @param object $object Das zu konvertierende Objekt.
		* @return string Der konvertierte String.
		*/
		public function objectToString($object){
			return Main::arrayToString($object);
		}
		
		/**
		* konvertiert einen String in ein Array.
		* @param string $string Der zu konvertierende String.
		* @return array Das konvertierte Array.
		*/
		public function stringToArray($string){
			return unserialize(base64_decode($string));
		}
		
		/**
		* konvertiert einen String in ein Objekt.
		* @param string $string Der zu konvertierende String.
		* @return object Das konvertierte Objekt.
		*/
		public function stringToObject($string){
			return Main::stringToArray($string);
		}
		
		
		/** Vergleicht den Inhalt zweier Arrays und gibt das Ergebnis als Array zurück.
		* @param $array1 Das erste zu vergleichende Array.
		* @param $array2 Das zweite zu vergleichende Array.
		* @return array Ein Array mit den Schlüsseln "match", "notIn1st" und "notIn2nd".
		*/
		public function compareArrayContent($array1,$array2){
			$match = array();
			$notIn1st = array();
			$notIn2nd = array();

			foreach ($array1 AS $key=>$value){
				if (in_array($value, $array2)){ // wenn $value auch in $array2 enthalten ist 
					if (!in_array($value,$match)){ // wenn $value nicht schon in $match enthalten is	t
						array_push($match,$value); // füge $value zu $match hinzu
					}
				} else { // wenn $value nicht in array2 enthalten ist
					if (!in_array($value,$notIn2nd)){ // wenn $value nicht schon in $notIn2nd enthalten ist
						array_push($notIn2nd,$value);
					}
				}
			}
			
			foreach ($array2 AS $key=>$value){
				if (in_array($value, $array1)){ // wenn $value auch in $array1 enthalten ist 
					if (!in_array($value,$match)){ // wenn $value nicht schon in $match enthalten ist
						array_push($match,$value); // füge $value zu $match hinzu
					}
				} else { // wenn $value nicht in array1 enthalten ist
					if (!in_array($value,$notIn1st)){ // wenn $value nicht schon in $notIn1st enthalten ist
						array_push($notIn1st,$value);
					}
				}
			}
			return array(
				"match" => $match,
				"notIn1st" => $notIn1st,
				"notIn2nd" => $notIn2nd,
			);
		}
		
		/**
		* löscht sämtliche Leerzeichen am Anfang und Ende einer Zeichenkette 
		* @param string $str Der übergebene String
		* @return string
		*/	
		protected function deleteSpaces($str){
			// löscht sämtliche Leerzeichen am Anfang und Ende einer Zeichenkette 
			while(substr($str,-1) == " "){ // solange letztes Zeichen ein Leerzeichen ist
				$str = substr($str,0,-1);	// schneide letztes Zeichen ab
			}	
			while(substr($str,0,1) == " "){ // solange erstes Zeichen ein Leerzeichen ist
				$str = substr($str,1);	// schneide erstes Zeichen ab
			}
			return $str;
		}
		
		/**
		* Entfernt die Klammern aus einem String, sofern die öffnende Klammer an erster und die schließende Klammer nach Entfernung führender Leerzeichen an letzter Stelle des Strings steht.
		* @param string Der zu überarbeitete String.
		* @return string Der überarbeitete String.
		*/
		protected function deleteBrackets($string){
			$string = $this->deleteSpaces($string);
			if(substr($string,0,1) == "(" AND substr($string,-1) == ")"){
				$string = substr($string,1);	// schneide erstes Zeichen ab
				$string = substr($string,0,-1);	// schneide letztes Zeichen ab
			}
			return $string;
		}
		
		/**
		* trennt einen String im Falle von sich öffnenden Klammern auf und gibt ihn als Array zurück
		* @param string $str Der übergebene String.
		* @return array Das Array.
		*/			
		protected function extractContentFromBrackets($str){
			if ($str == ""){
				return NULL;
			} else {
				if (substr($str,0,1) == "("){ // falls erstes Zeichen eine öffnende Klammer ist
					$start = 0 ;
				} else {
					$start = 1;
				}
				$result = explode("(",$str); // zerlegen des Strings nach öffnender Klammer
				for ($i = $start; $i< count($result); $i++){ // Klammer wieder hinzufügen
					if ($result[$i] == ""){
						unset($result[$i]); 
					} else {
						$result[$i] = "(" . $result[$i];
					}
				}
				return $result;
			}
		}
		/**
		* stellt den Schlüsseln eines Arrays ein Präfix voran.
		* @param string $prefix Das Präfix.
		* @param array $array Das Array, dessen Schlüsseln das Präfix des Objektes vorangestellt werden soll.
		* @return array Das überarbeitete Array.
		*/
		protected function addPrefix($prefix,$array){
			foreach($array as $key => $value) {
				$array[$prefix . '_' .  $key] = $array[$key];
				unset($array[$key]);
			}
			return $array;
		}
		
		/**
		* gibt aus einem Array von Objekten die IDs der Objekte @@-separiert zurück
		* @param array $array Das übergebene Array aus Objekten.
		* @return string
		*/	
		protected function ArrayElementIDs($array){
			$return = array();
			foreach($array AS $element){
				if ($element->getID() != ""){
					array_push($return,$element->getID());
				}
			}
			$return = implode("@@",$return);
			return $return;
		}
		
		/**
		* gibt den in der MySQL-Tabelle prefs hinterlegten Wert für standardAmount zurück, sofern $amount NULL ist.
		* @param int $amount Die Angaben zur Menge.
		* return int Der in der MySQL-Tabelle prefs hinterlegten Wert für standardAmount.
		*/
		public function getPrefAmountIfNULL($amount){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'prefs.standardAmount
				FROM ' . DB_TAB_PREFIX . 'prefs WHERE ID=1';
			
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if ($amount == NULL){
				return $row->standardAmount;
			} else {
				return $amount;
			}
		}
		
		/**
		* gibt den in der MySQL-Tabelle prefs hinterlegten Wert für standardMeasureID zurück, sofern $measureID NULL ist.
		* @param int $measureID Die ID der Maßangabe.
		* @return int Der in der MySQL-Tabelle prefs hinterlegten Wert für standardMeasureID.
		*/
		public function getPrefStandardMeasureIDIfNULL($measureID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'prefs.standardMeasureID
				FROM ' . DB_TAB_PREFIX . 'prefs WHERE ID=1';
			
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if ($measureID == NULL){
				return $row->standardMeasureID;
			} else {
				return $measureID;
			}
		}
		
		/**
		* gibt den in der MySQL-Tabelle prefs hinterlegten Wert für standardPriorityID zurück, sofern $priorityID NULL ist.
		* @param int $priorityID Die ID der Priorität.
		* return int Der in der MySQL-Tabelle prefs hinterlegten Wert für standardPriorityID.
		*/
		public function getPrefStandardPriorityIDIfNULL($priorityID){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'prefs.standardPriorityID
				FROM ' . DB_TAB_PREFIX . 'prefs WHERE ID=1';
			
			$query = mysqli_query($db, $selectStr);
			$row = mysqli_fetch_object($query);
			if ($priorityID == NULL){
				return $row->standardPriorityID;
			} else {
				return $priorityID;
			}
		}
		
		
		/**
		* führt eine Datenbankabfrage aus, wobei der Parameter sowohl ein einzelner String als auch ein Array aus Datenbankabfragen sein kann.
		* @param array $query Die Datenbankabfrage(n).
		* @return void.
		*/
		protected function dbquery($query){
			include(ABS_PATH . "dbconnect.php");
			if (is_array($query)){ // wenn es wirklich ein Array ist
				foreach($query AS $str){
					mysqli_query($db, $str);
				}
			} elseif (is_string($query)) { // wenn es ein String ist
				mysqli_query($db, $query);
			}
		}
	}
?>