<?php

	/** Datei, die die Definition der Klasse Page enthält. */
	
	/**
	* bündelt zentrale Funktionen, die zur Seitenerstellung notwendig sind. Hierzu zählen beispielsweise Funktionen für Quellcodeausgabe, zur Ausgabe von Header und Footer sowie zur Erstellung von Formularen, Buttons und Icons.
	*/
	class Page extends Main {
		/** @var int Der Zeileneinzug. */
		protected $indent;
		/** @var int Der Zeileneinzug für den Seiteninhalt (Ebene nach dem content-div). */
		protected $contentIndent;
		/** @var string Der Seitenmodus. Mögliche Werte: "default" für eine Standardseite und "edit" für eine Bearbeitungsseite. */		
		protected $mode;
		/** @var boolean Gibt an, ob die Seite löschbare Inhalte enthält. Falls true, wird auf der EditSeite ein Lösch-Icon angezeigt. */		
		protected $containsDeletableItems;
		/** @var string Der Dateiname der Indexseite. */		
		protected $indexPage;
		/** @var string Der Dateiname der Bearbeitungsseite. */		
		protected $editPage;
		/** @var string Der Dateiname der Bearbeitungsseite der Unterkategorie. */		
		protected $editPageSubkat;
		/** @var array Die Informationen zu den Icons, die im Header eingeblendet werden sollen. */		
		protected $headerIcons;
		
		/**
		* Der Konstruktor
		* @param string $name Der Name.
		* @param string $mode Der Modus. Mögliche Werte sind "default" und "edit". "Default" gibt eine Standardseite aus, "edit" eine Bearbeitungsseite.
		* @return object Ein Objekt vom Typ Page.
		*/ 
		public function __construct($name,$mode) {
			if ($mode == "edit"){
				$name = $name . ' bearbeiten';
			}
			parent::__construct($name);
			$this->indent = 0;
			$this->contentIndent = 3;
			$this->mode = $mode;
			$this->containsDeletableItems = false;
			$this->indexPage = NULL;
			$this->editPage = NULL;
			$this->headerIcons = $this->getHeaderIcons_wp(NULL,NULL);
		}
		
		
		/**
		* Setter der Variable $indent
		* @param int $indent Der Zeileneinzug
		* @return void
		*/ 
		public function setIndent($indent){
			$this->indent = $indent;
		}

		/**
		* Getter der Variable $indent
		* @return int Inhalt der Variable $indent.
		*/ 
		public function getIndent(){
			return $this->indent;
		}
		
		/**
		* Setter der Variable $contentIndent
		* @param int $contentIndent Der Zeileneinzug
		* @return void
		*/ 
		public function setContentIndent($contentIndent){
			$this->contentIndent = $contentIndent;
		}

		/**
		* Getter der Variable $contentIndent
		* @return int Inhalt der Variable $contentIndent.
		*/ 
		public function getContentIndent(){
			return $this->contentIndent;
		}
		
		/**
  		* sorgt für einen Zeilenumbruch, wobei $indent die Tiefe des Einzug angibt
  		* @param int $indent Die Tiefe des Einzugs nach dem Zeilenumbruch.
		* @return string
 		*/	 
		protected function linebreak($indent){
			$result = "\n" . $this->indent($indent);
			return $result;
		}
		
		
		/**
  		* sorgt für einen Einzug im Quelltext
  		* @param int $indent Die Einzugstiefe
		* @return string
 		*/	 
		protected function indent($indent){
			$result = "";
			for ($i = 0; $i < $indent; $i++){
			 	$result = $result . "	";
			}
			return $result;
		}
		
		/**
  		* Gibt den HTML-Head inklusive der DOCTYPE-Angaben zurück.
  		* @param array $headData Ein Array optionaler zusätzlicher Angaben, die in den Head eingebunden werden sollen.
  		* @param int $indent Die Einzugtiefe.
		* @return string Der HTML-Head.
 		*/	 
		protected function head($headData,$indent){
			$addData = "";

			if (!empty($headData)){
				foreach($headData AS $element){
					$addData .= $this->linebreak($indent+2) . $element;
				}
			}
			$result = $this->indent($indent) . '<!DOCTYPE html>' . 
			$this->linebreak($indent) . '<html lang="de">' .
			$this->linebreak($indent+1) . '<head>' . 
			$this->linebreak($indent+2) . '<title>' . 
			$this->linebreak($indent+3) . $this->name . 
			$this->linebreak($indent+2) . '</title>' . 
			$this->linebreak($indent+2) . '<link rel="stylesheet" type="text/css" href="' . URL . 'css/style.css">' . 
			$addData .
			$this->linebreak($indent+1) . '</head>';
			return $result;
		}
		
		/**
  		* gibt den Header der Seite inklusive der Nagigation zurück
  		* @param int $indent Die Tiefe des Einzugs nach dem Zeilenumbruch
		* @return string
 		*/	 
		function header($indent){
			// gibt den Header aus
			$left = "";
			$right = "";
			if (isset($this->headerIcons['left'])){
				foreach($this->headerIcons['left'] AS $leftIcon){
					if (isset($leftIcon['arrayHiddenFields'])){
						$arrayHiddenFields = $leftIcon['arrayHiddenFields'];
					} else {
						$arrayHiddenFields = NULL;
					}
					$left .= $this->printFormIcon($leftIcon['targetPage'],$arrayHiddenFields,$leftIcon['iconTyp'],"icon headericon left",$indent+2);
				}
			}

			if (isset($this->headerIcons['right'][0]) AND is_array($this->headerIcons['right'][0])){
				$right = $this->linebreak($indent+1) . "<span style=\"float:right;\">";
				foreach($this->headerIcons['right'] AS $rightIcon){
					$right .= $this->printFormIcon($rightIcon['targetPage'],$rightIcon['arrayHiddenFields'],$rightIcon['iconTyp'],"icon headericon right",$indent+2);
				}
				$right .= $this->linebreak($indent+1) . "</span>";
			}
			$result = $this->linebreak($indent) . '<div class="header">' . 
			$left .
			$right . 
			$this->linebreak($indent) . '</div>' . 
			$this->linebreak($indent) . '<div class="cleardiv"></div>';;
			return $result;
		}
		
		/**
  		* gibt Head, Header und Seitentitel (<h1>) zurück und leitet die Ausgabe des eigentlichen Seiteninhaltes ein. Das mit dieser Funktion eingeleitete Markup kann abschließend mit der Funktion end() geschlossen werden.
  		* @param array $headData Ein Array optionaler zusätzlicher Angaben, die in den Head eingebunden werden sollen.
		* @return string
 		*/	 
		public function start_wp($headData){
			return $this->head($headData,$this->indent) . 
			$this->linebreak($this->indent+1) . '<body>' . 
			$this->header($this->indent+2) . 
			$this->linebreak($this->indent+2) . '<div class="content">' . 
			$this->linebreak($this->contentIndent) . '<h1>' .
			$this->linebreak($this->contentIndent+1) . $this->name . 
			$this->linebreak($this->contentIndent) . '</h1>';
		}
		
		/**
  		* gibt den Fuß der Seite inklusive der Navigation zurück
  		* @param int $indent Die Tiefe des Einzugs nach dem Zeilenumbruch
		* @return string
 		*/	
		protected function footer($indent){
			// Gibt den Fußbereich aus
			$result = $this->indent($indent) . '<div class ="footer">' . 
			$this->linebreak($indent+2). '<a href="' . URL. 'doc/index.php">' .
			$this->linebreak($indent+3). '<img src="' . $this->getIcon("help"). '" alt="manual" class="footericon left">' .
			$this->linebreak($indent+2). '</a>' . 
			$this->linebreak($indent+1) . '<span style="float:right;">' .
			$this->linebreak($indent+2). '<a href="' . URL . 'about.php">' .
			$this->linebreak($indent+3). '<img src="' . $this->getIcon("about"). '" alt="about" class="footericon right">' .
			$this->linebreak($indent+2). '</a>' . 
			$this->linebreak($indent+1) . '</span>' . 
			$this->linebreak($indent) . '</div>';
			return $result;
		}
		
		/**
  		* leitet den Abschluss des Seiteninhaltes ein, gibt den Footer zurück und schließt das mit der Funktion start($indent) eingeleitete Markup.
  		* @param int $indent Der Einzug.
		* @return string
 		*/
		public function end(){
			return $this->linebreak($this->indent+2) . '</div>' . 
			$this->linebreak($this->indent) . $this->footer($this->indent+2) . 
			$this->linebreak($this->indent+1) . '</body>' . 
			$this->linebreak($this->indent) . '</html>';
		}


		/** gibt eine Warnung zurück, falls Artikel keiner Listenkategorie zugeordnet sind.
		 * @param int $indent Der Einzug.
		* @return string Der Quelltext.
		*/
		public function warningIfArticlesWithoutCategory($indent){
			include(ABS_PATH . "dbconnect.php");
			$result = "";
			$selectStr = 'SELECT ' . DB_TAB_PREFIX . 'articles.ID, ' . DB_TAB_PREFIX . 'articles.name FROM ' . DB_TAB_PREFIX . 'articles LEFT JOIN ' . DB_TAB_PREFIX . 'articlesCategories ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID WHERE ' . DB_TAB_PREFIX . 'articlesCategories.catID IS NULL';
			$query = mysqli_query($db, $selectStr);
			$numberOfArticles = $query->num_rows;
			if ($numberOfArticles > 0){
				if ($numberOfArticles == 1){
					$der_die = "der";
					$ist_sind = "ist";
				} else {
					$der_die = "die";
					$ist_sind = "sind";
				}

				$result = $this->linebreak($indent) . '<p class="warning">' .
				$this->linebreak($indent+1) . "Achtung! Diese " . $this->name . " enthält " . $numberOfArticles . " Artikel, " . $der_die . " keiner Listenkategorie zugeordnet ". $ist_sind . '! Bitte <a href="articlesWithoutCats.php">bearbeiten</a> Sie dieses Problem.' .
				$this->linebreak($indent) . '</p>';
			}
			return $result; 
		}
		


		/** zeigt die Artikel an, die keiner Listenkategorie zugeordnet sind.
		* @param int $indent Der Einzug.
		* @return string Der Quelltext.
		*/
		public function showArticlesWithoutCategory($indent){
			include(ABS_PATH . "dbconnect.php");
			$result = "";
			$selectStr = 'SELECT ' . DB_TAB_PREFIX . 'articles.ID, ' . DB_TAB_PREFIX . 'articles.name FROM ' . DB_TAB_PREFIX . 'articles LEFT JOIN ' . DB_TAB_PREFIX . 'articlesCategories ON ' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesCategories.articleID WHERE ' . DB_TAB_PREFIX . 'articlesCategories.catID IS NULL';
			$query = mysqli_query($db, $selectStr);
			$numberOfArticles = $query->num_rows;
			if ($numberOfArticles > 0){
				if ($numberOfArticles == 1){
					$er_sie = "er";
					$der_die = "der";
					$ist_sind = "ist";
					$diesem_diesen = "diesem";
					$artikel_dativ = "Artikel";
					$ihn_sie = "ihn";
					$kann_koennen = "kann";
				} else {
					$er_sie = "sie";
					$der_die = "die";
					$ist_sind = "sind";
					$diesem_diesen = "diesen";
					$artikel_dativ = "Artikeln";
					$ihn_sie = "sie";
					$kann_koennen = "können";
				}
				$articles = "";		
				while($row = mysqli_fetch_object($query)){
					$articles .= $this->linebreak($indent+1) . '<li>' .
					$this->linebreak($indent+2) . '<a href="editArticle.php?ID=' . $row->ID. '&catID=0">' . $row->name . '</a>' . 
					$this->linebreak($indent+1) . '</li>';
				};
							
				$result = $this->linebreak($indent) . "<p>" .
				$this->linebreak($indent+1) . "Diese " . $this->name . " enthält " . $numberOfArticles . " Artikel, " . $der_die . " keiner Listenkategorie zugeordnet ". $ist_sind . ":" .
				$this->linebreak($indent) . '</p>' .
				$this->linebreak($indent) . '<ul class="bulletlist">' .
				$articles . 
				$this->linebreak($indent) . "</ul>" .
				$this->linebreak($indent) . "<p>" .
				$this->linebreak($indent+1) . "Sie müssen  " . $diesem_diesen . " " . $artikel_dativ . " mindestens eine Listenrubrik zuordnen, damit " . $er_sie . " in der " . $this->name . " korrekt angezeigt werden " . $kann_koennen . ". Alternativ können sie " . $ihn_sie . " auch löschen." .  
				$this->linebreak($indent) . "</p>";
			}
			return $result; 
		}

		/**
  		* gibt einen Informationstext und ein bis zwei Submit-Button zurück, wobei der obligatorische Submit-Button immer die Bezeichnung "zurück" trägt.
  		* @param string $info Der Informationstext.
  		* @param string $hiddenFieldsForm1 Das Array der versteckten Input-Felder, die mit dem obligatorischen Submit-Button verbunden wird. Sie können mit der Funktion hiddenFields() übergeben werden.
  		* @param string $startForm2 Die Formular-Informationen zum optionalen Submit-Button. Sie sollten mit der Funktion startFormular() übergeben werden. Achtung: $startForm2 darf keine Angaben zum Einzug enthalten.
  		* @param string $hiddenFieldsForm2 Das Array mit den versteckten Input-Felder zum optionalen Submit-Button. Sie können mit der Funktion hiddenFields() übergeben werden.
  		* @param string $submitButtonForm2 Die Informationen zur Zurückgabe des optionalen Submit-Buttons. Sie sollten mit der Funktion submitButton() übergeben werden.
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/
		public function goBack($info,$hiddenFieldsForm1,$startForm2,$hiddenFieldsForm2,$submitButtonForm2,$indent){
			// 
			// Rückgabe des Informationstextes
			$result =   $this->linebreak($indent) . "<p>". 
			$this->linebreak($indent+1) . $info . 
			$this->linebreak($indent) . "</p>";
			
			// Rückgabe des obligatorischen Submit-Buttons 
			$result .= $this->startFormular("display: inline;","","goBack","","POST","","",$indent) . 
			$this->hiddenFields($hiddenFieldsForm1,$indent+1) .
			$this->endFormular($indent);
			// Einblendung des optionalen Submit-Buttons			
			if($startForm2 != ""){
				$result .=  $startForm2 . 
					$this->hiddenFields($hiddenFieldsForm2,$indent+1) .
					$this->endFormular($indent);
			}
			if (empty($submitButtonForm2)){
				$result .=  $this->submitButtonArea(
					NULL,
					$this->submitButton("goBack","submitButton","Submit","submit","","zurück"),
					$indent);
			} else {
				$result .=  $this->submitButtonArea(
					$this->submitButton("goBack","submitButton","Submit","submit","","zurück"),
					$submitButtonForm2,
					$indent);
			}
			return $result;
		}


		/**
  		* leitet auf eine andere Seite weiter
  		* @param string $URL Die Seite, auf die weitergeleitet wird
		* @return void
 		*/	
		public function goToPage($URL){
			?>
			<script type="text/javascript">
				window.location.replace('<?php echo $URL; ?>');
			</script>
			<?php 
		}	
		
		/**
  		* gibt den einleitenden HTML-Befehl für ein Formular zurück.
  		* Dieser Funktion sollte mit der Funktion endFormular($indent) abgeschlossen werden.
  		* @param string $style Der CSS-Stil des Formulars. Wird im Formular nur zurückgegeben, wenn $style definiert ist.
  		* @param string $action Die Adresse, an den die Daten nach dem Klick auf den Submit-Button gesendet werden. Wird im Formular nur zurückgegeben, wenn $action definiert ist. 
  		* @param string $ID Die ID des Formulars, um das Formular mit dem Submit-Button eindeutig ansprechen zu können. Wird im Formular nur zurückgegeben, wenn $ID definiert ist.
  		* @param string $enctype Die Art der Datenkodierung. Falls leer, wird "multipart/form-data" zurückgegeben, um auch Dateien senden zu können.
  		* @param string $method Die HTTP-Methode. Falls leer, wird "POST" zurückgegeben.
  		* @param string $name Gibt den Namen des Formulars zurück. Wird im Formular nur zurückgegeben, wenn $name definiert ist.
  		* @param string $target Gibt das Zielfenster zurück. Wird im Formular nur zurückgegeben, wenn $target definiert ist. Mögliche Werte sind "_blank", "_self", "_parent", "_top" sowie die Framebezeichnung.
  		* @param int $indent Die Einzugtiefe
		* @return string
 		*/			
		public function startFormular($style,$action,$ID,$enctype,$method,$name,$target,$indent){

			if ($style != ""){
				$style = ' style="' . $style . '"';
			}
			if ($action != ""){
				$action = ' action="' . $action . '"';
			}
			if ($ID != ""){
				$ID = ' id="' . $ID . '"';
			}
			if ($enctype != ""){
				$enctype = ' enctype="' . $enctype . '"';
			} else {
				$enctype = ' enctype="multipart/form-data"';
			}	
			if ($method != ""){
				$method = ' method="' . $method . '"';
			} else {
				$method = ' method="POST"';
			}
			if ($name != ""){
				$name = ' name="' . $name . '"';
			}
			if ($target != ""){
				$target = ' target="' . $target . '"';
			}			
						
			return $this->linebreak($indent) . '<form' . $style . $action . $ID . $method . $name . $target . $enctype . '>';
		}
		
		/**
  		* gibt die in dem Array gespeicherten Informationen als hidden input fields zurück.
  		* @param array $array Das Array.
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/	
		public function hiddenFields($array,$indent){
			$result = "";
			// gibt ein verstecktes Input-Feld aus, das in ein Formular eingebettet werden kann
			if (!empty($array )){
				foreach($array as $key => $value) {
					if (is_array($array[$key]) OR is_object($array[$key])){
						$result .= $this->linebreak($indent) . '<input type="hidden" name="' . $key. '" value=' . json_encode($array[$key]) . '>';
					} else {
						$result .= $this->linebreak($indent) . '<input type="hidden" name="' . $key. '" value="' . $array[$key] . '">';
					}
				}
  				return $result;
			}
		}
		
		/**
  		* gibt den HTML-Code zum Schließen eines HTML-Formulares zurück.
  		* Dieser Funktion sollte die Funktion startFormular($style,$action,$ID,$enctype,$method,$name,$target,$indent) vorausgehen
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/	
		public function endFormular($indent){
			return $this->linebreak($indent) . '</form>';
		}
		
		
		/**
  		* gibt den HTML-Code zur Ausgabe eines oder zweier Submit-Buttons zurück.
  		* @param string $leftButton Der HTML-Code für den linken Button. Er sollte über die Funktion submitButton erstellt werden.
  		* @param string $rightButton Der HTML-Code für den rechten Button. Er sollte über die Funktion submitButton erstellt werden.
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/	
		protected function submitButtonArea($leftButton,$rightButton,$indent){
			if ($leftButton != ""){
				$leftButton = $this->linebreak($indent + 1) . $leftButton;
			}
			if ($rightButton != ""){
				$rightButton = $this->linebreak($indent + 1) . "<span class = \"floatRight\">" . 
				$this->linebreak($indent + 2) . $rightButton . 
				$this->linebreak($indent + 1) . "</span>";
			}
			return $this->linebreak($indent) . '<div class ="submitButtonArea">' . 
			$leftButton . 
			$rightButton . 
			$this->linebreak($indent) . '</div>' .
			$this->linebreak($indent) . '<div class="cleardiv"></div>';
;
		}
		
		/**
  		* gibt den Code für einen Submit-Button zurück.
  		* @param string $form Das Formularelement, dem der Submit-Button zugeordnet werden soll. Wird nur zurückgegeben, wenn $form definiert ist.
  		* @param string $class Die CSS-Klasse des Submit-Buttons. Wird nur zurückgegeben, wenn $class definiert ist.
  		* @param string $name Der Name des Submit-Buttons. Wird nur zurückgegeben, wenn $name definiert ist.
  		* @param string $type Der Typ des Submit-Buttons. Wird nur zurückgegeben, wenn $type definiert ist.
  		* @param string $value Der initiale Wert, den der Submit-Button übermittelt. Wird nur zurückgegeben, wenn $value definiert ist.
  		* @param string $text Der Text, den der Button trägt.
		* @return string Der Code für den Submit-Button.
 		*/	
		protected function submitButton($form,$class,$name,$type,$value,$text){
			if ($form != ""){
				$form = " form=\"" . $form . "\"";
			}
			if ($class != ""){
				$class = " class=\"" . $class . "\"";
			}
			
			if ($name != ""){
				$name = " name=\"" . $name . "\"";
			}			
			if ($type != ""){
				$type = " type=\"" . $type . "\"";
			}
			if ($value != ""){
				$value = " value=\"" . $value . "\"";
			}
			$result = "<button type=\"submit\"" . $form . $class . $name . $type . $value . ">" . $text . "</button>";
			return $result;
		}
		
		/**
		* Gibt JavaScript-Code zurück, der das automatische Submit eines Formulars ermöglicht.
		* @param string $ID Die ID des Formulars.
		* @return string Der Quelltext. 
		*/
		public function submitFormular($ID){
		?>
			<script type="text/javascript">
    document.getElementById('<?php echo $ID; ?>').submit();
</script>

			<?php 
		}
		
		/**
  		* gibt den Dateinamen für einen bestimmten Icontyp zurück.
  		* @param string $iconTyp Der Icontyp. Mögliche Werte sind home,edit, addToTrolley, deleteTrolley, check, delete, sort_up und sort_down.
  		* @return string Der Dateiname.
  		*/
		public function getIcon($iconTyp){
			switch ($iconTyp) {
				case "home":
					return URL . "_icons/baseline-home-24px.svg";    			
    			case "edit":
        			return URL . "_icons/baseline-edit-24px.svg";
        		case "addToTrolley":
        			return URL . "_icons/baseline-add_shopping_cart-24px.svg";
        		case "deleteTrolley":
        			return URL . "_icons/baseline-remove_shopping_cart-24px.svg";
        		case "check":
        			return URL . "_icons/baseline-check-24px.svg";
    			case "delete":
        			return URL . "_icons/baseline-delete-24px.svg";
        		case "sort_up":
        			return URL . "_icons/baseline-arrow_upward-24px.svg";
        		case "sort_down":
        			return URL . "_icons/baseline-arrow_downward-24px.svg";
        		case "pageUp";
        			return URL . "_icons/baseline-arrow_upward-24px.svg";
        		case "about";
        			return URL . "_icons/baseline-info-24px.svg";
        		case "search";
        			return URL . "_icons/baseline-search-24px.svg";
        		case "help";
        			return URL . "_icons/baseline-help-24px.svg";
			}
		}
		
		
		/**
		* gibt ein Input-Feld zur Auswahl des Artikelnamens zurück. 
		* @param object $obj Ein Objekt vom Typ Article oder ListEntry.
		* @param int $indent Der Einzug. 
		* @return string Der Quellcode.
		*/
		protected function selectArticleName($obj,$indent){	
			if ($obj == ""){
				$obj = new ListElement(NULL,NULL);
			}
			if ($obj->getPrefix() == "listEntry"){
				$editIcon = $this->linebreak($indent+1) . '<span style="float:right;">' .
				$this->linebreak($indent+2). '<a href="editArticle.php?ID='. $obj->getArticleID(). '&catID=0">' .
				$this->linebreak($indent+3). '<img src="' . $this->getIcon("edit"). '" alt="edit" class="editicon right">' .
				$this->linebreak($indent+2). '</a>' . 
				$this->linebreak($indent+1) . '</span>';
			} else {
				$editIcon = "";
			}
			if ($obj->getName() == ""){
				$nameField = $this->linebreak($indent+2) . '<input type="text" id="articleName" name="name" class="typeahead tt-query" autocomplete="off" spellcheck="false" placeholder="Artikelbezeichnung">';
			} else {
				$nameField = $obj->getArticleNameAndArticleAnnotation() . $editIcon;
			}
			return $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . 'Artikel' . 
			$this->linebreak($indent+1) . '</legend>' . 
			$nameField . 
			$this->linebreak($indent) . '</fieldset>';
		}
		
		
		/**
  		* gibt die Beschreibung für einen bestimmten Icontyp zurück.
  		* @param string $iconTyp Der Icontyp. Mögliche Werte sind edit, addToTrolley, deleteTrolley, check, delete, sort_up und sort_down.
  		* @return string Die Beschreibung.
  		*/
		protected function getIconDescription($iconTyp){
			switch ($iconTyp) {
				case "home":
					return "Home";
				case "edit":
        			return "Bearbeiten";
        		case "addToTrolley":
        			return "Neuer Artikel";
        		case "deleteTrolley":
        			return "Warenkorb löschen";
        		case "check":
        			return "absenden";
        		case "delete":
        			return "löschen";
        		case "sort_up":
        			return "aufwärts";
        		case "sort_down":
        			return "abwärts";
			}
		}
		
		/**
  		* gibt den HTML-Code für ein Formular-Icon zurück.
  		* @param string $targetPage  Die Adresse, an den die Daten nach dem Klick auf den Submit-Button gesendet werden. 
  		* @param array $arrayHiddenFields  Das Array mit den Informationen für die versteckten Input-Fields. 
  		* @param string $iconTyp Der Icontyp. Mögliche Werte sind edit, addToTrolley, deleteTrolley, check, delete, sort_up und sort_down.
  		* @param string $iconClass Die CSS-Klasse des Icons.
  		* @param int $indent Die Einzugtiefe.
  		* @return string Der HTML-Code für das Formular-Code.
  		*/
		protected function printFormIcon($targetPage,$arrayHiddenFields,$iconTyp,$iconClass,$indent){
			return $this->startFormular("display: inline;",$targetPage,"","","POST","","",$indent) . 			
			$this->hiddenFields($arrayHiddenFields,$indent + 1) . 
			$this->linebreak($indent+1) . "<input type=\"image\" src=\"" . $this->getIcon($iconTyp). "\" class=\"" . $iconClass . "\" alt=\"" . $this->getIconDescription($iconTyp). "\">" . 
			$this->endFormular($indent);
		}

		
		/**
  		* Gibt einen String zurück, der prüft, ob eine Löschung tatsächlich durchgeführt werden soll, und stellt zwei Buttons bereit, um die Löschung abzubrechen bzw. durchzuführen.
  		* Ein Klick auf den Button "Abbrechen" leitet an das Formular "dontDelete" weiter, ein Klick auf den Button "Löschen" an das Formular "deleteChecked".
  		* @param string $question Die Frage, ob gelöscht werden soll.
  		* @param string $attention Ein zusätzlicher, optionaler Warnhinweis.
  		* @param string $formular1 Die Bezeichnung des Formulars, an die das Formular zur im Fall der bestätigten Löschung gesendet werden soll. Falls leer, wird "deleteChecked" gesetzt.
  		* @param string $formular2 Die Bezeichnung des Formulars, an die das Formular zur im Fall der abgelehnten Löschung gesendet werden soll. Falls leer, wird "dontDelete" gesetzt.
  		* @param array $arrayHiddenFields1 Daten in Form eines assoziativen Arrays, die im Falle des Klicks auf Formular1 weitergeleitet werden sollen.
  		* @param array $arrayHiddenFields2 Daten in Form eines assoziativen Arrays, die im Falle des Klicks auf Formular2 weitergeleitet werden sollen.
  		* @param int $indent Die Einzugtiefe.
		* @return string Der Rückgabetext.
 		*/	
		public function checkBeforeDeletion($question,$attention,$formular1,$formular2,$arrayHiddenFields1,$arrayHiddenFields2,$indent){
			if ($attention != ""){
				$attention = $this->linebreak($indent) . '<p>Achtung: ' . $attention . '</p>';
			};
			if (empty($formular1)){
				$formular1 = "deleteChecked";
			};
			if (empty($formular2)){
				$formular2 = "dontDelete";
			};
			if (empty($arrayHiddenFields1)){
				$arrayHiddenFields1 = array();
			};
			if (empty($arrayHiddenFields2)){
				$arrayHiddenFields2 = array();
			};
			return $this->linebreak($indent) .  '<p>' . $question . '</p>' .
			$attention .
			$this->startFormular(NULL,NULL,$formular1,NULL,NULL,NULL,NULL,$indent+2) . 
			$this->hiddenFields(array("formular" => $formular1)+$arrayHiddenFields1,$indent+2) . 
			$this->endFormular($indent+2) .
			$this->startFormular(NULL,NULL,$formular2,NULL,NULL,NULL,NULL,$indent+2) .
			$this->hiddenFields(array("formular" => $formular2)+$arrayHiddenFields2,$indent+2) . 
			$this->endFormular($indent+2) . 
			$this->submitButtonArea($this->submitButton($formular2,"submitButton","","","","abbrechen"),$this->submitButton($formular1,"submitButton","","","","löschen"),$indent);
		}
		
		/**
  		* gibt eine Liste von Elementen mit Sortierpfeilen aus
  		* @param string $title Der Titel (h2), der den Formular vorangestellt wird. 
  		* @param string $anchor Die Sprungmarke zur Überschrift. Ist sie angegeben, kann mit einem Link direkt auf die entsprechende Überschrift gesprungen werden.
  		* @param array $items Die Elemente als Array
  		* @param string $editPage Die Seite, auf die verlinkt werden soll. 
  		* @param string $hrefAddon Ein optionaler Zusatz zu der verlinkten Adresse. 
  		* @param string $sortAddon Optionaler String, der den Angaben zum verlinkten Formular vorangestellt wird. 
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/
		public function sortItems($title,$anchor,$items,$editPage,$hrefAddon,$sortAddon,$indent){
			if (isset($items) && count($items) >= 1){ // nur, wenn schon Subkats vorhanden sind
				if (!empty($sortAddon)){
					$sortAddon .= "_";
				}
				if (empty($anchor)){
					$h2ID = "";
				} else {
					$h2ID = ' id="' . $anchor . '"';
				}
				$result = $this->linebreak($indent) . '<h2' . $h2ID . '>' .
				$this->linebreak($indent+1) . $title . 
				$this->linebreak($indent) . '</h2>' . 
				$this->linebreak($indent) . '<ul>';
				foreach ($items AS $item){
					$sort_up = $this->printFormIcon(NULL,
					array($item->getPrefix() => $this->objectToString($item), "formular" => $sortAddon . "sort_up"),
					"sort_up","icon sorticon",$indent+3);	
					$sort_down = $this->printFormIcon("",
					array($item->getPrefix() => $this->objectToString($item), "formular" => $sortAddon . "sort_down"),"sort_down","icon sorticon",$indent+3);
				
					// Prüfen, ob beide Pfeile ausgegeben werden sollen
					if ($item->getSort() == 1){ // kein Up-Pfeil bei erstem Eintrag
						$sort_up = "";
					}
					if ($item->getSort() == count($items)){ // kein Downpfeil beim letzten Eintrag
						$sort_down = "";	
					}

					if ($item->getAnnotation() != ""){
						$nameAndAnnotation = $item->getName() . " (" . $item->getAnnotation() . ")";
					} else {
						$nameAndAnnotation = $item->getName();
					}
					
					// Veränderung $result
					$result .= $this->linebreak($indent+1) . '<li>' .
					$this->linebreak($indent+2) . '<span class="sorticons">' . 
					$sort_up . 
					$sort_down . 
					$this->linebreak($indent+2) . '</span>' . 
					$this->linebreak($indent+2) . '<a href="' . $editPage. '?ID=' . $item->getID() . $hrefAddon . ' ">' . 
					$this->linebreak($indent+3) . $nameAndAnnotation .
					$this->linebreak($indent+2) . '</a>' . 
					$this->linebreak($indent+1) . '</li>';
				}
				$result .= $this->linebreak($indent) . '</ul>';
				return $result;
			}
		}
		
		 /**
  		* gibt eine Liste der Subkategorien mit Sortierpfeilen aus
  		* @param string $title Der Titel (h2), der den Formular vorangestellt wird. 
  		* @param string $anchor Die Sprungmarke zur Überschrift. Ist sie angegeben, kann mit einem Link direkt auf die entsprechende Überschrift gesprungen werden.
  		* @param array $subkats Die Subkategorien als Array
  		* @param string $hrefAddon Ein optionaler Zusatz zu der verlinkten Adresse. 
  		* @param int $indent Die Einzugtiefe.
		* @return string
 		*/
		public function sortSubCats($title,$anchor,$subkats,$hrefAddon,$indent){
			return $this->sortItems($title,$anchor,$subkats,$this->editPageSubkat,$hrefAddon,NULL,$indent);
		}
		
		/**
		* liefert die Subkategorien eines Objektes in einer <option>-Liste aus.
		* Diese Funktion wird u.a. in der Funktion addSubCat verwendet.
		* @param array $newSubkat Die neue Subkategorie.
		* @param array $subkats Die bisherigen Subkategorien in einem Array.
		* @param int $indent Der Zeileneinzug.
		* @param string Der Quelltext.  
		*/
		protected function selectSubCats($newSubkat,$subkats,$indent){
			// Erstellung der Liste
			$list = "";
			if (isset($subkats)){
				foreach($subkats AS $subkat){
					$value = $subkat->getSort() +1;
					if($newSubkat->getSort() == $value){
						$selected = ' selected="selected"';
					} else {
						$selected = "";
					}
					if($subkat->getSort() != count($subkats)){ // wenn es nicht der letzte Eintrag ist
						$list .= $this->linebreak($indent+3) . '<option value="' . $value . '"' . $selected . ' >' . 
						$this->linebreak($indent+4) . 'hinter ' . $subkat->getName() .  
						$this->linebreak($indent+3) . '</option>';				
					} else { // wenn es der letzte Eintrag ist
						$list .= $this->linebreak($indent+3) .  '<option value="' . $value . '"' . $selected . ' >' . 
						$this->linebreak($indent+4) . 'am Ende der Liste' .  
						$this->linebreak($indent+3) . '</option>';	
					}
				}
			}
		
			$result = 		
			$this->linebreak($indent+1) . '<fieldset>' .
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Position'. 
			$this->linebreak($indent+2) . '</legend>' .
			$this->linebreak($indent+2) . '<select class="fullWidth" name="' . $newSubkat->getPrefix() . '_sort" size="1">' . 
			$this->linebreak($indent+3) . '<option value="1">' . 
			$this->linebreak($indent+4) . 'am Anfang der Liste' .
			$this->linebreak($indent+3) . '</option>' .
			$list .
			$this->linebreak($indent+2) . '</select>' . 
			$this->linebreak($indent+1) . '</fieldset>';
			return $result;
		}

		/**
		* gibt ein Formular für das Hinzufügen eines neuen Elementes zurück.
		* Es werden mindestens die Felder "Bezeichnung" und "Position" zurückgegeben. Ist für das entsprechende Objekt die Eigenschaft "tableAnnotation" definiert, wird zusätzlich ein Feld "Anmerkung" hinzugefügt.
		* @param string $title Die Überschrift über dem Formular (h2).
		* @param string $anchor Die Sprungmarke zur Überschrift. Ist sie angegeben, kann mit einem Link direkt auf die entsprechende Überschrift gesprungen werden.
		* @param array $items Die bisherigen Elemente als dreidimensionales Array.
		* @param object $newItem Die neue Subkategorie.
		* @param string $form Die ID des Formulars, um das Formular mit dem Submit-Button eindeutig ansprechen zu können.
		* @param int $indent Der Zeileneinzug.
		* @return void   
		*/
		public function addItem($title,$anchor,$items,$newItem,$form,$indent){
			if ($newItem->getTableAnnotation() != ""){
				$annotationStr = $this->linebreak($indent+1) . '<fieldset>' .
				$this->linebreak($indent+2) . '<legend>' . 
				$this->linebreak($indent+3) . 'Anmerkung'. 
				$this->linebreak($indent+2) . '</legend>' . 
				$this->linebreak($indent+2) . '<input type="text" class="fullWidth" name="' . $newItem->getPrefix() . '_annotation" value="' . $newItem->getAnnotation() . '">' . 
				$this->linebreak($indent+1) . '</fieldset>';
			} else {
				$annotationStr = "";
			}
			if (empty($anchor)){
				$h2ID = "";
			} else {
				$h2ID = ' id="' . $anchor . '"';
			}

			return $this->linebreak($indent) . '<h2' . $h2ID . '>' . $title . '</h2>' .
			$this->startFormular("","",$form,"","","","",$indent) . 
			$this->hiddenFields(array("formular" => $form),$indent + 2) .	
			$this->linebreak($indent+1) . '<fieldset>' .
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Bezeichnung'. 
			$this->linebreak($indent+2) . '</legend>' . 
			$this->linebreak($indent+2) . '<input type="text" class="fullWidth" name="' . $newItem->getPrefix() . '_name" value= "' . $newItem->getName() . '">' . 
			$this->linebreak($indent+1) . '</fieldset>' . 
			$annotationStr .
			$this->selectSubCats($newItem,$items,$indent) .
			$this->endFormular($indent) .
			$this->submitButtonArea("",$this->submitButton($form,"submitButton","","","","hinzufügen"),$indent);
		}

		
		/**
		* gibt ein Formular für das Hinzufügen einer neuen Subkategorie zurück.
		* Es werden mindestens die Felder "Bezeichnung" und "Position" zurückgegeben. Ist für das entsprechende Objekt die Eigenschaft "tableAnnotation" definiert, wird zusätzlich ein Feld "Anmerkung" hinzugefügt.
		* @param string $title Die Überschrift über dem Formular (h2).
		* @param string $anchor Die Sprungmarke zur Überschrift. Ist sie angegeben, kann mit einem Link direkt auf die entsprechende Überschrift gesprungen werden.
		* @param array $subkats Die bisherigen Subkategorien als dreidimensionales Array.
		* @param object $newSubkat Die neue Subkategorie.
		* @param int $indent Der Zeileneinzug.
		* @return void   
		*/
		public function addSubCat($title,$anchor,$subkats,$newSubkat,$indent){
			return $this->addItem($title,$anchor,$subkats,$newSubkat,"addSubCat",$indent);
		}		
		
		
		/**
		* liefert ein Formular, um die Bezeichnung eines Objektes zu ändern
		* Das Formular leitet anschließend per $_POST an das Formular "rename" weiter, das auf der einbindenden Seite vorhanden sein muss
		* @param string name Der bisherige Name des Objektes
		* @param int $indent Der Zeileneinzug.
		* @return void
		*/
		public function renameFormular($name,$indent){
			return $this->linebreak($indent) . '<h2>Bezeichnung ändern</h2>' .
			$this->startFormular(NULL,"","rename","","","","",$indent) . 
			$this->hiddenFields(array("formular" => "rename"),$indent) .	
			$this->linebreak($indent+1) . '<fieldset>' .
			$this->linebreak($indent+2) . '<legend>' . 
			$this->linebreak($indent+3) . 'Bezeichnung'. 
			$this->linebreak($indent+2) . '</legend>' .
		$this->linebreak($indent+2) . '<input type="text" class="fullWidth" name="newName" placeholder="' . $name . '">' . 
			$this->linebreak($indent+1) . '</fieldset>' . 
			$this->endFormular($indent) .
			$this->submitButtonArea("",$this->submitButton("rename","submitButton","","","","umbenennen"),$indent);
		}

		
		
		/**
		* liefert ein Formular mit dem Namen ($prefix_)amount, um die Menge des ListEntries zu bearbeiten.
		* @param int $amount Die Menge des ListEntries.
		* @param string $prefix Das Präfix, das dem Namen des Formularfeldes gegebenenfalls vorangestellt wird.
		* @param string $name Der Name des Formularfeldes.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		protected function printAmount($amount,$prefix,$name,$indent){
			if (empty($name)){
				$name = "amount";
			}
			if (!empty($prefix)){
				$name = $prefix . "_" . $name;
			}

			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="amount">' . 
			$this->linebreak($indent+2) . 'Menge: ' . 
			$this->linebreak($indent+1) . '</label>' . 
			$this->linebreak($indent+1) . '<input name="' . $name . '" id="amount" type="number" value="' . $amount . '" size="4">' .
			$this->linebreak($indent) . "</p>";
			return $result;
		}
		
		/**
		* liefert ein Formular mit dem Namen ($prefix_)measureFull, um das Maß des ListEntries zu bearbeiten.
		* @param int $measureID Die Maß-ID des ListEntries.
		* @param string $prefix Das Präfix, das dem Namen des Formularfeldes gegebenenfalls vorangestellt wird.
		* @param string $name Der Name des Formularfeldes.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		protected function printMeasure($measureID,$prefix,$name,$indent){
			if (empty($name)){
				$name = "measureSerialized";
			}
			if (!empty($prefix)){
				$name = $prefix . "_" . $name;
			}
			include(ABS_PATH . "dbconnect.php");
			$result = $this->linebreak($indent) . "<p>" . 
			$this->linebreak($indent+1) . '<label for="measure">' . 
			$this->linebreak($indent+2) . 'Maß: ' . 
			$this->linebreak($indent+1). '</label>' . 
			$this->linebreak($indent+1). '<select name="' . $name . '" id="measure" size="1">';
			$selectStr = '
				SELECT 
					ID,
					name 
				FROM 
					' . DB_TAB_PREFIX . 'measures';
			$query = mysqli_query($db, $selectStr);
			$result .= $this->linebreak($indent+2) . "<option value=\"\"></option>";
			while($measure = mysqli_fetch_object($query)){
				$result .= $this->linebreak($indent+2) . "<option value=\"" . $this->arrayToString(array("ID" => $measure->ID,"name" => $measure->name)) . "\"";
				if ($measure->ID == $measureID) {
					$result .= " selected = \"selected\"";
				}
				$result .= ">" . $measure->name . "</option>";
			}
			$result .= $this->linebreak($indent+1) . "</select>" .
			$this->linebreak($indent) . "</p>";; 
			return $result;
		}

		/**
		* liefert ein Formular mit dem Namen ($prefix_)annotation, um die Anmerkungen des Listeneintrages zu bearbeiten.
		* @param string $annotation Die zu bearbeitende Anmerkung.
		* @param string $legend Die Legende; falls leer, wird "Anmerkungen" gesetzt.
		* @param string $info Informationstext, der unter dem Eingabefeld ausgegeben werden kann.
		* @param string $prefix Das Präfix, das dem Namen des Formularfeldes gegebenenfalls vorangestellt wird.
		* @param string $name Der Name des Formularfeldes.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		protected function editAnnotation($annotation,$legend,$info,$prefix,$name,$indent){
			if ($annotation == ""){
				$annotation = "";
			}
			if ($info != ""){
				$info = '<p class="infotext">' . $info . '</p>';
			} 
			if ($legend == ""){
				$legend = "Anmerkungen";
			}
			if (empty($name)){
				$name = "annotation";
			}
			if (!empty($prefix)){
				$name = $prefix . "_" . $name;
			}
			$result = $this->linebreak($indent) . '<fieldset>' . 
			$this->linebreak($indent+1) . '<legend>' . 
			$this->linebreak($indent+2) . $legend . 
			$this->linebreak($indent+1) . '</legend>' . 
			$this->linebreak($indent+1) . '<input type="text" name="' . $name. '" class ="fullWidth" value="' . $annotation. '">' . $info .  
			$this->linebreak($indent) . '</fieldset>';
			return $result;
		}
		
		/**
		* liefert ein Formular mit dem Namen priorityFull, um die Priorität des ListEntries zu bearbeiten.
		* @param int $priorityID Die ID der Priorität des ListEntries.
		* @param int $preselectedPriorityID Die ID der Priorität, falls bei $priorityID nichts gesetzt ist; kann auch NULL sein.
		* @param string $prefix Das Präfix, das dem Namen des Formularfeldes gegebenenfalls vorangestellt wird.
		* @param string $name Der Name des Formularfeldes; "prioritySerialized" falls leer.
		* @param int $indent Die Einzugtiefe.
		* return string Der Quelltext.
		*/
		protected function selectPriority($priorityID,$preselectedPriorityID,$prefix,$name,$indent){
			if (empty($name)){
				$name = "prioritySerialized";
			}
			if (!empty($prefix)){
				$name = $prefix . "_" . $name;
			}
			
			// gibt in der Tabelle priority gespeicherte Werte als Select-Box aus
			include(ABS_PATH . "dbconnect.php");
			$counter = 1;
			$result = 
				$this->linebreak($indent) . '<fieldset>' .
				$this->linebreak($indent+1) . '<legend>Dringlichkeit</legend>';
			$selectStr = '
				SELECT 
					ID,
					name 
				FROM 
					' . DB_TAB_PREFIX . 'priority';
			$query = mysqli_query($db, $selectStr);
			if ($priorityID == "") { // wenn keine PriorityID als Argument übergeben wurde
				if (empty($this->priorityID)){ // wenn das Element keine PriorityID hat
					if (empty($preselectedPriorityID)){ // wenn keine $preselectedPriorityID als Parameter übergeben wurde
						$selectPriorityID = '
							SELECT 
								standardPriorityID 
							FROM 
								' . DB_TAB_PREFIX . 'prefs
							WHERE ID = 1';
						$queryPriorityID = mysqli_query($db, $selectPriorityID);
						$rowPriorityID = mysqli_fetch_object($queryPriorityID);
						$priorityID = $rowPriorityID->standardPriorityID; // wähle Standardpriorität aus den Voreinstellungen
					} else {
						$priorityID = $preselectedPriorityID; // voausgewählter Standardwert
					}
				} else {
					$priorityID = $this->priorityID;
				}
			}

			while($row = mysqli_fetch_object($query)){
				$result .= $this->linebreak($indent+1) . '<input type="radio" id="p' . $counter . '" name="' . $name. '" value="' . $this->arrayToString(array("ID" => $row->ID, "name" => $row->name)) . '"';
				if ($row->ID == $priorityID) {
					$result .= " checked = \"checked\"";
				}
				$result .= '><label for="p' . $counter . '">' . $row->name . '</label><br />';
				$counter++;
			}
			$result .=  $this->linebreak($indent) . "</fieldset>"; 	
			return $result;
		}

		/**
		* Liefert die Informationen zu den Icons, die mit dem Header ausgegeben werden sollen.
		* Standardmäßig werden der Link zur Index-Seite und, falls die Eigenschaft "editPage" gesetzt ist, ggf. das Bearbeitungs- oder das Lösch-Icon angezeigt. Das Bearbeitungsicon erscheint, wenn es sich nicht um eine Bearbeitungsseite handelt. Das Löschicon erscheint auf der Bearbeitungsseite, sofern die Eigenschaft "containsDeletableItems" true gesetzt ist.
		* @param array $leftIcons Angaben in Form eines Arrays für zusätzliche Icons, die neben dem Index-Icon auf der linken Seite angezeigt werden sollen. $leftIcons ist ein assoziatives Array mit den Schlüsseln "default" für eine Standard- und "edit" für eine Bearbeitungsseite, die für jedes anzuzeigendes Icon je ein assoziatives Array mit den Schlüsseln "targetPage", "arrayHiddenFields" und "iconTyp" enthalten müssen, um es mit der Funktion printFormIcon() bearbeiten zu können.  
		* @param array $rightIcons Angaben für zusätzliche Icons, die neben dem Lösch- oder Bearbeitungs-Icon auf der rechten Seite angezeigt werden sollen.  $rightIcons ist ein assoziatives Array mit den Schlüsseln "default" für eine Standard- und "edit" für eine Bearbeitungsseite, die für jedes anzuzeigendes Icon je ein assoziatives Array mit den Schlüsseln "targetPage", "arrayHiddenFields" und "iconTyp" enthalten müssen, um es mit der Funktion printFormIcon() bearbeiten zu können.  
		* @return array Array mit den Icon-Angaben
		*/

		protected function getHeaderIcons_wp($leftIcons,$rightIcons){
			$homeIcon = array(
					"targetPage" => URL . "index.php",
					"arrayHiddenFields" => "",
					"iconTyp" => "home",
			);
			if ($this->editPage != NULL){
				$editIcon = array(
					"targetPage" => URL . $this->editPage,
					"arrayHiddenFields" => "",
					"iconTyp" => "edit",
				);	
				$deleteIcon = array(
					"targetPage" => URL . $this->editPage,
					"arrayHiddenFields" => array(
						"formular" => "delete",
					),
					"iconTyp" => "delete",
				);		
			} else {
				$editIcon = NULL;
				$deleteIcon = NULL;
			}
			
			$result = array(
				"left" => array(),
				"right" => array(),
			);
				
			// Zuordnung der Icons			
			array_push($result['left'],$homeIcon);
			if ($this->mode != "edit"){ // wenn es sich nicht um eine Bearbeitungsseite handelt
				array_push($result['right'],$editIcon);
				if (!empty($leftIcons['default'])){
					foreach($leftIcons['default'] AS $leftIcon){
						array_push($result['left'],$leftIcon);
					}
				}
				if (!empty($rightIcons['default'])){
					foreach($rightIcons['default'] AS $rightIcon){
						array_push($result['right'],$rightIcon);
					}
				}
			}
			else { // wenn es sich um eine Bearbeitungsseite handelt
				if($this->containsDeletableItems){
					array_push($result['right'],$deleteIcon);
				}
				if (isset($leftIcons['edit'])){
					foreach($leftIcons['edit'] AS $leftIcon){
						array_push($result['left'],$leftIcon);
					}	
				}
				
				if (isset($rightIcons['edit'])){
					foreach($rightIcons['edit'] AS $rightIcon){
						array_push($result['right'],$rightIcon);
					}
				}
			}
			return $result;
		}
		
		/**
		* Führt die automatische Einfügung von Artikeln in Listen sowie die automatische Markierung als "wichtig" durch.
		* @return void.
		*/
		public function autoUpdateAndInsertArticles(){
			$array = $this->selectArticlesForAutoUpdateAndInsertion();
			foreach ($array['addToList'] AS $listEntry){ // füge Listeneinträge hinzu
				$listEntry->addToDB();
			}
			$this->setToUrgent($array['setToUrgent']);
		}
		
		/**
		* ermittelt aus einem Array von Wochentagsangaben die Zahl der Tage vom übergebenen Datum bis zum nächsten im Array enthaltenen Wochentag und gibt diese Zahl zurück.
		/* Falls die Anzahl der Tage "0" ist, wird "7" zurückgegeben.
		* @param string $datum Das übergebene Datum im Format YYYY-MM-TT, alternativ "now" für das aktuelle Datum.
		* @param array $array Das Array aus Wochentagsangaben.
		* @return int Die Zahl an Tagen bis zum nächsten Wochentag, wobei die Rückgabe von 0 ausgeschlossen ist.   
		*/
		public function selectShortestNumberOfDaysToNextBuy($datum,$array){
			if (!empty($array)){ // wenn $array Daten enthält
				$numberOfDaysToNextBuy = array(); 
				foreach($array AS $weekday){
					if($weekday == date("D", strtotime($datum))){ // wenn der Wochentag in $datum der aktuelle $weekday ist 
						array_push($numberOfDaysToNextBuy,7); // nextBuy in 7 Tagen
					} else {
						$i = 1;
						while($weekday != date("D", strtotime($datum . " + " . $i . "days"))){ // erhöhe $i, bis $weekday erreicht ist
						$i++;
						}
						array_push($numberOfDaysToNextBuy,$i);
					}
				}
			return min($numberOfDaysToNextBuy); // gibt den kleinsten Wert aus dem Array zurück.
			} 
		}
		
		/**
		* Markiert sämtliche Listeneinträge mit den entsprechenden ArticleIDs als "wichtig".
		* @param array $articleIDs Ein Array aus den IDs der Artikel.
		* @return void.
		*/
		private function setToUrgent($articleIDs){
			include(ABS_PATH . "dbconnect.php");
			$updateStr = '
				UPDATE
					' . DB_TAB_PREFIX . 'listEntries
				SET
					' . DB_TAB_PREFIX . 'listEntries.priorityID = 3
				WHERE
					articleID 
						IN (' . implode(",",$articleIDs) . ')';
			$query = mysqli_query($db, $updateStr);
		}	
		
		/**
	 	* Gibt ein Array mit den automatisch zu generierenden und den auf dringend zu setzenden Listeinträgen zurück.
	 	* @return array Ein Array, das im Schlüssel "addToList" die IDs der auf die Liste zu setzenden Artikel und im Schlüssel "setToUrgent" die IDs der als dringend zu markierenden Artikel enthält.
 		*/
		private function selectArticlesForAutoUpdateAndInsertion(){
			include(ABS_PATH . "dbconnect.php");
			$selectStr = '
				SELECT 
					' . DB_TAB_PREFIX . 'articles.ID,
					' . DB_TAB_PREFIX . 'articles.name,
					' . DB_TAB_PREFIX . 'articles.standardPriorityID,
					' . DB_TAB_PREFIX . 'articlesAnnotation.annotation AS articleAnnotation,
					' . DB_TAB_PREFIX . 'articles.lastBuy,
					' . DB_TAB_PREFIX . 'articles.nextBuy,
					' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore.addToListXDaysBefore,
					' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore.setToUrgentXDaysBefore,
					' . DB_TAB_PREFIX . 'articlesStandards.standardAmount,
					' . DB_TAB_PREFIX . 'articlesStandards.standardMeasureID,
					' . DB_TAB_PREFIX . 'articlesStandards.standardListEntryAnnotation,
					' . DB_TAB_PREFIX . 'listEntries.ID AS listEntryID,
					' . DB_TAB_PREFIX . 'listEntries.priorityID
				FROM 
					' . DB_TAB_PREFIX . 'articles 				
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAnnotation
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAnnotation.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesAddToListXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesSetToUrgentXDaysBefore.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'articlesStandards
						ON
							' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'articlesStandards.articleID
				LEFT JOIN
					' . DB_TAB_PREFIX . 'listEntries
					ON 
						' . DB_TAB_PREFIX . 'articles.ID = ' . DB_TAB_PREFIX . 'listEntries.articleID				
				WHERE 
					nextBuy 
						IS NOT NULL';
			$today = date("Y-m-d", strtotime("now")); // aktuelles Datum ermitteln
			$query = mysqli_query($db, $selectStr);
			$result = array(
				"addToList" => array(),
				"setToUrgent" => array()
			);
			while($row = mysqli_fetch_object($query)){
				$lastBuy = date("Y-m-d", strtotime($row->lastBuy));
				$nextBuy = date("Y-m-d", strtotime($row->nextBuy));
				if ($row->addToListXDaysBefore == NULL){
					$addToListXDaysBefore = 0;
				} else {
					$addToListXDaysBefore = $row->addToListXDaysBefore;
				}
				$addToListOn = date("Y-m-d", strtotime($nextBuy . "-" . $addToListXDaysBefore . " days")); 
				if ($row->setToUrgentXDaysBefore == NULL){
					$setToUrgentXDaysBefore = 0;
				} else {
					$setToUrgentXDaysBefore = $row->setToUrgentXDaysBefore;
				}
				 $setToUrgentOn = date("Y-m-d", strtotime($row->nextBuy . "- " . $setToUrgentXDaysBefore ." days")); 

				if ($row->addToListXDaysBefore != NULL && $today >= $addToListOn && $lastBuy < $addToListOn && $row->listEntryID == NULL){ // wenn Datum erreicht ist, den Artikel auf die Liste zu setzen, der letzte Kauf vor dem Datum erfolgte, an dem der Artikel auf die Liste gesetzt werden sollte, und wenn für den Artikel noch kein Listeneintrag besteht. $lastBuy < $addToListOn ist notwendig, damit automatisch erzeugter Listeneintrag überhaupt gelöscht werden kann (wird sonst immer wieder automatisch auf die Liste gesetzt).
					$listEntry = new ListEntry(
						NULL, // ID
						Main::getPrefAmountIfNULL($row->standardAmount),
						Main::getPrefStandardMeasureIDIfNULL($row->standardMeasureID),
						NULL, // $measureName	
						$row->ID, // $articleID
						$row->name, // $name
						$row->articleAnnotation,
						$row->standardListEntryAnnotation,
						NULL, // $listID
						Main::getPrefStandardMeasureIDIfNULL($row->standardPriorityID), // $priorityID,
						NULL, // $priorityName,
						FALSE // $inBasket,
					);
					array_push($result['addToList'], $listEntry); // wenn nicht auf der Liste
				}
				
				if ($today >= $setToUrgentOn && $row->priorityID != "3"){ // wenn das Datum erreicht ist, den Listeneintrag auf urgent zu setzen und der Listeneintrag noch nicht auf urgent steht
					array_push($result['setToUrgent'], $row->ID);
				}
			};
			return $result;
		}
		
		/**
		* prüft, ob ein Update vorliegt, und gibt gegebenenfalls einen Hinweis zurück.
		* @param object $thisBuildDate Das Build-Datum der verwendeten Version als Date-Objekt.
		* @return string Der Warnhinweis.
		*/
		
		public function checkForUpdate($thisBuildDate){
			if (! dontCheckForUpdates){
				$actualBuildDate = date(strtotime(implode('', file("https://www.einkaufsliste.net/checkUpdate.php"))));
    		
    			if ($thisBuildDate < $actualBuildDate){
 					return '<p class="warning">Für Ihre Einkaufsliste liegt ein <a href="https://www.einkaufsliste.net">Update</a> vor. Sie können die ZIP-Datei direkt <a href="https://www.einkaufsliste.net/downloads/einkaufsliste_latest.zip">von der Anbieterseite herunterladen</a>.</p>';
 				} 
			}
		}
		
	}
?>